/**
 * Created by dragank on 9/17/2014.
 */

var AutoCompleteController = (function (AutoCompleteController)
{
    var self = AutoCompleteController;
    self.view = "";
    self.viewListIds = [];
    self.data = {};
    self.requestTimeout = null;

    self.init = function (view) {
        self.data = {};
        self.view = view;
        self.viewListIds = [];
        self.requestTimeout = null;
    };

    self.addViewId = function (inputFieldId, listViewId, classObject, callback) {
        self.viewListIds.push({
            inputFieldId: '#' + inputFieldId,
            listViewId: '#' + listViewId,
            classObject: classObject,
            callbackOnChange: callback
        });
        self.bindElementControls();
    };

    self.bindElementControls = function () {
        self.unbindElementControls();
        for (var i = 0, max = self.viewListIds.length; i < max; i++) {
            $(document).on('keyup', self.view + ' ' + self.viewListIds[i].inputFieldId, self.autocomplete);
            $(document).on('click', self.view + ' .' + self.viewListIds[i].classObject, self.addObject);
        }
    };

    self.unbindElementControls = function () {
        for (var i = 0, max = self.viewListIds.length; i < max; i++) {
            $(document).off('keyup', self.view + ' ' + self.viewListIds[i].inputFieldId);
            $(document).off('click', self.view + ' .' + self.viewListIds[i].classObject);
        }
    };

    self.autocomplete = function () {
        var objectType = $(this).attr('data-objecttype');
        var inputId = "#" + $(this).attr('id');
        var text = $(this).val();
        var i, max, listViewId, pattern, request, classObject, html = "";

        for (i = 0, max = self.viewListIds.length; i < max; i++) {
            if (self.viewListIds[i].inputFieldId == inputId) {
                listViewId = self.viewListIds[i].listViewId;
                classObject = self.viewListIds[i]['classObject'];
            }
        }

        if (text.length >= 3) {
            showLoadingIndikator();
            if (objectType == 'owner' || objectType == 'responsible') {
                if (connection) {
                    if (self.requestTimeout) {
                        clearTimeout(self.findTime);
                    }
                    ClientAPI.getAppUsersAndGroups(text, 0, 5, function (err, data) {
                        if (!err) {
                            for (i = 0, max = data.length; i < max; i++) {
                                var title = data[i]['identityDescription'];
                                if(data[i].identityType == "Group"){
                                    title = data[i]['identityName'];
                                }
                                html += '<li data-id="' + data[i]["_id"].replace(/\s+/g, '&#32;') + '"  class="' + classObject + '" data-objecttype="' + objectType + '" data-listview=' + listViewId + ' data-inputid=' + inputId + '>';
                                html += title;
                                html += "</li>";
                            }
                        }
                        hideLoadingIndikator();
                        $(listViewId, self.view).empty().append(html).listview('refresh');
                    });
                } else {
                    db.selectByQuery('SELECT * FROM user where name LIKE ? LIMIT ?, ?', ['%' + text + '%', 0, 5], 'user', function (data) {
                        for (i = 0, max = data.length; i < max; i++) {
                            html += '<li data-id="' + data[i]["id"]['text'].replace(/\s+/g, '&#32;') + '"  class="' + classObject + '" data-objecttype="' + objectType + '" data-listview=' + listViewId + ' data-inputid=' + inputId + '>';
                            html += data[i]['name'];
                            html += "</li>";
                        }
                        hideLoadingIndikator();
                        $(listViewId, self.view).empty().append(html).listview('refresh');
                    });
                }
            } else if (objectType == 'contact') {
                if (connection) {
                    if (self.requestTimeout) {
                        clearTimeout(self.findTime);
                    }

                    pattern = new RegExp(text.replace(' ', '|'), "i");
                    request = {'identity_id': {$exists: false}};
                    request['$or'] = [{'objectType': "Person"}, {'objectType': "Company"}];
                    request['$or'] = [{'firstName': pattern}, {'lastName': pattern}, {'companyName': pattern}];
                    ClientAPI.findPaged(request, [['modificationDate', -1]], 1, 5, function (err, data) {
                        for (i = 0, max = data.length; i < max; i++) {
                            var displayText = "";
                            if (data[i].objectType == "Company") {
                                displayText = data[i]['companyName'];
                            } else if (data[i].objectType == "Person") {
                                displayText = data[i]['firstName'] + ' ' + data[i]['lastName'];
                            }
                            html += '<li data-id="' + data[i]["_id"].replace(/\s+/g, '&#32;') + '"  class="' + classObject + '" data-objecttype="' + objectType + '" data-listview=' + listViewId + ' data-inputid=' + inputId + '>';
                            html += displayText;
                            html += "</li>";
                        }
                        hideLoadingIndikator();
                        $(listViewId, self.view).empty().append(html).listview('refresh');
                    });
                } else {
                    //offline
                    var sqlQuery = "SELECT * FROM (SELECT * FROM (SELECT people.id, ([first-name] || [last-name]) AS name  FROM people WHERE identity_id LIKE ?) UNION SELECT id, [name] FROM company) WHERE name LIKE ? LIMIT ?,?";
                    var sqlQueryData = [];
                    sqlQueryData.push('%undefined%');
                    sqlQueryData.push('%' + text + '%');
                    sqlQueryData.push(0);
                    sqlQueryData.push(5);
                    db.selectByQuery(sqlQuery, sqlQueryData, 'contact', function (data) {
                        for (var i = 0, max = data.length; i < max; i++) {
                            html += '<li data-id="' + data[i]["id"]['text'].replace(/\s+/g, '&#32;') + '"  class="' + classObject + '" data-objecttype="' + objectType + '" data-listview=' + listViewId + ' data-inputid=' + inputId + '>';
                            html += data[i]["name"];
                            html += "</li>";
                        }
                        hideLoadingIndikator();
                        $(listViewId, self.view).empty().append(html).listview('refresh');

                    });
                }
            } else {
                if (connection) {
                    if (self.requestTimeout) {
                        clearTimeout(self.findTime);
                    }
                    pattern = new RegExp(text.trim().replace(' ', '|'), "i");
                    request = {objectType: objectType};

                    if (objectType.toLocaleLowerCase() == "Note".toLocaleLowerCase()) {
                        request['note'] = pattern;
                    } else if (objectType.toLocaleLowerCase() == "Task".toLocaleLowerCase()) {
                        request['taskName'] = pattern;
                    } else if (objectType.toLocaleLowerCase() == "Company".toLocaleLowerCase()) {
                        request['companyName'] = pattern;
                    } else if (objectType.toLocaleLowerCase() == "Person".toLocaleLowerCase()) {
                        request['$or'] = [{'firstName': pattern}, {'lastName': pattern}];
                        request['identity_id'] = {$exists: false};
                    } else if (objectType.toLocaleLowerCase() == "Project".toLocaleLowerCase()) {
                        request['projectName'] = pattern;
                    } else if (objectType.toLocaleLowerCase() == "Deal".toLocaleLowerCase()) {
                        request['dealName'] = pattern;
                    }

                    ClientAPI.findPaged(request, [['modificationDate', -1]], 1, 5, function (err, data) {
                        if (!err) {
                            for (i = 0, max = data.length; i < max; i++) {
                                var displayText = "";
                                if (objectType.toLocaleLowerCase() == "Note".toLocaleLowerCase()) {
                                    displayText = data[i]['note'];
                                } else if (objectType.toLocaleLowerCase() == "Task".toLocaleLowerCase()) {
                                    displayText = data[i]['taskName'];
                                } else if (objectType.toLocaleLowerCase() == "Company".toLocaleLowerCase()) {
                                    displayText = data[i]['companyName'];
                                } else if (objectType.toLocaleLowerCase() == "Person".toLocaleLowerCase()) {
                                    displayText = data[i]['firstName'] + ' ' + data[i]['lastName'];
                                } else if (objectType.toLocaleLowerCase() == "Project".toLocaleLowerCase()) {
                                    displayText = data[i]['projectName'];
                                } else if (objectType.toLocaleLowerCase() == "Deal".toLocaleLowerCase()) {
                                    displayText = data[i]['dealName'];
                                }
                                html += '<li data-id="' + data[i]["_id"].replace(/\s+/g, '&#32;') + '"  class="' + classObject + '" data-objecttype="' + objectType + '" data-listview=' + listViewId + ' data-inputid=' + inputId + '>';
                                html += displayText;
                                html += "</li>";
                            }
                        }
                        hideLoadingIndikator();
                        $(listViewId, self.view).empty().append(html).listview('refresh');
                    });
                } else {
                    var table = objectType;
                    var query = "";
                    var queryData = [];
                    if (objectType.toLocaleLowerCase() == "Note".toLocaleLowerCase()) {
                        table = 'note';
                        query = 'SELECT * FROM ' + table + ' where note LIKE ? LIMIT ?,?';
                        queryData.push('%' + text + '%');
                    } else if (objectType.toLocaleLowerCase() == "Task".toLocaleLowerCase()) {
                        table = 'task';
                        query = 'SELECT * FROM ' + table + ' where body LIKE ? LIMIT ?,?';
                        queryData.push('%' + text + '%');
                    } else if (objectType.toLocaleLowerCase() == "Company".toLocaleLowerCase()) {
                        table = 'company';
                        query = 'SELECT * FROM ' + table + ' where name LIKE ? LIMIT ?,?';
                        queryData.push('%' + text + '%');
                    } else if (objectType.toLocaleLowerCase() == "Person".toLocaleLowerCase()) {
                        table = 'people';
                        query = 'SELECT * FROM ' + table + ' where [identity_id] = ? AND [first-name] LIKE ? OR [last-name] LIKE ? LIMIT ?,?';
                        queryData.push('undefined');
                        queryData.push('%' + text + '%');
                        queryData.push('%' + text + '%');
                    } else if (objectType.toLocaleLowerCase() == "Project".toLocaleLowerCase()) {
                        table = 'kase';
                        query = 'SELECT * FROM ' + table + ' where name LIKE ? LIMIT ?,?';
                        queryData.push('%' + text + '%');
                    } else if (objectType.toLocaleLowerCase() == "Deal".toLocaleLowerCase()) {
                        table = 'deal';
                        query = 'SELECT * FROM ' + table + ' where name LIKE ? LIMIT ?,?';
                        queryData.push('%' + text + '%');
                    }

                    queryData.push(0);
                    queryData.push(5);

                    db.selectByQuery(query, queryData, table, function (data) {
                        for (i = 0, max = data.length; i < max; i++) {
                            html += '<li data-id="' + data[i]["id"]['text'].replace(/\s+/g, '&#32;') + '"  class="' + classObject + '" data-objecttype="' + objectType + '" data-listview=' + listViewId + ' data-inputid=' + inputId + '>';
                            if (objectType == "Note") {
                                html += data[i]['name'];
                            } else if (objectType == "Task") {
                                html += data[i]['body'];
                            } else if (objectType == "Company") {
                                html += data[i]['name'];
                            } else if (objectType == "Person") {
                                html += data[i]['first-name'] + ' ' + data[i]['last-name'];
                            } else if (objectType == "Project") {
                                html += data[i]['name'];
                            } else if (objectType == "Deal") {
                                html += data[i]['name'];
                            }
                            html += "</li>";
                        }
                        hideLoadingIndikator();
                        $(listViewId, self.view).empty().append(html).listview('refresh');
                    });
                }
            }
        } else {
            $(listViewId, self.view).empty().listview('refresh');
        }
    };

    self.addObject = function () {
        var index;
        var listViewId = $(this).data('listview');
        var inputId = $(this).data('inputid');
        var objectType = $(this).data('objecttype');
        var id = $(this).data('id');
        $(inputId, self.view).val($(this).text());
        self.data[objectType] = id;
        $(listViewId, self.view).empty().listview('refresh');

        //odabir odgovarajuce callaback funkcije
        for(var i=0; i<self.viewListIds.length; i++){
            if(inputId == self.viewListIds[i].inputFieldId){
                index = i;
                break;
            }
        }
        var callback = self.viewListIds[index].callbackOnChange;
        if(callback){
            callback({id: id, objectType: objectType});
        }

    };

    self.getObject = function (objectType) {
        if (self.data[objectType]) {
            return self.data[objectType];
        } else {
            return null;
        }
    };

    self.setObject = function (type, value) {
        self.data[type] = value;
    };

    return self;
}(AutoCompleteController || {}));
