var CalendarAttendees = (function (CalendarAttendees) {
    var self = CalendarAttendees;
    self.view = null;
    self.objectType = null;
    self.internalIds = [];
    self.externalIds = [];

    self.init = function (view) {
        self.view = view;
        $(document).off('keyup', view + " #external-autocompletion");
        $(document).off('keyup', view + " #internal-autocompletion");
        $(document).off('click', view + " .attendees-item");
        $(document).off('click', view + " .delete-attendee");
        $(document).on('keyup', view + " #external-autocompletion", self.autocomplete);
        $(document).on('keyup', view + " #internal-autocompletion", self.autocomplete);
        $(document).on('click', view + " .attendees-item", self.addObject);
        $(document).on('click', view + " .delete-attendee", self.deleteAttendee);
    };

    self.autocomplete = function () {
        var html = "";
        var text = $(this).val();
        var $contactList = null;
        self.objectType = $(this).attr('data-objectType');
        if (self.objectType == "externalAttendees") {
            $contactList = $(self.view + " #external-user-list");
        } else if (self.objectType == "internalAttendees") {
            $contactList = $(self.view + " #internal-user-list");
        }
        if (text.length >= 3) {
            if (connection) {
                if (self.objectType == "externalAttendees") {
                    var pattern = new RegExp("^" + text.replace(' ', '|'), "i");
                    var request = {'identity_id': {$exists: false}};
                    request['$or'] = [{'objectType': "Person"}, {'objectType': "Company"}];
                    request['$or'] = [{'firstName': pattern}, {'lastName': pattern}, {'companyName': pattern}];
                    ClientAPI.findPaged(request, [['modificationDate', -1]], 1, 5, function (err, data) {
                        if (!err) {
                            for (var i = 0; i < data.length; i++) {
                                var displayText = "";
                                if (data[i].objectType == "Company") {
                                    displayText = data[i]['companyName'];
                                } else if (data[i].objectType == "Person") {
                                    displayText = data[i]['firstName'] + ' ' + data[i]['lastName'];
                                }
                                html += '<li data-id="' + data[i]["_id"].replace(/\s+/g, '&#32;') + '" class="attendees-item">';
                                html += displayText;
                                html += "</li>";
                            }
                            if (html != "") {
                                $contactList.show();
                                $contactList.empty().append(html);
                                if ($contactList.hasClass('ui-listview')) {
                                    $contactList.listview('refresh');
                                } else {
                                    $contactList.trigger('create');
                                }
                            }
                        }
                    });
                } else if (self.objectType == "internalAttendees") {
                    ClientAPI.getAppUsersAndGroups(text, 0, 5, function (err, data) {
                        if (!err) {
                            for (var i = 0; i < data.length; i++) {
                                html += '<li data-id=' + data[i]["person"].replace(/\s+/g, '&#32;') + ' class="attendees-item">';
                                if (data[i].identityType == "User") {
                                    html += data[i]['firstName'] + " " + data[i]['lastName'];
                                } else {
                                    html += data[i]['identityName'];
                                }
                                html += "</li>";
                            }
                        }
                        $contactList.show();
                        $contactList.empty().append(html);

                        if ($contactList.hasClass('ui-listview')) {
                            $contactList.listview('refresh');
                        } else {
                            $contactList.trigger('create');
                        }
                    });
                }
            }
        } else {
            $contactList.hide();
        }
    };

    self.addObject = function () {
        var id = $(this).attr('data-id');
        var add = true;
        if (self.objectType == "externalAttendees") {
            for (var i = 0; i < self.externalIds.length; i++) {
                if (id == self.externalIds[i].id) {
                    add = false;
                    break;
                }
            }
            if (add) {
                self.externalIds.push({id: id, name: $(this).text()});
            }
            $(self.view + " #external-user-list").hide();
            $(self.view + " #external-autocompletion").val('');
        } else if (self.objectType == "internalAttendees") {
            for (var i = 0; i < self.internalIds.length; i++) {
                if (id == self.internalIds[i].id) {
                    add = false;
                    break;
                }
            }
            if (add) {
                self.internalIds.push({id: id, name: $(this).text()});
            }
            $(self.view + " #internal-user-list").hide();
            $(self.view + " #internal-autocompletion").val('');
        }

        self.drawAttendees();
    };

    self.drawAttendees = function () {
        var html = "";
        html += "<table>";
        if (self.objectType == "externalAttendees") {
            for (var i = 0; i < self.externalIds.length; i++) {
                html += "<tr>"
                        + "<td>" + self.externalIds[i].name + "</td>"
                        + "<td id='" + self.externalIds[i].id + "' data-deleteNumber='" + i +"' data-type='external' class='delete-attendee'><i class='md-delete md-2x'></i></td>"
                        + "</tr>";
            }
            html += "</table>";
            $(self.view + " #external-group-users").empty().append(html);
        } else if (self.objectType == "internalAttendees") {
            for (var i = 0; i < self.internalIds.length; i++) {
                html += "<tr>"
                        + "<td>" + self.internalIds[i].name + "</td>"
                        + "<td id='" + self.internalIds[i].id + "' data-deleteNumber='" + i + "' data-type='internal' class='delete-attendee'><i class='md-delete md-2x'></i></td>"
                        + "</tr>";
            }
            html += "</table>";
            $(self.view + " #internal-group-users").empty().append(html);
        }
    };
    
    self.deleteAttendee = function(){
        var type = $(this).attr('data-type');
        if(type == "external"){
            self.externalIds.splice($(this).attr('data-deleteNumber'), 1);
            self.objectType = "externalAttendees";
        } else if (type == "internal"){
            self.internalIds.splice($(this).attr('data-deleteNumber'), 1);
            self.objectType = "internalAttendees";
        }
        self.drawAttendees();
    };
    
    self.clear = function(){
        self.externalIds = [];
        self.internalIds = [];
    };

    return self;

}(CalendarAttendees || {}));