var CaseController = (function (CaseController)
{
    var self = CaseController;
    var table = models.kaseDBModel.name;
    var objectType = {objectType: "Project"};
    var countRequest;
    var changeParent;

    self.synchronization = function (items, callback) {
        if (items != null && items.length != null && items.length > 0) {
            self.encode(items, function (results) {
                var keys = [];
                for (var i = 0, max = results.length; i < max; i++) {
                    keys.push(results[i].id);
                }
                if (keys.length != 0) {
                    db.deleteFromTableByIds(table, 'id', keys, function () {
                        db.insertMultiple(table, [results], function () {
                            if (callback != null && typeof callback == "function") {
                                callback(results);
                            }
                        });
                    });
                } else {
                    if (callback != null && typeof callback == "function") {
                        callback([]);
                    }
                }
            });
        } else {
            if (callback != null && typeof callback == "function") {
                callback([]);
            }
        }
    };

    self.syncPartData = function (err, data, callback) {
        if (err == null) {
            if (data != null && data.length > 0) {
                var keys = [];
                var projects = [];

                for (var i = 0, max = data.length; i < max; i++) {
                    var project = self.encodeItem(data[i]);
                    keys.push(project.id);
                    projects.push(project);
                }

                if (keys.length != 0) {
                    db.deleteFromTableByIds(table, 'id', keys, function () {
                        db.insertMultiple(table, [projects], function () {
                            if (callback != null && typeof callback == "function") {
                                callback(projects);
                            }
                        });
                    });
                } else {
                    if (callback != null && typeof callback == "function") {
                        callback([]);
                    }
                }
            } else {
                if (callback != null && typeof callback == "function") {
                    callback();
                }
            }
        } else {
            if (callback != null && typeof callback == "function") {
                callback();
            }
        }
    };

    self.encode = function (items, callback) {
        var result = [], resultItem;
        if (items != null) {
            if (items.length != null && items.length > 0) {
                for (var i = 0, max = items.length; i < max; i++) {
                    resultItem = self.encodeItem(items[i]);
                    if (resultItem != null) {
                        result.push(resultItem);
                    }
                }
                if (callback != null && typeof callback == "function") {
                    callback(result);
                }
            } else {
                resultItem = self.encodeItem(items);
                if (resultItem != null) {
                    result.push(resultItem);
                }
                if (callback != null && typeof callback == "function") {
                    callback(result);
                }
            }
        }
    };

    self.encodeItem = function (item) {
        if (item['_id'] != null) {
            var result = {};
            var links = [];
            result['id'] = getTextTypeObject(item['_id'], 'integer');
            result['entityId'] = item['entityId'];
            result['name'] = item['projectName'].escape();
            result['author-id'] = getTextTypeObject(item['creator'], 'integer');
            result['created-at'] = getTextTypeObject(item['creationDate'], 'datetime');
            result['updated-at'] = getTextTypeObject(item['modificationDate'], 'datetime');
            result['due-at'] = getTextTypeObject(item['dueDate'], 'datetime');
            result['group-id'] = {"type": "integer", "nil": "true"};
            result['group'] = "";
            result['parties'] = {"type": "array"};

            //getting parent info
            if (item.immediateParent && item.immediateParentDisplay && item.immediateParentObjectType) {
                var parentObject = {};
                parentObject.id = item.immediateParent;
                parentObject.name = item.immediateParentDisplay;
                parentObject.type = item.immediateParentObjectType;
                var partyId = item.immediateParent;
                result['party-id'] = getTextTypeObject(partyId, 'integer');
                result['party'] = parentObject;
            } else {
                result['party-id'] = {"type": "integer", "nil": "true"};
                result['party'] = {"type": "integer", "nil": "true"};
            }

            result['searchKeywords'] = item['searchKeywords'];
            result['status'] = item['status'];

            if (item['assignedTo'] != null) {
                result['owner-id'] = getTextTypeObject(item['assignedTo'], 'integer');
            } else {
                result['owner-id'] = getTextTypeObject(item['owner'], 'integer');
            }

            result['visible-to'] = item.accessList;

            if (item['projectDescription'] != null) {
                result['background'] = item['projectDescription'];
            } else {
                result['background'] = "";
            }

            if (item['closedAt'] != null) {
                result['closed-at'] = getTextTypeObject(item['closedAt'], 'datetime');
            } else {
                result['closed-at'] = {"type": "datetime", "nil": "true"};
            }

            if (item.statusHistory) {
                result["statusHistory"] = item.statusHistory;
            }

            if (item['urlList'] && item['urlList'].length > 0) {
                if (item['urlList'].length == 1) {
                    links = {"id": getTextTypeObject(UtilController.randomNumber(), 'integer'),
                        "url": item.urlList[0].url,
                        "urlDesc": item.urlList[0].urlDescription,
                        "urlType": item.urlList[0].urlType
                    };
                } else {
                    for (var i = 0; i < item['urlList'].length; i++) {
                        links.push({
                            "id": getTextTypeObject(UtilController.randomNumber(), 'integer'),
                            "url": item.urlList[i].url,
                            "urlDesc": item.urlList[i].urlDescription,
                            "urlType": item.urlList[i].urlType
                        });
                    }
                }
            }
            result['link'] = links;

            return result;
        } else {
            return undefined;
        }
    };

    self.save = function (name, info, ownerId, visibleTo, root, immediate, dueDate, caseList, callback) {
        var project = null;
        if (typeof name == "object") {
            project = name;
        } else {
            project = objectType;
            if (name) {
                project['projectName'] = name.escape();
            } else {
                project['projectName'] = "";
            }

            if (info) {
                project['projectDescription'] = info;
            } else {
                project['projectDescription'] = "";
            }

            if (ownerId != null) {
                project['owner'] = ownerId.toString();
            }

            if (root != null) {
                project['rootObject'] = new ObjectId(root);
            }

            if (dueDate) {
                project['dueDate'] = dueDate;
            }

            project['urlList'] = caseList;
            project['accessList'] = visibleTo;
            project['status'] = "Created";
        }

        if (connection) {
            if (immediate) {
                ClientAPI.attachChildObject(project, immediate, function (err, res) {
                    var project = self.encodeItem(res.object);
                    db.insert(table, [project], function () {
                        if (res != null && callback != null && typeof callback == "function") {
                            callback(project);
                        }
                    });
                });
            } else {
                ClientAPI.insertObject(project, function (err, res) {
                    if (err) {
                        if (callback != null && typeof callback == "function") {
                            callback();
                        }
                    } else {
                        var project = self.encodeItem(res.object);
                        db.insert(table, [project], function () {
                        });
                        if (res != null && callback != null && typeof callback == "function") {
                            callback(project);
                        }
                    }
                });
            }
        } else {
            var objectProject = {'object': project, 'objectType': table, id: UtilController.randomNumber(), 'parent-id': immediate};

            db.insert(models.offlineInsertDBModel.name, [objectProject], function () {
                alert(Messages.Offline.offlineMode);
                if (info != null && typeof info == "function") {
                    info();
                } else if (callback != null && typeof callback == "function") {
                    callback();
                }
            });
        }
    };

    self.update = function (id, name, info, ownerId, visibleTo, root, immediate, dueDate, dueFrame, caseList, callback) {
        if (connection) {
            if (id) {
                self.findById(id, function (data) {
                    if (data != null) {
                        var caseObjectImmediateId = null;
                        if (data.immediateParent) {
                            caseObjectImmediateId = data.immediateParent;
                        }

                        data['projectName'] = name.escape();
                        data['projectDescription'] = info;

                        if (root != null) {
                            data['rootObject'] = new ObjectId();
                        }

                        if (immediate != null) {
                            data['immediateParent'] = new ObjectId();
                        }

                        if (dueDate != null) {
                            data['dueDate'] = dueDate;
                        }

                        if (dueFrame == "Later") {
                            data['dueDate'] = "";
                        }

                        data['urlList'] = caseList;

                        countRequest = 0;
                        changeParent = false;
                        ClientAPI.updateObject(data, function (err, data) {
                            if (data.statusCode != 0) {
                                alert(data.statusDesc);
                                if (callback != null && typeof callback == 'function') {
                                    callback();
                                }
                            } else {
                                if (visibleTo != null) {
                                    countRequest++;
                                    console.log("countRequest-visibleto #" + countRequest);
                                    ClientAPI.updateACL(data.object._id, visibleTo, function (err, item) {
                                        console.log(err||item);
                                        countRequest--;
                                        console.log("CountRequest #" + countRequest);
                                        if (!changeParent) {
                                            self.changeCaseParent(id, immediate, caseObjectImmediateId, callback);
                                        }
                                        if (countRequest == 0) {
                                            self.findById(id, function (data) {
                                                self.syncPartData(null, [data], function (data) {
                                                    if (callback != null && typeof callback == 'function') {
                                                        callback(data);
                                                    }
                                                });
                                            });
                                        }
                                    });
                                }
                                if (ownerId != data.object.owner) {
                                    countRequest++;
                                    console.log("countRequest-ownerchange #" + countRequest);
                                    debugger;
                                    ClientAPI.changeOwner(data.object._id, JSON.stringify(ownerId), function (err, data) {
                                        console.log(err||data);
                                        countRequest--;
                                        console.log("Count request #" + countRequest);
                                        console.log(countRequest + " - owner");
                                        if (!changeParent) {
                                            self.changeCaseParent(id, immediate, caseObjectImmediateId, callback);
                                        }
                                        if (countRequest == 0) {
                                            self.findById(id, function (data) {
                                                self.syncPartData(null, data, function (data) {
                                                    if (callback != null && typeof callback == 'function') {
                                                        callback(data);
                                                    }
                                                });

                                            });
                                        }
                                    });
                                }

                                if(!(visibleTo != null) && !(ownerId != data.object.owner)){
                                    self.changeCaseParent(id, immediate, caseObjectImmediateId, callback);
                                }
                                if (countRequest == 0 && callback != null && typeof callback == 'function') {
                                    callback([self.encodeItem(data.object)]);
                                }
                            }
                        });
                    } else {
                        if (callback != null && typeof callback == "function") {
                            callback();
                        }
                    }
                });
            } else {
                if (callback != null && typeof callback == "function") {
                    callback();
                }
            }
        } else {
            alert("You are in offline mode. You can't save this comment!");
        }
    };

    self.changeCaseParent = function (id, immediate, caseObjectImmediateId, callback) {
        countRequest++;
        changeParent = true;
        ClientAPI.find({_id: new ObjectId(id)}, null, function (err, data) {
            if (immediate != null && immediate != caseObjectImmediateId) {
                console.log("countRequest-immediateParent #" + countRequest);
                ClientAPI.attachChildObject(data[0], immediate, function (err, res) {
                    console.log(err || res);
                    countRequest--;
                    console.log("Count Request #" + countRequest);
                    if (countRequest == 0) {
                        self.findById(id, function (data) {
                            self.syncPartData(null, data, function (data) {
                                if (callback != null && typeof callback == 'function') {
                                    callback(data);
                                }
                            });
                        });
                    }
                });
            } else if (immediate == null) {
                console.log("countRequest-promote to root object #" + countRequest);
                ClientAPI.promoteToRootObject(data[0]['_id'], data[0].objectType, function () {
                    countRequest--;
                    console.log("Count Request #" + countRequest);
                    if (countRequest == 0) {
                        self.findById(id, function (data) {
                            self.syncPartData(null, data, function (data) {
                                if (callback != null && typeof callback == 'function') {
                                    callback(data);
                                }
                            });
                        });
                    }
                });
            } else {
                if (callback != null && typeof callback == 'function') {
                    callback(data);
                }
            }
        });
    };

    self.changeStatus = function (status, callback) {
        if (connection) {
            ClientAPI.executeTransitionalAction(status, 'Project', data.object._id, null, function (err, item) {
                if (!err) {
                    self.findById(data.object._id, function (data) {
                        self.syncPartData(null, [data], callback);
                    });
                } else {
                    console.log("CaseController - changeStatus: " + status);
                }
            });
        }
    };

    self.findAll = function (callback) {
        if (connection) {
            ClientAPI.find({objectType: "Project"}, [["modificationDate", -1]], function (err, data) {
                if (err == null) {
                    self.synchronization(data, function (items) {
                        if (callback != null && typeof callback == "function") {
                            callback(items);
                        }
                    });
                } else {
                    if (callback != null && typeof callback == "function") {
                        callback([]);
                    }
                }
            });
        } else {
            db.selectAll(table, function (data) {
                if (callback != null && typeof callback == "function") {
                    callback(data);
                }
            });
        }
    };

    self.findById = function (id, callback) {
        if (id == null) {
            if (callback != null && typeof callback == "function") {
                callback();
            }
        }
        if (connection) {
            ClientAPI.find({objectType: "Project", _id: new ObjectId(id)}, null, function (err, data) {
                if (err == null && callback != null && typeof callback == "function") {
                    ClientAPI.getFeed(
                            [
                                {$or:
                                            [
                                                {objectId: new ObjectId(data[0]['_id'])},
                                                {immediateParent: new ObjectId(data[0]['_id'])},
                                                {rootObject: new ObjectId(data[0]['_id'])}
                                            ]
                                },
                                {visible: false}
                            ],
                            [['eventDate', -1]], 1, 100,
                            function (err, feedItems) {
                                if (!err) {
                                    var feedElements = feedItems.filter(function (item) {
                                        if (item.objectType != 'Note' && item.objectType != 'Comment') {

                                            if (item.eventType == 'GrantedAccess') {
                                                item.grantedTo = item.info.split('; ');
                                                item.grantedTo = item.grantedTo.slice(0, item.grantedTo.length - 1);
                                            }
                                            return true;
                                        }
                                    });
                                    var project = self.encodeItem(data[0]);
                                    db.deleteObject(table, "id", getTextTypeObject(project.id.text, 'integer'), function () {
                                        db.insert(table, [project], function () {
                                            callback(data[0], feedElements);
                                        });
                                    });
                                } else {
                                    callback(self.encodeItem(data[0]));
                                }
                            }
                    );
                } else {
                    if (callback != null && typeof callback == "function") {
                        callback();
                    }
                }
            });
        } else {
            db.selectByColumn(table, "id", getTextTypeObject(id, 'integer'), function (data) {
                if (callback != null && typeof callback == "function") {
                    callback(data);
                }
            });
        }
    };

    self.findByObjectId = function (id, callback) {
        if (id == null) {
            if (callback != null && typeof callback == "function") {
                callback();
            }
        }

        if (connection) {
            ClientAPI.find({objectType: "Project", _id: new ObjectId(id)}, null, function (err, data) {
                if (err == null && typeof callback == "function") {
                    if (typeof callback == "function") {
                        callback(data[0]);
                    }
                } else {
                    if (typeof callback == "function") {
                        callback();
                    }
                }
            });
        } else {
            db.selectByColumn(table, "id", getTextTypeObject(id, 'integer'), function (data) {
                if (callback != null && typeof callback == "function") {
                    callback(data[0].kase);
                }
            });
        }
    };


    self.deleteCase = function (id, callback) {
        if (connection) {
            ClientAPI.removeObject(id, function (err, data) {
                if (data != null && data.statusCode != null) {
                    if (callback != null && typeof callback == "function") {
                        if (data.statusCode == 0) {
                            db.deleteObject(table, "id", getTextTypeObject(id, 'integer'), function () {
                                callback(data.statusCode);
                            });
                        } else {
                            callback();
                        }
                    }
                }
            });
        }
    };
    self.findAllFavoriteProject = function(callback) {
      FavoriteController.findByType("Case", function(data) {
          var favorites = [];
          for (var i = 0; i < data.length; i++) {
              favorites.push(data[i].refObject);
          }
          self.syncPartData(null, favorites, callback);
      });
    }

    self.getCases = function (request, start, offset, callback) {
        if (request != null) {
            if (connection) {
                request['objectType'] = "Project";
                ClientAPI.findPaged(request, [['modificationDate', -1]], start / offset + 1, offset, function (err, data) {
                    if (!err) {
                        var result = [];
                        for (var i = 0, max = data.length; i < max; i++) {
                            result.push(self.encodeItem(data[i]));
                        }
                        if (callback != null && typeof callback == "function") {
                            callback(result);
                        }
                    }
                });
            } else {
                var queryParams = [];
                var query = "SELECT * FROM " + table + " WHERE";
                var temp = false;

                for (var propery in request) {
                    if (propery == '$or') {
                        if (temp) {
                            query += " AND";
                        } else {
                            temp = true;
                        }

                        if (request[propery][0].owner) {
                            query += " ([owner-id] =? OR [author-id] =?)";
                            queryParams.push(getTextTypeObject(request[propery][0].owner, 'integer'));
                            queryParams.push(getTextTypeObject(request[propery][1].creator, 'integer'));
                        }

                        if (request[propery][1].owner) {
                            query += " ([owner-id] =? OR [author-id] =?)";
                            queryParams.push(request[propery][1].owner);
                            queryParams.push(request[propery][0].creator);
                        }

                        if (request[propery][0].status) {
                            query += " (status=? OR status=?)";
                            queryParams.push(request[propery][0].status);
                            queryParams.push(request[propery][1].status);
                        }
                    } else {
                        if (temp) {
                            query += " AND " + propery + "=?";
                            queryParams.push(request[propery]);
                        } else {
                            temp = true;
                            query += " " + propery + "=?";
                            queryParams.push(request[propery]);
                        }
                    }
                }

                query += " ORDER BY 'updated-at' DESC LIMIT ?,?";
                queryParams.push(start);
                queryParams.push(offset);
                db.selectByQuery(query, queryParams, table, function (data) {
                    if (callback != null && typeof callback == "function") {
                        callback(data);
                    }
                });
            }
        } else {
            console.log("CaseController - getCases - missed request!");
        }
    };

    self.getOpenInProgressCase = function (callback) {
        if (connection) {
            ClientAPI.find({objectType: "Project", status: {$ne: 'Closed'}}, [['modificationDate', -1]], function (err, data) {
                var result = [];
                for (var i = 0, max = data.length; i < max; i++) {
                    result.push(self.encodeItem(data[i]));
                }
                if (callback != null && typeof callback == "function") {
                    callback(result);
                }
            });
        } else {
            db.selectByQuery('SELECT * FROM ' + table + ' WHERE "status" !=? ORDER BY "updated-at" DESC', ['Closed'], table, function (data) {
                if (callback != null && typeof callback == "function") {
                    callback(data);
                }
            });
        }
    };

    self.updateCaseLinks = function (id, caseLinks) {
        self.findById(id, function (data) {
            if (data != null) {
                data['urlList'] = caseLinks;
                ClientAPI.updateObject(data, function (err, data) {
                    if (!err) {
                        goBack();
                    } else {
                        alert("Incorrect format of the link, please specify full address.");
                    }
                });
            }
        });
    };

    self.findAllFilters = function(callback) {
        if (connection) {
          ClientAPI.findSettings({query:{settingType: 'CustomFilter',
          creator: window.localStorage.getItem('userId'), refObjectType: 'Project' }}, function(err, data){
            if (callback != null && typeof callback == "function") {
                callback(data);
            }
          });
        } else {
          if (callback != null && typeof callback == "function") {
              callback([]);
          }
        }
    };

    self.findCustomCase = function(filterQuery, callback) {
      if (connection){
        if(filterQuery[0].$match.immediateParent){
            filterQuery[0].$match.immediateParent = new ObjectId(filterQuery[0].$match.immediateParent.val);
        }
        ClientAPI.aggregate(filterQuery, function(err, items){
          var projects = [];
          if (!err) {
              for (var i = 0, max = items.length; i < max; i++) {
                  projects.push(CaseController.encodeItem(items[i]));
              }
          }
          if (callback != null && typeof callback == "function") {
              callback(projects);
          }
        });
      } else {
        if (callback != null && typeof callback == "function") {
            callback([]);
        }
      }
    };


    return self;
}(CaseController || {}));
