var CommentController = (function (CommentController) {
    var self = CommentController;
    var table = models.commentDBModel.name;
    var objectType = {objectType: "Comment"};

    self.synchronization = function (items, callback) {
        if (callback != null && typeof callback == "function") {
            callback();
        }
    };

    self.syncPartData = function (err, data, callback) {
        var comments = [];
        if (err == null) {
            for (var i = 0, max = data.length; i < max; i++) {
                var comment = self.encodeItem(data[i]);
                comments.push(comment);
            }
        }

        if (callback != null && typeof callback == "function") {
            callback(comments);
        }
    };

    self.encode = function (items, callback) {
        var resultItem, result = [];
        if (items != null) {
            if (items.length != null && items.length > 0) {
                for (var i = 0, max = items.length; i < max; i++) {
                    resultItem = self.encodeItem(items[i]);
                    if (resultItem != null) {
                        result.push(resultItem);
                    }
                }
                if (callback != null && typeof callback == "function") {
                    callback(result);
                }
            } else {
                resultItem = self.encodeItem(items);
                if (resultItem != null) {
                    result.push(resultItem);
                }
                if (callback != null && typeof callback == "function") {
                    callback(result);
                }
            }
        }
    };

    self.encodeItem = function (item) {
        if (item['_id'] != null) {
            var result = {};
            result['id'] = getTextTypeObject(item['_id'], 'integer');
            result['created-at'] = getTextTypeObject(item['creationDate'], 'datetime');
            result['updated-at'] = getTextTypeObject(item['modificationDate'], 'datetime');
            result['author-id'] = getTextTypeObject(item['creator'], 'integer');
            result['group-id'] = {"type": "integer", "nil": "true"};
            result['group'] = "";
            result['owner-id'] = {"type": "integer", "nil": "true"};
            result['visible-to'] = "Owner";
            result['attachments'] = "";
            result['body'] = item.comment;

            for (var i = 0, max = item.accessList.length; i < max; i++) {
                if (item.accessList[i].identity == "Everyone@" + localStorage.getItem('userDomain')) {
                    result['visible-to'] = "Everyone";
                }
            }

            if (item['assignedTo'] != null) {
                result['owner-id'] = getTextTypeObject(item['assignedTo'], 'integer');
            } else {
                result['owner-id'] = getTextTypeObject(item['owner'], 'integer');
            }

            if (item['immediateParent'] != null) {
                result['parent-id'] = getTextTypeObject(item['immediateParent'], 'integer');
            }

            result['collection-id'] = {"type": "integer", "nil": "true"};
            result['collection-type'] = {"nil": "true"};

            if (item['rootObject'] != null) {
                result['subject-id'] = getTextTypeObject(item['rootObject'], 'integer');
                switch (item.rootObjectDisplay.toLowerCase()) {
                    case "note":
                        result['subject-type'] = item.rootObjectType;
                        result['subject-name'] = item.rootObjectDisplay;
                        break;
                    case "company":
                        result['subject-type'] = "Party";
                        result['subject-name'] = item.rootObjectDisplay;
                        break;
                    case "person":
                        result['subject-type'] = "Party";
                        result['subject-name'] = "";
                        var names = item.rootObjectDisplay.split(" ");
                        if (names[1]) {
                            result['subject-name'] += names[1] + " ";
                        }
                        if (names[0]) {
                            result['subject-name'] += names[0];
                        }
                        break;
                    case "deal":
                        result['subject-type'] = item.rootObjectType;
                        result['subject-name'] = item.rootObjectDisplay;
                        result['collection-id'] = getTextTypeObject(item.rootObject, 'integer');
                        result['collection-type'] = item.rootObjectDisplay;
                        break;
                    case "project":
                        result['subject-type'] = "Kase";
                        result['subject-name'] = item.rootObjectDisplay;
                        result['collection-id'] = getTextTypeObject(item.rootObject, 'integer');
                        result['collection-type'] = "Kase";
                        break;
                }
            }
            return result;
        } else {
            return undefined;
        }
    };


    self.save = function (name, immediate, callback) {
        var comment = null;
        if (typeof name == "object") {
            comment = name;
        } else {
            comment = objectType;
            if (name) {
                comment['comment'] = name;
            }

            comment['status'] = "Created";
            comment['immediate'] = immediate;
        }

        if (connection) {
            ClientAPI.attachChildObject(comment, immediate, function (err, res) {
                if (err) {
                    if (callback != null && typeof callback == "function") {
                        callback();
                    }
                } else {
                    var commentSave = self.encodeItem(res);
//			   		ClientAPI.find({_id : new ObjectId(immediate)}, null, function(err, data){
//			   			if (!err) {
//			   				if (data.length > 0) {
//			   					var permission = data[0].accessList;
//			   					permission.push({canUpdateACL: false, canUpdateData: true, identity: data[0].owner});
//			   					self.updateAllCommentsACL(immediate, permission);
//			   				}
//			   			}
//			   		});

                    if (commentSave == null && res.object != null) {
                        commentSave = self.encodeItem(res.object);
                    }

                    if (res != null && callback != null && typeof callback == "function") {
                        callback(commentSave);
                    }
                }
            });
        } else {
            var objectTask = {'object': comment, 'objectType': table, id: UtilController.randomNumber()};
            db.insert(models.offlineInsertDBModel.name, [objectTask], function () {
                alert(Messages.Offline.offlineMode);
                if (immediate != null && typeof immediate == "function") {
                    immediate();
                } else if (callback != null && typeof callback == "function") {
                    callback();
                }
            });
        }
    };

    self.update = function (id, name, callback) {
        if (connection) {
            if (id) {
                self.findById(id, function (data) {
                    if (data != null) {
                        if (name) {
                            data.comment = name;
                        }
                        data.comment = name;
                        ClientAPI.updateObject(data, function (err, data) {
                            if (data.object) {
                                var comment = self.encodeItem(data.object);
                                if (data != null && callback != null && typeof callback == "function") {
                                    callback(comment);
                                }
                            } else {
                                alert(data['statusDesc']);
                                if (callback != null && typeof callback == 'function') {
                                    callback();
                                }
                            }

                        });
                    } else {
                        if (callback != null && typeof callback == "function") {
                            callback();
                        }
                    }
                });
            } else {
                if (callback != null && typeof callback == "function") {
                    callback();
                }
            }
        } else {
            alert("You are in offline mode. You can't save this comment!");
        }
    };

    self.findAll = function (callback) {
        if (connection) {
            ClientAPI.find({objectType: "Comment"}, [["modificationDate", -1]], function (err) {
                if (err == null) {
                    self.encode(items, callback);
                } else {
                    if (callback != null && typeof callback == "function") {
                        callback();
                    }
                }
            });
        } else {
            if (callback != null && typeof callback == "function") {
                callback([]);
            }

        }
    };

    self.findById = function (id, callback) {
        if (id == null) {
            if (callback != null && typeof callback == "function") {
                callback();
            }
        }

        if (connection) {
            ClientAPI.find({objectType: "Comment", _id: new ObjectId(id)}, null, function (err, data) {
                if (err == null && callback != null && typeof callback == "function") {
                    callback(data[0]);
                } else {
                    if (callback != null && typeof callback == "function") {
                        callback();
                    }
                }
            });
        } else {
            if (callback != null && typeof callback == "function") {
                callback();
            }
        }
    };

    self.delete = function (id, callback) {
        if (connection) {
            ClientAPI.removeObject(id, function (err, data) {
                if (data != null && data.statusCode != null) {
                    if (callback != null && typeof callback == "function") {
                        callback(data.statusCode);
                    }
                }
            });
        }
    };

    self.getAllByNoteId = function (id, callBack) {
        if (connection) {
            if (id != null) {
                ClientAPI.find({immediateParent: new ObjectId(id), objectType: "Comment"}, [['creationDate', 1]], function (err, data) {
                    self.syncPartData(err, data, callBack);
                });
            } else {
                if (callBack != null && typeof callBack == "function") {
                    callBack([]);
                }
            }
        } else {
            if (callBack != null && typeof callBack == "function") {
                callBack([]);
            }
        }
    };

    self.updateAllCommentsACL = function (id, visible) {
        if (connection) {
            ClientAPI.find({objectType: "Comment", immediateParent: new ObjectId(id)}, null, function (err, data) {
                $(data).each(function (index, value) {
                    ClientAPI.updateACL(value._id, visible, function (err, data) {
                        console.log(err || data);
                    });
                });
            });
        }
    };

    return self;
}(CommentController || {}));
