var CompanyController = (function (CompanyController)
{
    var self = CompanyController;
    var table = models.companiesDBModel.name;
    var objectType = {objectType: "Company"};

    self.synchronization = function (items, callback) {
        if (items != null && items.length != null && items.length > 0) {
            self.encode(items, function (results) {
                var keys = [];
                for (var i = 0, max = results.length; i < max; i++) {
                    keys.push(results[i].id);
                }
                if (keys.length != 0) {
                    db.deleteFromTableByIds(table, 'id', keys, function () {
                        db.insertMultiple(table, [results], function () {
                            if (callback != null && typeof callback == "function") {
                                callback(results);
                            }
                        });
                    });
                } else {
                    if (callback != null && typeof callback == "function") {
                        callback([]);
                    }
                }
            });
        } else {
            if (callback != null && typeof callback == "function") {
                callback([]);
            }
        }
    };

    self.syncPartData = function (err, data, callback) {
        if (err == null) {
            if (data != null && data.length > 0) {
                var companies = [];
                var keys = [];

                for (var i = 0, max = data.length; i < max; i++) {
                    var company = self.encodeItem(data[i]);
                    keys.push(company.id);
                    companies.push(company);
                }

                if (keys.length != 0) {
                    db.deleteFromTableByIds(table, 'id', keys, function () {
                        db.insertMultiple(table, [companies], function () {
                            if (callback != null && typeof callback == "function") {
                                callback(companies);
                            }
                        });
                    });
                } else {
                    if (callback != null && typeof callback == "function") {
                        callback([]);
                    }
                }
            } else {
                if (callback != null && typeof callback == "function") {
                    callback([]);
                }
            }
        } else {
            if (callback != null && typeof callback == "function") {
                callback([]);
            }
        }
    };


    self.encode = function (items, callback) {
        var resultItem, result = [];
        if (items != null) {
            if (items.length != null && items.length > 0) {
                for (var i = 0, max = items.length; i < max; i++) {
                    resultItem = self.encodeItem(items[i]);
                    if (resultItem != null) {
                        result.push(resultItem);
                    }
                }
                if (callback != null && typeof callback == "function") {
                    callback(result);
                }
            } else {
                resultItem = self.encodeItem(items);
                if (resultItem != null) {
                    result.push(resultItem);
                }
                if (callback != null && typeof callback == "function") {
                    callback(result);
                }
            }
        }
    };

    self.encodeItem = function (item) {
        if (item['_id'] != null) {
            var i, max, result = {};
            result['id'] = getTextTypeObject(item['_id'], 'integer');
            if (item['companyName']) {
                result['name'] = item['companyName'].escape();
            } else {
                item['companyName'] = "";
            }

            result['objectType'] = item['objectType'];
            result['searchKeywords'] = item['searchKeywords'];

            if (item['companyInfo'] != null) {
                result['background'] = item['companyInfo'].escape();
            } else {
                result['background'] = {"nil": "true"};
            }

            result['created-at'] = getTextTypeObject(item['creationDate'], 'datetime');
            result['updated-at'] = getTextTypeObject(item['modificationDate'], 'datetime');

            if (item['owner'] != null) {
                result['owner-id'] = getTextTypeObject(item['owner'], 'integer');
            } else {
                result['owner-id'] = {"type": "integer", "nil": "true"};
            }

            result['author-id'] = getTextTypeObject(item['creator'], 'integer');
            result['subject_datas'] = "";
            result['tags'] = "";
            result['visible-to'] = item.accessList;


            if (item['picture'] != null && item['picture']['file_name'] != null) {
                result['avatar-url'] = item['picture']['file_name'];
            } else {
//                                                            result['avatar-url']                           = 'http://dge9rmgqjs8m1.cloudfront.net/highrise/missing/avatar.gif?r=3';
                result['avatar-url'] = null;
            }

            result['group-id'] = {"type": "integer", "nil": "true"};
            result['group'] = "";

            var phones = [];
            var emails = [];
            var webAddress = [];

            if (item.phoneNumbers.length > 0) {
                if (item.phoneNumbers.length == 1) {
                    phones = {
                        "id": getTextTypeObject(UtilController.randomNumber(), 'integer'),
                        "number": item.phoneNumbers[0].phoneNumber,
                        "location": item.phoneNumbers[0].numberType
                    };
                } else {
                    for (i = 0, max = item.phoneNumbers.length; i < max; i++) {
                        phones.push({
                            "id": getTextTypeObject(UtilController.randomNumber(), 'integer'),
                            "number": item.phoneNumbers[i].phoneNumber,
                            "location": item.phoneNumbers[i].numberType
                        });
                    }
                }
            }

            if (item.eMailAddresses.length > 0) {
                if (item.eMailAddresses.length == 1) {
                    emails = {
                        "id": getTextTypeObject(UtilController.randomNumber(), 'integer'),
                        "address": item.eMailAddresses[0].eMailAddress,
                        "location": item.eMailAddresses[0].eMailAddressType
                    };
                } else {
                    for (i = 0, max = item.eMailAddresses.length; i < max; i++) {
                        emails.push({
                            "id": getTextTypeObject(UtilController.randomNumber(), 'integer'),
                            "address": item.eMailAddresses[i].eMailAddress,
                            "location": item.eMailAddresses[i].eMailAddressType
                        });
                    }
                }
            }

            if (item.webAddresses.length > 0) {
                if (item.webAddresses.length == 1) {
                    webAddress = {
                        "id": getTextTypeObject(UtilController.randomNumber(), 'integer'),
                        "URL": item.webAddresses[0].URL,
                        "urlType": item.webAddresses[0].urlType
                    };
                } else {
                    for (i = 0, max = item.webAddresses.length; i < max; i++) {
                        webAddress.push({
                            "id": getTextTypeObject(UtilController.randomNumber(), 'integer'),
                            "URL": item.webAddresses[i].URL,
                            "urlType": item.webAddresses[i].urlType
                        });
                    }
                }
            }

            var tags = [];
            if (item['tags'].length > 0) {
                if (item['tags'].length == 1) {
                    tags = {
                        'id': getTextTypeObject(UtilController.randomNumber(), 'integer'),
                        'name': item['tags'][0].tag
                    };
                } else {
                    for (i = 0, max = item.tags.length; i < max; i++) {
                        tags.push({
                            'id': getTextTypeObject(UtilController.randomNumber(), 'integer'),
                            'name': item['tags'][i].tag
                        });
                    }
                }
            }

            result['tags'] = {"type": "array", "tag": tags};

            result['contact-data'] = {
                "instant-messengers": {"type": "array"},
                "twitter-accounts": {"type": "array"},
                "email-addresses": {"type": "array", "email-address": emails},
                "phone-numbers": {"type": "array", "phone-number": phones},
                "addresses": {"type": "array"},
                "web-addresses": {"type": "array", "web-address": webAddress}
            };

            return result;
        } else {
            return undefined;
        }
    };

    self.save = function (name, phones, emails, webs, info, visibleTo, callback) {
        var company = null;
        if (typeof name == "object") {
            company = name;
        } else {
            company = objectType;
            if (name) {
                company['companyName'] = name.escape();
            } else {
                company['companyName'] = "";
            }
            if (info) {
                company['companyInfo'] = info.escape();
            } else {
                company['companyInfo'] = "";
            }

            company['eMailAddresses'] = emails;
            company['phoneNumbers'] = phones;
            company['webAddresses'] = webs;
            company['status'] = "Active";

            company['accessList'] = visibleTo;
        }
        if (connection) {
            ClientAPI.insertObject(company, function (err, res) {
                if (err) {
                    if (callback != null && typeof callback == "function") {
                        callback();
                    }
                } else {
                    var company = self.encodeItem(res.object);
                    db.insert(table, [company], function () {
                        if (res != null && callback != null && typeof callback == "function") {
                            // workaround sync for sip users
                            OfflineController.synchronization(function () {
                                SyncController.syncCompany(function () {
                                    callback(company);
                                });
                            });
                        }
                    });
                }
            });
        } else {
            var objectCompany = {'object': company, 'objectType': table, id: UtilController.randomNumber()};
            db.insert(models.offlineInsertDBModel.name, [objectCompany], function () {
                alert(Messages.Offline.offlineMode);
                if (phones != null && typeof phones == "function") {
                    phones();
                } else if (callback != null && typeof callback == "function") {
                    // workaround sync for sip users
//                    OfflineController.synchronization(function () {
//                        SyncController.sync(function () {
//                            callback();
//                        });
//                    });
                        callback();
                }
            });
        }
    };

    self.update = function (id, name, phones, emails, webs, info, visibleTo, callback) {
        if (connection) {
            if (id) {
                self.findById(id, function (data) {
                    if (data != null && data[0] != null) {
                        var company = data[0];
                        if (name) {
                            company['companyName'] = name;
                        } else {
                            company['companyName'] = "";
                        }

                        if (info) {
                            company['companyInfo'] = info;
                        } else {
                            company['companyInfo'] = "";
                        }

                        if (emails != null && emails.length > 0) {
                            company['eMailAddresses'] = emails;
                        } else {
                            company['eMailAddresses'] = null;
                        }

                        if (phones != null && phones.length > 0) {
                            company['phoneNumbers'] = phones;
                        } else {
                            company['phoneNumbers'] = null;
                        }

                        if (webs != null && webs.length > 0) {
                            company['webAddresses'] = webs;
                        } else {
                            company['webAddresses'] = null;
                        }

                        company['status'] = "Active";

                        ClientAPI.updateObject(company, function (err, data) {
                            if (err) {
                                if (callback != null && typeof callback == "function") {
                                    callback();
                                }
                            } else {
                                if (data.object) {
                                    var updatedCompany = self.encodeItem(data.object);
                                    if (visibleTo != null) {
                                        ClientAPI.updateACL(data.object._id, visibleTo, function (err) {
                                            if (!err) {
                                                CompanyController.findById(id, function (data) {
                                                    if (data[0] != null && data[0]._id != null) {
                                                        sharedContactForOpen = CompanyController.encodeItem(data[0]);
                                                        self.syncPartData(null, data, null);
                                                        callback(sharedContactForOpen);
                                                    }
                                                });
                                            } else {
                                                db.deleteObject(table, "id", getTextTypeObject(id, 'integer'), function () {
                                                    db.insert(table, [updatedCompany], function () {
                                                        if (updatedCompany != null && callback != null && typeof callback == "function") {
                                                            sharedContactForOpen = updatedCompany;
                                                            callback(updatedCompany);
                                                        }
                                                    });
                                                });
                                            }
                                        });
                                    } else {
                                        CompanyController.findById(id, function (data) {
                                            if (data[0] != null && data[0]._id != null) {
                                                sharedContactForOpen = CompanyController.encodeItem(data[0]);
                                                self.syncPartData(null, data, null);
                                                callback(sharedContactForOpen);
                                            }
                                        });
                                    }
                                } else {
                                    alert(data['statusDesc']);
                                    if (callback != null && typeof callback == 'function') {
                                        callback();
                                    }
                                }
                            }
                        });
                    } else {
                        if (callback != null && typeof callback == "function") {
                            callback();
                        }
                    }
                });
            } else {
                if (callback != null && typeof callback == "function") {
                    callback();
                }
            }
        } else {
            alert("You are in offline mode. You can't save this company!");
        }
    };

    self.findAll = function (callback) {
        if (connection) {
            ClientAPI.find(objectType, [["modificationDate", -1]], function (err, data) {
                if (err == null) {
                    self.synchronization(data, function (items) {
                        if (callback != null && typeof callback == "function") {
                            callback(items);
                        }
                    });
                } else {
                    if (callback != null && typeof callback == "function") {
                        callback();
                    }
                }
            });
        } else {
            db.selectAll(table, function (data) {
                if (callback != null && typeof callback == "function") {
                    callback(data);
                }
            });
        }
    };

    self.findById = function (id, callback) {
        if (id == null) {
            console.log('Id is empty');
            if (callback != null && typeof callback == "function") {
                callback();
            }
        }

        if (connection) {
            ClientAPI.find({objectType: "Company", _id: new ObjectId(id)}, null, function (err, data) {
                if (err == null && typeof callback == "function") {
                    callback(data);
                } else {
                    if (typeof callback == "function") {
                        callback();
                    }
                }
            });
        } else {
            db.selectByColumn(table, "id", getTextTypeObject(id, 'integer'), function (data) {
                if (callback != null && typeof callback == "function") {
                    callback(data);
                }
            });
        }
    };

    self.findCompanyByName = function (name, callback) {
        if (name == null || name == "") {
            if (callback != null && typeof callback == "function") {
                callback();
            }
        }

        if (connection) {
            ClientAPI.find({objectType: objectType.objectType, companyName: name}, null, function (err, data) {
                self.syncPartData(err, data, callback);
            });
        } else {
            db.selectByColumn(table, "name", name, function (data) {

                if (data[0].company != null) {
                    data[0] = data[0].company;
                }
                if (callback != null && typeof callback == "function") {
                    callback(data);
                }
            });
        }

    };

    self.delete = function (id, callback) {
        if (connection) {
            ClientAPI.removeObject(id, function (err, data) {
                if (data != null && data.statusCode != null) {
                    if (callback != null && typeof callback == "function") {
                        callback(data.statusCode);
                    }
                }
            });
        }
    };

    return self;
}(CompanyController || {}));
