
var ContactsController = (function (ContactsController) {
    var self = ContactsController;
    var peopleArray = [];
    var companyArray = [];
    var dataRecive = false;
    var i, j, max;

    self.findPhoneContacts = function (filter, callback) {
        if (!isPhoneGap()) {
            callback();
            return;
        }

        showLoadingIndikator();
        var options = new ContactFindOptions();
        options.filter = filter;
        options.multiple = true;

        navigator.contacts.find(["*"], function (contacts) {
            callback(contacts);
        }, function (err) {
            hideLoadingIndikator();
            alert("Error while searching... " + err);
        }, options);
    };

    self.findPhoneContactById = function (id, callback) {
        if (!isPhoneGap()) {
            callback();
            return;
        }

        showLoadingIndikator();
        var options = new ContactFindOptions();
        options.filter = "";
        options.multiple = true;
        navigator.contacts.find(["*"], function (contacts) {
            var contact;
            for (var key in contacts) {
                if (contacts[key].id == id) {
                    contact = contacts[key];
                    break;
                }
            }

            callback(contact);
        }, function (err) {
            hideLoadingIndikator();
            alert("Error while searching... " + err);
        }, options);
    };

    self.capitalize = function (txt) {
        if (txt != null && txt.charAt(0) != null) {
            return txt.charAt(0).toUpperCase() + txt.slice(1);
        } else {
            return txt;
        }
    };

    self.prepareAndSendSingleContact = function (contact) {
        var contactData = self.preparePhoneContactsToSend(contact);
        if (contactData.organization != "") {
            CompanyController.save(contactData.companyName, [], [], "", false, function (newCompany) {
                if (newCompany != null) {
                    PeopleController.save(
                            contactData.givenName, contactData.familyName,
                            newCompany.id.text, contactData.title,
                            contactData.phones, contactData.emails,
                            contactData.info, contactData.visible,
                            newCompany.name,
                            function () {
                                console.log('contact add');
                            });
                }
            });
        } else {
            PeopleController.save(
                    contactData.givenName, contactData.familyName, null, null,
                    contactData.phones, contactData.emails, contactData.info, contactData.visible, null,
                    function () {
                        console.log('contact add');
                    });
        }
    };

    self.preparePhoneContactsToSend = function (contact) {
        var result = {
            'id': "",
            'organization': "",
            'companyName': "",
            'title': "",
            'givenName': "",
            'familyName': "",
            'phones': [],
            'emails': [],
            'info': "",
            'visible': [],
            'avatar': []
        };


        var phoneNumbers = [];
        if (contact.phoneNumbers != null) {
            for (j = 0, max = contact.phoneNumbers.length; j < max; j++) {
                phoneNumbers.push({
                    phoneNumber: $.trim(contact.phoneNumbers[j].value),
                    numberType: $.trim(self.capitalize(contact.phoneNumbers[j].type))
                });
            }
        }

        result.phones = phoneNumbers;

        //emails
        //create email xml for single person
        var emails = [];
        if (contact.emails != null) {
            for (j = 0, max = contact.emails.length; j < max; j++) {
                emails.push({
                    eMailAddress: $.trim(contact.emails[j].value),
                    eMailAddressType: $.trim(self.capitalize(contact.emails[j].type))
                });
            }
        }
        result.emails = emails;

        if (contact.name != null) {
            result.givenName = (contact.name.givenName == null) ? " " : contact.name.givenName;
            result.familyName = (contact.name.familyName == null) ? " " : contact.name.familyName;
        } else {
            result.givenName = (contact.displayName == null) ? " " : contact.displayName;
        }

        if (contact.organizations != null) {
            for (j = 0, max = contact.organizations.length; j < max; j++) {
                if (contact.organizations[j].type == "work") {
                    result.title = (contact.organizations[j].title == null) ? "" : contact.organizations[j].title;
                    result.organization = (contact.organizations[j].name == null) ? "" : contact.organizations[j].name;
                }
            }
        }

        if (contact.photos != null && contact.photos[0] != null && contact.photos[0].value != null && contact.photos[0].value != "") {
            result.avatar = contact.photos[0].value;
        } else {
            result.avatar = "";
        }

        result.id = contact.id;
        return result;
    };

    self.insertContactInDatabaseByTableName = function (table, contacts) {
        db.insertMultiple(table, [contacts], function () {
        });
    };

    self.insertContactInDatabase = function () {
        if (Object.prototype.toString.call(peopleArray) === '[object Array]') {
            db.insertMultiple(models.peopleDBModel.name, [peopleArray], function () {
                peopleArray.length = 0;
            });

        }

        if (Object.prototype.toString.call(companyArray) === '[object Array]') {
            db.insertMultiple(models.companiesDBModel.name, [companyArray], function () {
                companyArray.length = 0;
            });
        }
    };

    /* function param are all contacts from phone */
    self.prepareAndSendAllPhoneContacts = function (contacts) {
        syncAllConnectionCheck = true;
        var myInterval;
        companyArray = [];
        peopleArray = [];
        //maybe sync with existing contacts

        for (var i = 0; i < contacts.length; i++) {
            myInterval = window.setInterval(self.prepareAndSendSingleContact(contacts[i]), 30);
            if (dataRecive) {
                self.insertContactInDatabase();
                gotoPage({url: "contacts", param: null});
                var hideLoaderInterval = setInterval(function () {
                    hideLoadingIndikator();
                    clearInterval(hideLoaderInterval);
                }, 4000);
                return;
            }
        }

        clearInterval(myInterval);
        dataRecive = false;
        self.insertContactInDatabase();
        hideLoadingIndikator();
        $('#add-contact-menu').popup('close');
    };

    self.filterPhone = function (phone) {
        var result = '';
        var testZero = true;
        if (phone == null) {
            return result;
        }

        for (i = 0; i < phone.length; i++) {
            if (!isNaN(phone[i])) {
                if (testZero) {
                    if (phone[i] != '0') {
                        result += phone[i];
                        testZero = false;
                    }
                } else {
                    result += phone[i];
                }

            }
        }
        return result;
    };

    self.findAllFilters = function(callback) {
        if (connection) {
          ClientAPI.findSettings({
            query:{
            'settingType': 'CustomFilter',
            'creator': window.localStorage.getItem('userId'),
            'refObjectType': 'Contact' }
          }, function(err, data){
            if (callback != null && typeof callback == "function") {
                callback(data);
            }
          });
        } else {
          if (callback != null && typeof callback == "function") {
              callback([]);
          }
        }
    };

    self.findCustomContact = function(filterQuery, callback) {
      if (connection){
        if(filterQuery[0].$match.immediateParent){
            filterQuery[0].$match.immediateParent = new ObjectId(filterQuery[0].$match.immediateParent.val);
        }
        if (filterQuery[1]){
           var sortObj = [];
            for (var key in filterQuery[1]) {
              if (filterQuery[1].hasOwnProperty(key)) {
                sortObj.push(key);
                sortObj.push(filterQuery[1][key]);
              }
            }
        }

        if (sortObj) {
          sortObj = [sortObj]
        } else {
          sortObj = [];
        }

        ClientAPI.find(filterQuery[0].$match, sortObj, function(err, items){
          var contacts = [];
          if (!err) {
              for (var i = 0, max = items.length; i < max; i++) {
                  if (items[i]._id) {
                      if (items[i].objectType == "Company") {
                          contacts.push(CompanyController.encodeItem(items[i]));
                      } else if (items[i].objectType == "Person") {
                          contacts.push(PeopleController.encodeItem(items[i]));
                      }
                  }
              }
          }
          if (callback != null && typeof callback == "function") {
              callback(contacts);
          }
        });
      } else {
        if (callback != null && typeof callback == "function") {
            callback([]);
        }
      }
    };

    return self;
}(ContactsController || {}));
