var DealController = (function (DealController)
{
    var self = DealController;
    var table = models.dealDBModel.name;
    var countRequest;
    var changeParent;
    self.synchronization = function (items, callback) {
        if (items != null && items.length != null && items.length > 0) {
            self.encode(items, function (results) {
                var keys = [];
                for (var i = 0, max = results.length; i < max; i++) {
                    keys.push(results[i].id);
                }
                if (keys.length > 0) {
                    DEALS = results;
                    db.deleteFromTableByIds(table, 'id', keys, function () {
                        db.insertMultiple(table, [results], function (data) {
                            if (callback != null && typeof callback == "function") {
                                callback(results);
                            }
                        });
                    });
                } else {
                    if (callback != null && typeof callback == "function") {
                        callback([]);
                    }
                }
            });
        } else {
            if (callback != null && typeof callback == "function") {
                callback([]);
            }
        }
    };

    self.syncPartData = function (err, data, callback) {
        if (err == null) {
            if (data != null && data.length > 0) {
                var deals = [];
                var keys = [];

                for (var i = 0, max = data.length; i < max; i++) {
                    //check if encoded before
                    if (data[i]) {
                      if (data[i].id) {
                          var deal = data[i];
                          keys.push(deal.id);
                          deals.push(deal);
                      } else {
                          var deal = self.encodeItem(data[i]);
                          keys.push(deal.id);
                          deals.push(deal);
                      }
                    }
                }

                if (keys.length != 0) {
                    db.deleteFromTableByIds(table, 'id', keys, function () {
                        db.insertMultiple(table, [deals], function () {
                            if (callback != null && typeof callback == "function") {
                                callback(deals);
                            }
                        });
                    });
                } else {
                    if (callback != null && typeof callback == "function") {
                        callback([]);
                    }
                }
            } else {
                if (callback != null && typeof callback == "function") {
                    callback([]);
                }
            }
        } else {
            if (callback != null && typeof callback == "function") {
                callback([]);
            }
        }
    };

    self.encode = function (items, callback) {
        var result = [];
        var resultItem = {};
        if (items != null) {
            if (items.length != null && items.length > 0) {
                for (var i = 0, max = items.length; i < max; i++) {
                    resultItem = self.encodeItem(items[i]);
                    if (resultItem != null) {
                        result.push(resultItem);
                    }
                }
                if (callback != null && typeof callback == "function") {
                    callback(result);
                }
            } else {
                resultItem = self.encodeItem(items);
                if (resultItem != null) {
                    result.push(resultItem);
                }
                if (callback != null && typeof callback == "function") {
                    callback(result);
                }
            }
        }
    };

    self.encodeItem = function (item) {
        if (item && item['_id'] != null) {
            var partyId = null;
            var result = {};
            var links = [];
            result['id'] = getTextTypeObject(item['_id'], 'integer');
            result['name'] = item['dealName'].escape();
            if (item['dealInfo'] != null) {
                result['background'] = item['dealInfo'];
            } else {
                result['background'] = {"nil": "true"};
            }
            result['created-at'] = getTextTypeObject(item['creationDate'], 'datetime');
            result['updated-at'] = getTextTypeObject(item['modificationDate'], 'datetime');

            result['owner-id'] = getTextTypeObject(item['owner'], 'integer');
            result['author-id'] = getTextTypeObject(item['creator'], 'integer');
            result['account-id'] = {"type": "integer", "nil": "true"};
            result['category-id'] = getTextTypeObject(item['dealCategory'], 'integer');
            result['group-id'] = {"type": "integer", "nil": "true"};
            result['group'] = "";
            result['searchKeywords'] = item['searchKeywords'];
            result['probability'] = item.probability;
            result['referenceId'] = item.referenceId;
            result['entityId'] = item.entityId;
            if (item.closeDate) {
                result['closeDate'] = item.closeDate;
            }

            if (item['currency'] == undefined) {
                result['currency'] = "USD";
            } else {
                result['currency'] = item['currency'];
            }

            result['duration'] = {"type": "integer", "text": "1"};
            result['visible-to'] = item.accessList;
            result['salesStage'] = item.salesStage || "";

            var dealCategory = dealCategories.filter(function (category) {
                return category.id.text == item['dealCategory'];
            });

            if (dealCategory.length > 0) {
                result['category'] = {
                    "name": dealCategory[0].name,
                    "id": getTextTypeObject(dealCategory[0].id.text, 'integer')
                };
            }
            if (item.immediateParent) {
                partyId = item.immediateParent;
                result['party-id'] = getTextTypeObject(partyId, 'integer');
            } else {
                result['party-id'] = {"type": "integer", "nil": "true"};
            }

            if (item.immediateParent && item.immediateParentDisplay && item.immediateParentObjectType) {
                var parentObject = {};
                parentObject.id = item.immediateParent;
                parentObject.name = item.immediateParentDisplay;
                parentObject.type = item.immediateParentObjectType;
                result['party'] = parentObject;
                result['subjectDes'] = "Account : " + item.immediateParentDisplay;
            } else {
                result['party'] = {"type": "integer", "nil": "true"};
            }

            if (item['value'] != null) {
                result['price'] = getTextTypeObject(item['value'], 'integer');
            } else {
                result['price'] = {"type": "integer", "nil": "true"};
            }

            result['price-type'] = item['valueType'];

            result['responsible-party-id'] = {"type": "integer", "nil": "true"};

            if (item['assignedTo'] != null) {
                result['responsible-party-id'] = getTextTypeObject(item['assignedTo'], 'integer');
            }

            result['status'] = item['status'];
            result['status-changed-on'] = {"type": "date", "nil": "true"};
            result['parties'] = {"type": "array"};

            if (item.statusHistory) {
                result["statusHistory"] = item.statusHistory;
            }

            if (item['urlList'] && item['urlList'].length > 0) {
                if (item['urlList'].length == 1) {
                    links = {"id": getTextTypeObject(UtilController.randomNumber(), 'integer'),
                        "url": item.urlList[0].url,
                        "urlDesc": item.urlList[0].urlDescription,
                        "urlType": item.urlList[0].urlType
                    };
                } else {
                    for (var i = 0; i < item['urlList'].length; i++) {
                        links.push({
                            "id": getTextTypeObject(UtilController.randomNumber(), 'integer'),
                            "url": item.urlList[i].url,
                            "urlDesc": item.urlList[i].urlDescription,
                            "urlType": item.urlList[i].urlType
                        });
                    }
                }
            }
            result['link'] = links;

            return result;
        } else {
            return undefined;
        }
    };

    self.syncDealByParentId = function (id, callBack) {
        if (connection) {
            ClientAPI.find({dealParticipants: {$elemMatch: {person: new ObjectId(id)}}}, null, function (err, data) {
                self.syncPartData(err, data, callBack);
            });
        } else {
            if (callBack != null && typeof callBack == "function") {
                callBack([]);
            }
        }
    };

    self.save = function (name, info, categoryId, currency, value, valueType, assignedTo, ownerId, participantId, visibleTo, probability, closeDate, referenceId, dealLinks, callback) {
        var dealObject = {objectType: "Deal"};
        var ownerId = ownerId;

        if (name) {
            dealObject.dealName = name.escape();
        }

        if (closeDate) {
            dealObject.closeDate = new Date(closeDate);
        }
        if (info) {
            dealObject.dealInfo = info;
        }

        if (categoryId) {
            dealObject.dealCategory = new ObjectId(categoryId);
        }

        if (referenceId) {
            dealObject.referenceId = referenceId.escape();
        }

        if (value) {
            dealObject.value = value;
        }

        if (ownerId != null) {
            dealObject.owner = ownerId;
        }

        if (probability) {
            dealObject['probability'] = probability;
        }

        dealObject.currency = currency;
        dealObject.valueType = valueType;

        if (assignedTo) {
            dealObject.assignedTo = assignedTo;
        }

        dealObject.status = 'Negotiation';
        dealObject.duration = 1;
        dealObject.durationType = "Days";

        if (participantId != null) {
            dealObject.dealParticipants = [];
            dealObject.dealParticipants.push({participant_id: new ObjectId(participantId)});
        }
        dealObject['accessList'] = visibleTo;

        if (dealLinks && dealLinks.length > 0) {
            dealObject['urlList'] = dealLinks;
        }

        if (connection) {
            if (participantId != null) {
                ClientAPI.attachChildObject(dealObject, participantId, function (err, res) {
                    var dealSave = self.encodeItem(res.object);
                    db.insert(table, [dealSave], function () {
                        if (res != null && callback != null && typeof callback == "function") {
                            callback([res.object]);
                        }
                    });
                });
            } else {
                ClientAPI.insertObject(dealObject, function (err, res) {
                    if (err) {
                        if (callback != null && typeof callback == "function") {
                            callback();
                        }
                    } else {
                        var dealSave = self.encodeItem(res.object);
                        db.insert(table, [dealSave], function () {
                            if (res != null && callback != null && typeof callback == "function") {
                                callback([res.object]);
                            }
                        });
                    }
                });
            }
        } else {
            var objectDeal = {'object': dealObject, 'objectType': table, 'id': UtilController.randomNumber(), 'parent-id': participantId};
            db.insert(models.offlineInsertDBModel.name, [objectDeal], function () {
                alert(Messages.Offline.offlineMode);
                if (callback != null && typeof callback == "function") {
                    callback();
                }
            });
        }
    };

    self.update = function (id, name, info, categoryId, currency, value, valueType, assignedTo, ownerId, participantId, visibleTo, probability, closeDate, referenceId, dealLinks, callback) {
        if (connection) {
            if (id) {
                ClientAPI.find({'objectType': "Deal", _id: new ObjectId(id)}, null, function (err, data) {
                    if (!err && data != null && data[0] != null) {
                        var dealObject = data[0];
                        var dealObjectParticipantId = null;
                        if (dealObject.immediateParent) {
                            dealObjectParticipantId = dealObject.immediateParent;
                        }
                        if (name) {
                            dealObject.dealName = name.escape();
                        }

                        if (closeDate) {
                            dealObject.closeDate = new Date(closeDate);
                        }

                        if (info) {
                            dealObject.dealInfo = info;
                        }

                        if (categoryId) {
                            dealObject.dealCategory = new ObjectId(categoryId);
                        }

                        if (referenceId) {
                            dealObject.referenceId = referenceId.escape();
                        } else {
                            dealObject.referenceId = "";
                        }

                        if (value) {
                            dealObject.value = value;
                            dealObject.currency = currency;
                            dealObject.valueType = valueType;
                        }
                        if (assignedTo) {
                            dealObject['assignedTo'] = assignedTo;
                        }

                        if (probability) {
                            dealObject['probability'] = probability;
                        }

                        dealObject['urlList'] = dealLinks;

                        dealObject.duration = 1;
                        dealObject.durationType = "Days";

                        if (participantId != null) {
                            dealObject.dealParticipants = [];
                            dealObject.dealParticipants.push({participant_id: new ObjectId(participantId)});
                        } else {
                            dealObject.dealParticipants = [];
                        }

                        var visible = visibleTo;

                        countRequest = 0;
                        changeParent = false;
                        ClientAPI.updateObject(dealObject, function (err, data) {
                            if (!err && data != null && data.object != null) {
                                if (visibleTo != null) {
                                    countRequest++;
                                    console.log(countRequest + " - visibleTo");
                                    ClientAPI.updateACL(id, visible, function (err, res) {
                                        console.log(err||res);
                                        countRequest--;
                                        console.log(countRequest + " - visibleTo");
                                        if(!changeParent){
                                            self.changeDealParent(id, participantId, dealObjectParticipantId, callback);
                                        }
                                        if (countRequest == 0) {
                                            self.findById(id, function (data) {
                                                self.syncPartData(null, [data], function (data) {
                                                    if (callback != null && typeof callback == 'function') {
                                                        callback(data);
                                                    }
                                                });
                                            });
                                        }
                                    });
                                }
                                if (assignedTo != data.object.assignedTo) {
                                    countRequest++;
                                    ClientAPI.executeTransitionalAction('Assign', 'Deal', data.object._id, {assignedTo: assignedTo}, function () {
                                        countRequest--;
                                        if (countRequest == 0) {
                                            self.findById(id, function (data) {
                                                self.syncPartData(null, data, function (data) {
                                                    if (callback != null && typeof callback == 'function') {
                                                        callback(data);
                                                    }
                                                });
                                            });
                                        }
                                    });
                                }

                                if (ownerId != data.object.owner) {
                                    countRequest++;
                                    console.log(countRequest + " - owner");
                                    ClientAPI.changeOwner(data.object._id, JSON.stringify(ownerId), function (err, data) {
                                        console.log(err||data);
                                        countRequest--;
                                        console.log(countRequest + " - owner");
                                        if(!changeParent){
                                            self.changeDealParent(id, participantId, dealObjectParticipantId, callback);
                                        }
                                        if (countRequest == 0) {
                                            self.findById(id, function (data) {
                                                self.syncPartData(null, data, function (data) {
                                                    if (callback != null && typeof callback == 'function') {
                                                        callback(data);
                                                    }
                                                });

                                            });
                                        }
                                    });
                                }

                                if((!(ownerId != data.object.owner) && !(visibleTo != null))){
                                    self.changeDealParent(id, participantId, dealObjectParticipantId, callback);
                                }

                                if (countRequest == 0 && callback != null && typeof callback == 'function') {
                                    callback([self.encodeItem(data.object)]);
                                }
                            } else {
                                alert(data.statusDesc);
                                if (callback != null && typeof callback == 'function') {
                                    callback();
                                }
                            }
                        });
                    } else {
                        if (callback != null && typeof callback == "function") {
                            callback();
                        }
                    }
                });
            } else {
                if (callback != null && typeof callback == "function") {
                    callback();
                }
            }
        } else {
            alert("You are in offline mode. You can't save this comment!");
        }
    };

    self.updateDealLinks = function (id, dealLinks) {
        if (connection) {
            ClientAPI.find({'objectType': "Deal", _id: new ObjectId(id)}, null, function (err, data) {
                var dealObject = data[0];
                if (dealLinks && dealLinks.length > 0) {
                    dealObject['urlList'] = dealLinks;
                    ClientAPI.updateObject(dealObject, function (err, data) {
                        if (!err) {
                            goBack();
                        } else {
                            alert("Incorrect format of the link, please specify full address.");
                        }
                    });
                }
            });
        }
    };

    self.changeDealParent = function (dealId, subjectId, oldParticipant, callback) {
        changeParent = true;
        countRequest++;
        ClientAPI.find({_id: new ObjectId(dealId)}, null, function (err, data) {
            if (subjectId != null && subjectId != oldParticipant) {
                console.log(countRequest + " - participantId");
                ClientAPI.attachChildObject(data[0], subjectId, function (err, data) {
                    console.log(err||data);
                    countRequest--;
                    console.log(countRequest + " - participantId");
                    if (countRequest == 0) {
                        self.findById(dealId, function (data) {
                            self.syncPartData(null, data, function (data) {
                                if (callback != null && typeof callback == 'function') {
                                    callback(data);
                                }
                            });
                        });
                    }
                });
            } else if (subjectId == null) {
                ClientAPI.promoteToRootObject(data[0]['_id'], data[0].objectType, function () {
                    countRequest--;
                    if (countRequest == 0) {
                        self.findById(dealId, function (data) {
                            self.syncPartData(null, data, function (data) {
                                if (callback != null && typeof callback == 'function') {
                                    callback(data);
                                }
                            });
                        });
                    }
                });
            } else {
                countRequest--;
                if (callback != null && typeof callback == 'function' && countRequest == 0) {
                    callback(data);
                }
            }
        });
    };

    self.findAll = function (callback) {
        if (connection) {
            ClientAPI.find({objectType: "Deal"}, [["modificationDate", -1]], function (err, data) {
                if (err == null) {
                    self.synchronization(data, function (items) {
                        if (callback != null && typeof callback == "function") {
                            callback(items);
                        }
                    });
                } else {
                    if (callback != null && typeof callback == "function") {
                        callback();
                    }
                }
            });
        } else {
            db.selectAll(table, function (data) {
                if (callback != null && typeof callback == "function") {
                    callback(data);
                }
            });
        }
    };

    self.findByObjectId = function (id, callback) {
        if (id == null) {
            if (callback != null && typeof callback == "function") {
                callback();
            }
        }

        if (connection) {
            ClientAPI.find({objectType: "Deal", _id: new ObjectId(id)}, null, function (err, data) {
                if (err == null && typeof callback == "function") {
                    if (typeof callback == "function") {
                        callback(data[0]);
                    }
                } else {
                    if (typeof callback == "function") {
                        callback();
                    }
                }
            });
        } else {
            db.selectByColumn(table, "id", getTextTypeObject(id, 'integer'), function (data) {
                if (callback != null && typeof callback == "function") {
                    callback(data[0].deal);
                }
            });
        }
    };

    self.findById = function (id, callback) {
        if (id == null) {
            if (callback != null && typeof callback == "function") {
                callback();
            }
        }

        if (connection) {
            ClientAPI.find({objectType: "Deal", _id: new ObjectId(id)}, null, function (err, data) {
                if (err == null && typeof callback == "function") {
                    ClientAPI.getFeed(
                            [
                                {$or:
                                            [
                                                {objectId: new ObjectId(data[0]['_id'])},
                                                {immediateParent: new ObjectId(data[0]['_id'])},
                                                {rootObject: new ObjectId(data[0]['_id'])}
                                            ]
                                },
                                {visible: false}
                            ],
                            [['eventDate', -1]], 1, 100,
                            function (err, feedItems) {
                                if (!err) {
                                    var feedElements = feedItems.filter(function (item) {
                                        if (item.objectType != 'Note' && item.objectType != 'Comment') {

                                            if (item.eventType == 'GrantedAccess') {
                                                item.grantedTo = item.info.split('; ');
                                                item.grantedTo = item.grantedTo.slice(0, item.grantedTo.length - 1);
                                            }
                                            return true;
                                        }
                                    });
                                    callback(self.encodeItem(data[0]), feedElements);
                                } else {
                                    callback(self.encodeItem(data[0]));
                                }
                            }
                    );
                } else {
                    if (typeof callback == "function") {
                        callback();
                    }
                }
            });
        } else {
            db.selectByColumn(table, "id", getTextTypeObject(id, 'integer'), function (data) {
                if (callback != null && typeof callback == "function") {
                    callback(data[0].deal);
                }
            });
        }
    };

    self.delete = function (id, callback) {
        if (connection) {
            ClientAPI.removeObject(id, function (err, data) {
                if (data != null && data.statusCode != null) {
                    if (callback != null && typeof callback == "function") {
                        if (data.statusCode == 0) {
                            db.deleteObject(table, "id", getTextTypeObject(id, 'integer'), function () {
                                callback(data.statusCode);
                            });
                        } else {
                            callback(data.statusCode);
                        }
                    }
                }
            });
        }
    };

    self.findDealByStatus = function (listViewId, id, isUser, callback) {
        var currentStatus = DealFunctions.drawListParams[listViewId]['status'];
        if (connection) {
            var request = {objectType: "Deal"};
            var countItem = DealFunctions.drawListParams[listViewId]['start'] / DealFunctions.drawListParams[listViewId]['offset'] + 1;
            if (currentStatus == "Open") {
                request['$or'] = [{status: "ProposalSent"}, {status: "Negotiation"}, {status: "Analyses"}, {status: "Prospecting"}, {status: "Pending"}];
            } else if (currentStatus == "Won") {
                request['status'] = "Won";
                if (isUser) {
                    request['creator'] = window.localStorage.getItem('userId');
                } else {
                    request['owner'] = window.localStorage.getItem('userId');
                }
            } else if (currentStatus == "Lost") {
                request['status'] = "Lost";
                if (isUser) {
                    request['creator'] = window.localStorage.getItem('userId');
                } else {
                    request['owner'] = window.localStorage.getItem('userId');
                }
            } else if (currentStatus == "Active") {
                request['$or'] = [{status: "ProposalSent"}, {status: "Negotiation"}, {status: "Analyses"}, {status: "Prospecting"}, {status: "Pending"}];
                if (isUser) {
                    request['creator'] = window.localStorage.getItem('userId');
                } else {
                    request['owner'] = window.localStorage.getItem('userId');
                }
            } else if (currentStatus == "Closed") {
                request['$or'] = [{status: "Won"}, {status: "Lost"}, {status: "Inactive"}];
            }

            if (currentStatus != "Favorite") {
                if (id) {
                    if (isUser) {
                        request['owner'] = id;
                    } else {
                        request['$and'] = [{'$and': [{immediateParent: new ObjectId(id)}]}];
                    }
                }
                ClientAPI.findPaged(request, [['modificationDate', -1]], countItem, DealFunctions.drawListParams[listViewId]['offset'], function (err, data) {
                    self.syncPartData(err, data, callback);
                });
            } else {
                FavoriteController.findByType("Deal", function(data) {
                    var favorites = [];
                    for (var i = 0; i < data.length; i++) {
                        favorites.push(data[i].refObject);
                    }
                    self.syncPartData(null, favorites, callback);
                });
            }
        } else {
             if (currentStatus != "Favorite") {
                var query = "SELECT * FROM DEAL WHERE (status =?";
                var queryDate = [];
                if (currentStatus == "Open") {
                    queryDate.push('ProposalSent');
                    queryDate.push('Negotiation');
                    queryDate.push('Analyses');
                    queryDate.push('Prospecting');
                    queryDate.push('Pending');
                    query += " OR status=? OR status=? OR status=? OR status=?)";
                    if (isUser) {
                        queryDate.push(getTextTypeObject(id, 'integer'));
                        query += ' AND [owner-id]=?';
                    }
                } else if (currentStatus == "Won") {
                    queryDate.push(currentStatus);
                    query += ")";
                    if (isUser) {
                        queryDate.push(getTextTypeObject(id, 'integer'));
                        query += " AND [owner-id]=? AND [author-id]=?";
                    } else {
                        query += " AND [owner-id]=?";
                    }
                    queryDate.push(getTextTypeObject(window.localStorage.getItem('userId'), 'integer'));
                } else if (currentStatus == "Lost") {
                    queryDate.push(currentStatus);
                    query += ")";
                    if (isUser) {
                        queryDate.push(getTextTypeObject(id, 'integer'));
                        query += " AND [owner-id]=? AND [author-id]=?";
                    } else {
                        query += " AND [owner-id]=?";
                    }
                    queryDate.push(getTextTypeObject(window.localStorage.getItem('userId'), 'integer'));

                } else if (currentStatus == "Active") {
                    queryDate.push('ProposalSent');
                    queryDate.push('Negotiation');
                    queryDate.push('Analyses');
                    queryDate.push('Prospecting');
                    queryDate.push('Pending');
                    queryDate.push(getTextTypeObject(window.localStorage.getItem('userId'), 'integer'));
                    if (isUser) {
                        queryDate.push(getTextTypeObject(id, 'integer'));
                        query += " OR status=? OR status=? OR status=? OR status=?) AND [owner-id]=? AND [author-id]=?";
                    } else {
                        query += " OR status=? OR status=? OR status=? OR status=?) AND [owner-id]=?";
                    }

                } else if (currentStatus == "Closed") {
                    queryDate.push('Won');
                    queryDate.push('Lost');
                    queryDate.push('Inactive');
                    query += " OR status=? OR status=?)";
                    if (isUser) {
                        queryDate.push(getTextTypeObject(id, 'integer'));
                        query += ' AND [owner-id]=?';
                    }
                }

                if (!isUser) {
                    if (id) {
                        query += " AND [party-id] =?";
                        queryDate.push(getTextTypeObject(id, 'integer'));
                    }
                }

                query += " ORDER BY [updated-at] DESC LIMIT ?, ?";
                queryDate.push(DealFunctions.drawListParams[listViewId]['start']);
                queryDate.push(DealFunctions.drawListParams[listViewId]['offset']);
                db.selectByQuery(query, queryDate, 'deal', function (items) {
                    if (callback != null && typeof callback == 'function') {
                        callback(items);
                    }
                });
            } else {
                FavoriteController.findByType("Deal", function(data) {
                    var favorites = [];
                    for (var i = 0; i < data.length; i++) {
                        favorites.push(data[i].refObject);
                    }
                    if (callback != null && typeof callback == "function") {
                        callback(favorites);
                    }
                });
            }
        }
    };

    self.findDealByStatusCTSales = function (listViewId, id, isUser, callback) {
        var currentStatus = DealFunctions.drawListParams[listViewId]['status'];
        if (connection) {
            var request = {objectType: "Deal"};
            var countItem = DealFunctions.drawListParams[listViewId]['start'] / DealFunctions.drawListParams[listViewId]['offset'] + 1;
            if (currentStatus == "filter-all-deals") {

            } else if (currentStatus == "filter-active-deals") {
              request['$or'] = [
                {salesStage: "00"},
                {salesStage: "01"},
                {salesStage: "02"},
                {salesStage: "03"},
                {salesStage: "04"},
                {salesStage: "05"},
                {salesStage: "07"},
                {salesStage: "09"}
              ];
            } else if (currentStatus == "filter-inactive-deals") {
                request['salesStage'] = "08";
            } else if (currentStatus == "filter-my-active-deals") {
                request['$or'] = [
                  {salesStage: "00"},
                  {salesStage: "01"},
                  {salesStage: "02"},
                  {salesStage: "03"},
                  {salesStage: "04"},
                  {salesStage: "05"},
                  {salesStage: "07"},
                  {salesStage: "09"}
                ];
                if (isUser) {
                    request['creator'] = window.localStorage.getItem('userId');
                } else {
                    request['owner'] = window.localStorage.getItem('userId');
                }
            } else if (currentStatus == "filter-my-won-deals") {
              request['salesStage'] = "07";
              if (isUser) {
                  request['creator'] = window.localStorage.getItem('userId');
              } else {
                  request['owner'] = window.localStorage.getItem('userId');
              }
            } else if (currentStatus == "filter-my-inactive-deals") {
              request['$or'] = [
                {salesStage: "08"},
                {salesStage: "06"}
              ];
              if (isUser) {
                  request['creator'] = window.localStorage.getItem('userId');
              } else {
                  request['owner'] = window.localStorage.getItem('userId');
              }
            }
            if (currentStatus != "filter-favorite-deals") {
                if (id) {
                    if (isUser) {
                        request['owner'] = id;
                    } else {
                        request['$and'] = [{'$and': [{immediateParent: new ObjectId(id)}]}];
                    }
                }
                ClientAPI.findPaged(request, [['modificationDate', -1]], countItem, DealFunctions.drawListParams[listViewId]['offset'], function (err, data) {
                    self.syncPartData(err, data, callback);
                });
            } else {
                FavoriteController.findByType("Deal", function(data) {
                    var favorites = [];
                    for (var i = 0; i < data.length; i++) {
                        favorites.push(data[i].refObject);
                    }
                    self.syncPartData(null, favorites, callback);
                });
            }
        } else {
            if (currentStatus != "filter-favorite-deals") {
               var query = "SELECT * FROM DEAL";
               var queryDate = [];
               if (currentStatus == "filter-all-deals") {
               } else if (currentStatus == "filter-active-deals") {
                  queryDate.push('00');
                  queryDate.push('01');
                  queryDate.push('02');
                  queryDate.push('03');
                  queryDate.push('04');
                  queryDate.push('05');
                  queryDate.push('07');
                  queryDate.push('09');
                  query += " WHERE (salesStage=? OR salesStage=? OR salesStage=? OR salesStage=? OR salesStage=? OR salesStage=? OR salesStage=? OR salesStage=?)";
               } else if (currentStatus == "filter-inactive-deals") {
                 queryDate.push('08');
                 query += " WHERE (salesStage=?)";
               } else if (currentStatus == "filter-my-active-deals") {
                 queryDate.push('00');
                 queryDate.push('01');
                 queryDate.push('02');
                 queryDate.push('03');
                 queryDate.push('04');
                 queryDate.push('05');
                 queryDate.push('07');
                 queryDate.push('09');
                 query += " WHERE (salesStage=? OR salesStage=? OR salesStage=? OR salesStage=? OR salesStage=? OR salesStage=? OR salesStage=? OR salesStage=?)";
                 if (isUser) {
                     queryDate.push(getTextTypeObject(id, 'integer'));
                     query += ' AND [owner-id]=?';
                 }
               } else if (currentStatus == "filter-my-won-deals") {
                 queryDate.push('07');
                 query += " WHERE (salesStage=?)";
                 if (isUser) {
                     queryDate.push(getTextTypeObject(id, 'integer'));
                     query += ' AND [owner-id]=?';
                 }
               } else if (currentStatus == "filter-my-inactive-deals") {
                 queryDate.push('08');
                 queryDate.push('06');
                 query += " WHERE (salesStage=? OR salesStage=?)";
                 if (isUser) {
                     queryDate.push(getTextTypeObject(id, 'integer'));
                     query += ' AND [owner-id]=?';
                 }
               }

               if (!isUser) {
                   if (id) {
                       query += " AND [party-id] =?";
                       queryDate.push(getTextTypeObject(id, 'integer'));
                   }
               }

               query += " ORDER BY [updated-at] DESC LIMIT ?, ?";
               queryDate.push(DealFunctions.drawListParams[listViewId]['start']);
               queryDate.push(DealFunctions.drawListParams[listViewId]['offset']);
               db.selectByQuery(query, queryDate, 'deal', function (items) {
                   if (callback != null && typeof callback == 'function') {
                       callback(items);
                   }
               });
           } else {
               FavoriteController.findByType("Deal", function(data) {
                   var favorites = [];
                   for (var i = 0; i < data.length; i++) {
                       favorites.push(data[i].refObject);
                   }
                   if (callback != null && typeof callback == "function") {
                       callback(favorites);
                   }
               });
           }
        }
    };
    self.refresh = function (callback) {
        if (connection) {
            ClientAPI.find({objectType: "Deal"}, null, function (err, data) {
                if (!err) {
                    self.synchronization(data, callback);
                } else {
                    if (callback != null && typeof callback == "function") {
                        callback();
                    }
                }
            });
        }
    };

    self.findAllFilters = function(callback) {
        if (connection) {
          ClientAPI.findSettings({query:{settingType: 'CustomFilter',
          creator: window.localStorage.getItem('userId'), refObjectType: 'Deal' }}, function(err, data){
            if (callback != null && typeof callback == "function") {
                callback(data);
            }
          });
        } else {
          if (callback != null && typeof callback == "function") {
              callback([]);
          }
        }
    };

    self.findCustomDeal = function(filterQuery, callback) {
      if (connection){
        if (filterQuery[0].$match.immediateParent){
            filterQuery[0].$match.immediateParent = new ObjectId(filterQuery[0].$match.immediateParent.val);
        }
        if (filterQuery[1]){
           var sortObj = [];
            for (var key in filterQuery[1]) {
              if (filterQuery[1].hasOwnProperty(key)) {
                sortObj.push(key);
                sortObj.push(filterQuery[1][key]);
              }
            }
        }

        if (sortObj) {
          sortObj = [sortObj]
        } else {
          sortObj = [];
        }

        ClientAPI.find(filterQuery[0].$match, sortObj, function(err, items){
          var deals = [];
          if (!err) {
              for (var i = 0, max = items.length; i < max; i++) {
                  deals.push(DealController.encodeItem(items[i]));
              }
          }
          if (callback != null && typeof callback == "function") {
              callback(deals);
          }
        });
      } else {
        if (callback != null && typeof callback == "function") {
            callback([]);
        }
      }
    };


    return self;
}(DealController || {}));
