var FavoriteController = (function (FavoriteController)
{
    var self = FavoriteController;
    var table = models.favoriteDBModel.name;
    var syncTypes = ["Deal", "Task", "Project", "Company", "Person", "User"];

    self.encodeItem = function(type, object) {
        var itemData = null;
        if (type == "Company") {
            itemData = CompanyController.encodeItem(object);
        } else if (type == "Person" || type == "User") {
            itemData = PeopleController.encodeItem(object);
        } else if (type == "Deal") {
            itemData = DealController.encodeItem(object);
        } else if (type == "Task") {
          itemData = TaskController.encodeItem(object);
        } else if (type == "Project" || type == "Case") {
          itemData = CaseController.encodeItem(object);
        }
        return itemData;
    };

    self.synchronization = function (items, callback) {
         ClientAPI.findSettings({query: {settingType: 'Favorites', refObjectType: {$in: syncTypes}}}, function(err, data){
            if (!err) {
              db.deleteServerItemFromTableFavorite(data, function() {
                  var insertObjects = [];
                  for (var i = 0, max = data.length; i < max; i++){
                      var itemData = self.encodeItem(data[i].refObjectType, data[i].refObject);
                      if (itemData) {
                          itemData['favorite'] = true;
                          insertObjects.push({
                              "id" : data[i]._id,
                              "refObjectType" :data[i].refObjectType,
                              "refObject"  : itemData,
                              "refObjectId" : data[i].refObject._id
                          });
                      }
                  }

                  db.insertMultiple(table, [insertObjects], function () {
                      if (callback != null && typeof callback == "function") {
                          callback(insertObjects);
                      }
                  });
              });
            } else {
                if (callback != null && typeof callback == "function") {
                    callback();
                }
            }
        });
    };


    self.save = function (refObjectType, refObject, callback) {
        if (refObjectType == "PhoneNumber") {
            var insertObject = {
                "id"    : refObject['id'],
                "refObjectType" :refObjectType,
                "refObject"  : refObject,
                "refObjectId" : refObject.id
            };

            db.insert(table, [insertObject], function () {
                if (callback != null && typeof callback == "function") {
                    callback(insertObject);
                }
            });
        } else {
            var request = {"settingType": "Favorites", 'refObjectType': refObjectType, 'refObject' : refObject};
            if (connection) {
                ClientAPI.addSettings(request, function(err, data){
                    if (err) {
                        if (callback != null && typeof callback == "function") {
                            callback();
                        }
                    } else {
                        var itemData = self.encodeItem(request.refObjectType, request.refObject);
                        if (itemData) {
                            itemData['favorite'] = true;
                            var insertObject = {
                                "id"    : data['object']['_id'],
                                "refObjectType" :refObjectType,
                                "refObject"  : itemData,
                                "refObjectId" : refObject._id
                            };
                            db.insert(table, [insertObject], function () {
                                if (callback != null && typeof callback == "function") {
                                    callback(insertObject);
                                }
                            });
                        } else {
                            if (callback != null && typeof callback == "function") {
                                callback();
                            }
                        }
                    }
                });
            } else {
                var objectContact = {'object': {"favorite": refObject, status: "Add", 'objectType': refObjectType}, 'objectType': table, id: UtilController.randomNumber()};
                db.insert(models.offlineInsertDBModel.name, [objectContact], function () {
                    alert(Messages.Offline.offlineMode);
                    if (callback != null && typeof callback == "function") {
                        callback();
                    }
                });
            }
        }
    };

    self.findByObjectIdDB = function(id, callback){
      if (connection) {
        ClientAPI.findSettings({query: {settingType: 'Favorites', refObject: id}}, function (err, data) {
            if (!err) {
              if (data && data.length > 0) {
                var itemData = self.encodeItem(data[0].refObjectType, data[0].refObject);
                if (itemData) {
                    itemData['favorite'] = true;
                    var insertObject = {
                        "id"    : data[0]['_id'],
                        "refObjectType" :data[0]['refObjectType'],
                        "refObject"  : itemData,
                        "refObjectId" : data[0]['refObject']['_id']
                    };
                    db.insert(table, [insertObject], function () {
                        if (callback != null && typeof callback == "function") {
                            callback([insertObject]);
                        }
                    });
                } else {
                    if (callback != null && typeof callback == "function") {
                        callback();
                    }
                }
              } else {
                if (callback != null && typeof callback == "function") {
                    callback();
                }
              }
            } else {
              if (callback != null && typeof callback == "function") {
                  callback();
              }
            }
        });
      } else {
          db.selectByQuery('SELECT * FROM ' + table + ' WHERE refObjectId=?', [id], table, function (data) {
              if (callback != null && typeof callback == "function") {
                callback(data);
              }
          });
      }
    };

    self.removeFavorite = function (id, callback) {
        if (id) {
            if (connection) {
                ClientAPI.removeSettings({id: id}, function(err, data){
                    db.deleteObject(table, "id", id , function () {
                        if (callback != null && typeof callback == "function") {
                            callback();
                        }
                    });
                });
            } else {
                var objectContact = {'object': {"favorite": {"id" : id}, status: "Remove"}, 'objectType': table, id: UtilController.randomNumber()};
                db.insert(models.offlineInsertDBModel.name, [objectContact], function () {
                    alert(Messages.Offline.offlineMode);
                    if (callback != null && typeof callback == "function") {
                        callback();
                    }
                });
            }
        } else {
            console.log("Id doesn't exists!!!");
        }
    };

    self.removeFavoriteDB = function (id, callback) {
        if (id) {
            db.deleteObject(table, "id", id , function () {
                if (callback != null && typeof callback == "function") {
                    callback();
                }
            });
        } else {
            console.log("Id doesn't exists!!!");
        }
    };

    self.findByType = function (type, callback) {
        if (connection) {
            ClientAPI.findSettings({query: {settingType: 'Favorites', refObjectType: type}}, function(err, data){
                if (!err) {
                    self.synchronization();
                    if (callback != null && typeof callback == "function") {
                        callback(data);
                    }
                } else {
                    if (callback != null && typeof callback == "function") {
                        callback();
                    }
                }
            });
        } else {
            var query = 'SELECT * FROM ' + table + ' WHERE refObjectType=?';
            if (type) {
                db.selectByQuery(query, [type], table, function (data) {
                    if (callback != null && typeof callback == "function") {
                        callback(data);
                    }
                });
            } else {
                if (callback != null && typeof callback == "function") {
                    callback();
                }
            }
        }
    };

    self.findContactAll = function (callback) {
        if (connection) {
            ClientAPI.findSettings({query: {settingType: 'Favorites', refObjectType: {$in: ["Company", "Person"]}}}, function(err, data) {
                if (!err) {
                     db.selectByQuery('SELECT * FROM ' + table + ' WHERE refObjectType=?', ["PhoneNumber"], table, function (datasql) {
                        if (datasql) {
                            for (var i = 0, max = datasql.length; i < max; i++) {
                                var itemDataSql = datasql[i];
                                if (itemDataSql.refObject) {
                                    itemDataSql.refObject.objectType = "PhoneNumber";
                                }
                                data.push(itemDataSql);
                            }
                        }
                        if (callback != null && typeof callback == "function") {
                            callback(data);
                        }
                     });
                } else {
                    if (callback != null && typeof callback == "function") {
                        callback();
                    }
                }
            });
        } else {
            db.selectByQuery('SELECT * FROM ' + table + ' WHERE refObjectType=? OR refObjectType=? OR refObjectType=?', ["Company", "Person", "PhoneNumber"], table, function (data) {
                if (callback != null && typeof callback == "function") {
                    callback(data);
                }
            });
        }
    };

    self.findAllMapByTypes = function (type, callback) {
        if (connection) {
            ClientAPI.findSettings({query: {settingType: 'Favorites', refObjectType: {$in: type}}}, function(err, data){
                if (!err) {

                    db.selectByQuery('SELECT * FROM ' + table + ' WHERE refObjectType=?', ["PhoneNumber"], table, function (datasql) {
                        if (callback != null && typeof callback == "function") {
                            if (data) {
                                var map = {};
                                for (var i = 0, max = data.length; i < max; i++) {
                                  if (data[i] && data[i].refObject && data[i]._id) {
                                    map[data[i].refObject._id] = data[0];
                                  }
                                }
                                for (var i = 0, max = datasql.length; i < max; i++) {
                                    map[datasql[i].refObjectId] = datasql[i];
                                }
                                self.synchronization();
                                callback(map);
                            } else {
                                callback({});
                            }
                        }
                    });
                } else {
                    if (callback != null && typeof callback == "function") {
                        callback({});
                    }
                }
            });
        } else {
            var sql = 'SELECT * FROM ' + table + ' WHERE refObjectType=?';
            for (var i = 0, max = type.length; i < max; i++){
                sql += ' OR refObjectType=?';
            }
            type.push("PhoneNumber");
            db.selectByQuery(sql, type, table, function (data) {
                if (callback != null && typeof callback == "function") {
                    if (data) {
                        var map = {};
                        for (var i = 0, max = data.length; i < max; i++) {
                            map[data[i].refObjectId] = data[i];
                        }
                        callback(map);
                    } else {
                        callback({});
                    }
                }
            });
        }
    };

    self.findUserIds = function(callback){
      if (connection) {
        ClientAPI.findSettings({query: {settingType: 'Favorites', refObjectType: "User"}}, function(err, data){
            var result = [];
            if (!err) {
                data.forEach(function(element){
                  if (element.refObject && element.refObject.identity_id) {
                    result.push(element.refObject.identity_id);
                  }
                });
            }
            if (callback != null && typeof callback == "function") {
                callback(result);
            }
        });
      } else {
        var query = 'SELECT * FROM ' + table + ' WHERE refObjectType=?';
        db.selectByQuery(query, ["User"], table, function (data) {
          var result = [];
          if (data) {
            data.forEach(function(element){
              if (element.refObject && element.refObject.identity_id) {
                result.push(element.refObject.identity_id);
              }
            });
          }
          if (callback != null && typeof callback == "function") {
              callback(result);
          }
        });
      }
    }
    return self;
}(FavoriteController || {}));
