var OverviewController = (function (OverviewController)
{
    var self = OverviewController;

    self.getNumberOfWorkingDays = function(month, year) {
        // Get total number of days in current month
        var days = self.daysInMonth(month + 1, year),
            workingDays = 0;
        for(var i=0; i< days; i++) {
            if (!self.isWeekday(year, month, i+1)) workingDays++;
        }
        return workingDays;
  };

    self.isWeekday = function(year, month, day) {
        // If day is 0 or 6, it is weekend
        var day = new Date(year, month, day).getDay();
        return day === 0 || day === 6;
  };

    self.daysInMonth = function(month,year) {
        return new Date(year, month, 0).getDate();
  };

  self.findMyEfforts = function(currentUserId, startDate, callback) {
    var effortsQuery = [
      {$match:  { type: "Effort" }
      },{$unwind: "$efforts"},
      { $match: { 'efforts.submitter_id': currentUserId, 'efforts.effortDate': { $gte: startDate}} },
      { $group: { _id: "0",
          totalValue: {$sum: '$efforts.hours'},
          immediateParent: { $first: "$immediateParent" }
      }}];

    ClientAPI.findEfforts({
        query: {pipeline: effortsQuery},
        relatedType: 'Task',
        aggregated: true
    }, function(err, data){
      if (callback != null && typeof callback == "function") {
          callback(err, data);
      }
    });
  }

  self.findNumberOfClosedTaskPerProject = function(callback) {
    var query = [
      {$match:  {objectType: "Task", rootObjectType: "Project",
      status: { $in: ["Completed", "Canceled", 'Verified']}}},
      { $group: { _id: "$rootObject", count: { $sum: 1 } }}
    ];
    ClientAPI.aggregate(query, function(err, data){
      if (callback != null && typeof callback == "function") {
          callback(err, data);
      }
    });
  };

  self.findAmountPerOpenOpportunities = function(callback) {
      self.findAmountQuery(
        ['Pending', 'Prospecting', 'Analyses', 'Negotiation', 'ProposalSent'],
        callback
      );
  };

  self.findAmountPerWonOpportunities = function(callback) {
      self.findAmountQuery(['Won'], callback);
  };

  self.findAmountQuery = function(status, callback) {
    var query = {objectType: 'Deal', status: {$in: status}};
    ClientAPI.find(query, [], function(err, openDeals) {
        if (!err) {
          var revenues = [];
          if (openDeals && openDeals.length > 0) {
            for (var i = 0, max = openDeals.length; i < max; i++){
              revenues.push(openDeals[i]['value']);
            }
          }
          if (callback != null && typeof callback == "function") {
               callback(err, revenues);
          }
          return; 
        }
        if (callback != null && typeof callback == "function") {
            callback(err, data);
        }
    });
  };


  self.topSales = function(start, end, callback) {
        if (start != null && end != null) {
            var query = [{
                $match: {
                    objectType: 'Deal',
                    status: 'Won',
                    creationDate: {$gte: start, $lt: end}
                }},
                {$group: {_id: {"owner": "$owner"}, count: {$sum: 1}}},
                {$sort: {'count':-1}},
                {$limit: 5}
            ];
        } else {
            var query = [{
                $match: {
                    objectType: 'Deal',
                    status: 'Won'
                }},
                {$group: {_id: {"owner": "$owner"}, count: {$sum: 1}}},
                {$sort: {'count':-1}},
                {$limit: 5}
            ];
        }

        ClientAPI.aggregate(query, function(err, data) {
          if (!err) {
              var result = [];
              for (var i = 0, max = data.length; i < max; i++) {
                var resultItem = data[i];
                resultItem['user'] = ALL_USERS[data[i]._id.owner];
                result.push(resultItem);
              }
              if (callback != null && typeof callback == "function") {
                  callback(err, result);
              }
              return;
          }
          if (callback != null && typeof callback == "function") {
              callback(err, data);
          }
        });
  };

  self.topClients = function (start, end, callback) {
      if (start != null && end != null) {
          var query = [{
              $match: {
                  objectType: 'Deal',
                  status: 'Won',
                  creationDate: {$gte: start, $lt: end},
                  immediateParentObjectType: {$in: ['Person', 'Company']},
              }
          },
              {$group: {_id: {parent: '$immediateParent'}, count: {$sum: 1}}},
              {$project: {parent: "$_id.parent", count: 1, _id: 0}},
              {$sort: {'count': -1}},
              {$limit: 5}
          ];
      } else {
          var query = [{
              $match: {
                  objectType: 'Deal',
                  status: 'Won',
                  immediateParentObjectType: {$in: ['Person', 'Company']},
              }
          },
              {$group: {_id: {parent: '$immediateParent'}, count: {$sum: 1}}},
              {$project: {parent: "$_id.parent", count: 1, _id: 0}},
              {$sort: {'count': -1}},
              {$limit: 5}
          ];
      }

      ClientAPI.aggregate(query, function (err, data) {
        if (!err) {
            var persons = [];
            for (var i = 0, max = data.length; i < max; i++) {
                persons.push(new ObjectId(data[i].parent));
            }
            ClientAPI.find({objectType:"Person", _id: {$in: persons}},[], function(err, personData) {
              var result = [];
              var collectionPerson = {};
              for (var i = 0, max = personData.length; i < max; i++) {
                var itemData = {};
                if (personData[i].objectType === "Person") {
                    itemData['name'] = personData[i].firstName + " " + personData[i].lastName;
                } else if (personData[i].objectType === "Company") {
                    itemData['name'] = personData[i].companyName;
                }
                if (personData[i]['picture'] && personData[i]['picture']['file_name']) {
                    itemData['avatar-url'] = personData[i]['picture']['file_name'];
                }
                collectionPerson[personData[i]._id] = itemData;
              }
              for (var i = 0, max = data.length; i < max; i++) {
                  var item = data[i];
                  item['user'] = collectionPerson[data[i].parent];
                  result.push(item);
              }
              if (callback != null && typeof callback == "function") {
                  callback(err, result);
              }
            });
        } else {
          if (callback != null && typeof callback == "function") {
              callback(err, data);
          }
        }
      });
  };

  self.collectDataForDealProject = function(start, end, types, callback) {
      if (start != null && end != null) {
          var query = [{$match:
          {objectType: {$in: types}, creationDate: {$gte: start, $lt: end}}},
              {$group: {_id: {"status": "$status"},
                  sum: {$sum: 1}}}, {$sort: {_id: -1}}
          ];
      } else {
          var query = [
              {$match: {objectType: {$in: types}}},
              {$group: {_id: {"status": "$status"},
                  sum: {$sum: 1}}}, {$sort: {_id: -1}}
          ];
      }
      ClientAPI.aggregate(query, function(err, data) {
          if (!err && data && data.length > 0) {
            var dealTotal = 0, projectTotal = 0, won = 0,
                lost = 0, activeDeals = 0, closedProject = 0,
                activeProjects = 0 , createdProjects = 0;
            var dealData = [], projectData = [];

            for (var i = 0, max = data.length; i < max; i++) {
                if ("Won" === data[i]['_id']['status']) {
                    won += data[i].sum;
                } else if ("Lost" === data[i]['_id']['status']) {
                    lost += data[i].sum;
                } else if ("Closed" === data[i]['_id']['status']) {
                    closedProject += data[i].sum;
                } else if ("InProgress" === data[i]['_id']['status']) {
                    activeProjects += data[i].sum;
                } else if ("Created" === data[i]['_id']['status']) {
                    createdProjects += data[i].sum;
                } else {
                    activeDeals += data[i].sum;
                }
            }

            dealTotal = won + lost + activeDeals;
            projectTotal = activeProjects + closedProject + createdProjects;

            if (dealTotal > 0) {
                dealData = [{
                    data: won,
                    color: '#364BB0',
                    label: 'Won'
                }, {
                    data: lost,
                    color: '#AC0015',
                    label: 'Lost'
                }, {
                    data: activeDeals,
                    color: '#31484F',
                    label: 'Active'
                }];
            }

            if (projectTotal > 0) {
                projectData = [{
                    data: activeProjects,
                    color: '#31484F',
                    label: 'Active'
                }, {
                    data: closedProject,
                    color: '#AC0015',
                    label: 'Closed'
                }, {
                    data: createdProjects,
                    color: '#33499E',
                    label: 'Created'
                }];
            }

            var result = {
                dealStats: {
                    dealData: dealData,
                    won: won,
                    lost: lost,
                    active: activeDeals,
                    total: dealTotal
                },
                projectStats: {
                    projectData: projectData,
                    active: activeProjects,
                    created: createdProjects,
                    closed: closedProject,
                    total: projectTotal
                }
            };
            if (callback != null && typeof callback == "function") {
                callback(null, result);
            }
            return;
          }
          if (callback != null && typeof callback == "function") {
              callback(err, data);
          }
      });
    };
    return self;
}(OverviewController || {}));
