var PeopleController = (function (PeopleController)
{
    var self = PeopleController;
    var table = models.peopleDBModel.name;
    var objectType = {objectType: "Person", identity_id: {$exists: false}};
    var i, max;

    self.synchronization = function (items, callback) {
        if (items != null && items.length != null && items.length > 0) {
            self.encode(items, function (results) {
                var keys = [];
                for (i = 0, max = results.length; i < max; i++) {
                    keys.push(results[i].id);
                }
                if (keys.length > 0) {
                    db.deleteFromTableByIds(table, 'id', keys, function () {
                        db.insertMultiple(table, [results], function () {
                            if (callback != null && typeof callback == "function") {
                                callback(results);
                            }
                        });
                    });
                } else {
                    if (callback != null && typeof callback == "function") {
                        callback([]);
                    }
                }
            });
        } else {
            if (callback != null && typeof callback == "function") {
                callback([]);
            }
        }
    };

    self.syncPartData = function (err, data, callback) {
        if (err == null) {
            if (data != null && data.length > 0) {
                var deals = [];
                var keys = [];

                for (var i = 0, max = data.length; i < max; i++) {
                    //check if encoded before
                    if (data[i].id) {
                        var deal = data[i];
                        keys.push(deal.id);
                        deals.push(deal);
                    } else {
                        var deal = self.encodeItem(data[i]);
                        keys.push(deal.id);
                        deals.push(deal);
                    }
                }

                if (keys.length != 0) {
                    db.deleteFromTableByIds(table, 'id', keys, function () {
                        db.insertMultiple(table, [deals], function () {
                            if (callback != null && typeof callback == "function") {
                                callback(deals);
                            }
                        });
                    });
                } else {
                    if (callback != null && typeof callback == "function") {
                        callback([]);
                    }
                }
            } else {
                if (callback != null && typeof callback == "function") {
                    callback([]);
                }
            }
        } else {
            if (callback != null && typeof callback == "function") {
                callback([]);
            }
        }
    };

    self.encode = function (items, callback) {
        var resultItem, result = [];
        if (items != null) {
            if (items.length != null && items.length > 0) {
                $(items).each(function (index, item) {
                    resultItem = self.encodeItem(item);
                    if (resultItem != null) {
                        result.push(resultItem);
                    }
                });
                if (callback != null && typeof callback == "function") {
                    callback(result);
                }
            } else {
                resultItem = self.encodeItem(items);
                if (resultItem != null) {
                    result.push(resultItem);
                }
                if (callback != null && typeof callback == "function") {
                    callback(result);
                }
            }
        }
    };

    self.encodeItem = function (item, companyName) {
        if (item['_id'] != null) {
            var result = {};
            result['id'] = getTextTypeObject(item['_id'], 'integer');

            if (item['firstName']) {
                result['first-name'] = item['firstName'].escape();
            } else {
                result['first-name'] = "";
            }

            if (item['lastName']) {
                result['last-name'] = item['lastName'].escape();
            } else {
                result['last-name'] = "";
            }

            result['objectType'] = item['objectType'];
            result['searchKeywords'] = item['searchKeywords'];
            result['identity_id'] = item['identity_id'];
            if (item.companies != null && item.companies.length > 0) {
                if (item.companies[0].company) {
                    result['company-id'] = getTextTypeObject(item['companies'][0]['company']['_id'], 'integer');
                } else {
                    result['company-id'] = {"type": "integer", "nil": "true"};
                }

                if (item.companies[0].company &&item['companies'][0].company.companyName) {
                    result['company-name'] = item['companies'][0].company.companyName.escape();
                } else {
                    result['company-name'] = "";
                }
                if (item['companies'][0]['position']) {
                    result['title'] = item['companies'][0]['position'].escape();
                } else {
                    result['title'] = "";
                }
            } else {
                result['company-id'] = {"type": "integer", "nil": "true"};
                result['company-name'] = "";
                result['title'] = "";
            }

            if (item.companies && item.companies.length > 0 && companyName && companyName != null) {
                result['company-id'] = getTextTypeObject(item.companies[0].company);
                result['company-name'] = companyName;
                if (item['companies'][0]['position']) {
                    result['title'] = item['companies'][0]['position'].escape();
                }
            }
            if (item['info'] != null) {
                result['background'] = item['info'].escape();
            } else {
                result['background'] = {"nil": "true"};
            }

            result['linkedin-url'] = {"nil": "true"};

            result['created-at'] = getTextTypeObject(item['creationDate'], 'datetime');
            result['updated-at'] = getTextTypeObject(item['modificationDate'], 'datetime');

            if (item['referenceId']) {
                result['referenceId'] = item['referenceId'].escape();
            } else {
                result['referenceId'] = "";
            }


            if (item['owner'] != null) {
                result['owner-id'] = getTextTypeObject(item['owner'], 'integer');
            } else {
                result['owner-id'] = {"type": "integer", "nil": "true"};
            }

            result['author-id'] = getTextTypeObject(item['creator'], 'integer');
            result['subject_datas'] = "";
            result['tags'] = "";
            result['visible-to'] = item.accessList;

            if (item['picture'] != null && item['picture']['file_name'] != null) {
                result['avatar-url'] = item['picture']['file_name'];
            } else {
//				result['avatar-url'] 		= 'http://dge9rmgqjs8m1.cloudfront.net/highrise/missing/avatar.gif?r=3';
                result['avatar-url'] = null;
            }
            result['group-id'] = {"type": "integer", "nil": "true"};
            result['group'] = "";

            var phones = [];
            var emails = [];
            var webAddress = [];

            if (item.phoneNumbers.length > 0) {
                if (item.phoneNumbers.length == 1) {
                    phones = {
                        "id": getTextTypeObject(UtilController.randomNumber(), 'integer'),
                        "number": item.phoneNumbers[0].phoneNumber,
                        "location": item.phoneNumbers[0].numberType
                    };
                } else {
                    for (i = 0, max = item.phoneNumbers.length; i < max; i++) {
                        phones.push({
                            "id": getTextTypeObject(UtilController.randomNumber(), 'integer'),
                            "number": item.phoneNumbers[i].phoneNumber,
                            "location": item.phoneNumbers[i].numberType
                        });
                    }
                }
            }

            if (item.eMailAddresses.length > 0) {
                if (item.eMailAddresses.length == 1) {
                    emails = {
                        "id": getTextTypeObject(UtilController.randomNumber(), 'integer'),
                        "address": item.eMailAddresses[0].eMailAddress,
                        "location": item.eMailAddresses[0].eMailAddressType
                    };
                } else {
                    for (i = 0, max = item.eMailAddresses.length; i < max; i++) {
                        emails.push({
                            "id": getTextTypeObject(UtilController.randomNumber(), 'integer'),
                            "address": item.eMailAddresses[i].eMailAddress,
                            "location": item.eMailAddresses[i].eMailAddressType
                        });
                    }
                }
            }

            if (item.webAddresses.length > 0) {
                if (item.webAddresses.length == 1) {
                    webAddress = {
                        "id": getTextTypeObject(UtilController.randomNumber(), 'integer'),
                        "URL": item.webAddresses[0].URL,
                        "urlType": item.webAddresses[0].urlType
                    };
                } else {
                    for (i = 0, max = item.webAddresses.length; i < max; i++) {
                        webAddress.push({
                            "id": getTextTypeObject(UtilController.randomNumber(), 'integer'),
                            "URL": item.webAddresses[i].URL,
                            "urlType": item.webAddresses[i].urlType
                        });
                    }
                }
            }

            var tags = [];
            if (item['tags'].length > 0) {
                if (item['tags'].length == 1) {
                    tags = {
                        'id': getTextTypeObject(UtilController.randomNumber(), 'integer'),
                        'name': item['tags'][0].tag
                    };
                } else {
                    for (i = 0, max = item.tags.length; i < max; i++) {
                        tags.push({
                            'id': getTextTypeObject(UtilController.randomNumber(), 'integer'),
                            'name': item['tags'][i].tag
                        });
                    }
                }
            }

            result['tags'] = {"type": "array", "tag": tags};

            result['contact-data'] = {
                "instant-messengers": {"type": "array"},
                "twitter-accounts": {"type": "array"},
                "email-addresses": {"type": "array", "email-address": emails},
                "phone-numbers": {"type": "array", "phone-number": phones},
                "addresses": {"type": "array"},
                "web-addresses": {"type": "array", "web-address": webAddress}
            };

            return result;
        } else {
            return undefined;
        }
    };

    self.save = function (firstName, lastName, companyId, title, phones, emails, webs, info, referenceId, visible, companyName, callback) {
        var person = {"objectType": "Person"};

        if (firstName) {
            person['firstName'] = firstName.escape();
        }
        if (lastName) {
            person['lastName'] = lastName.escape();
        }
        if (info) {
            person['info'] = info.escape();
        } else {
            person['info'] = "";
        }
        if (referenceId) {
            person['referenceId'] = referenceId.escape();
        }
        person['accessList'] = visible;

        if (emails.length !== 0) {
            person['eMailAddresses'] = emails;
        }

        if (phones.length !== 0) {
            person['phoneNumbers'] = phones;
        }

        if (webs != null && webs.length > 0) {
            for (i = 0, max = webs.length; i < max; i++) {
                if (!webs[i].URL.startsWith('http')) {
                    webs[i].URL = 'http://' + webs[i].URL;
                }
            }
        }

        if (webs.length !== 0) {
            person['webAddresses'] = webs;
        }

        if (companyId != null && companyId != "none") {
            person['companies'] = [{company: new ObjectId(companyId), position: title}];
        }

        person['status'] = "Active";
        console.log(JSON.stringify(person));
        if (connection) {
            ClientAPI.insertObject(person, function (err, res) {
                if (err) {
                    if (callback != null && typeof callback == "function") {
                        callback();
                    }
                } else {
                    var savedPerson = self.encodeItem(res.object, companyName);
                    db.insert(table, [savedPerson], function () {
                        if (res != null && callback != null && typeof callback == "function") {
                            // workaround sync for sip users
                                OfflineController.synchronization(function(){
                                    SyncController.syncPeople(function(){
                                        callback(savedPerson);
                                    });
                                });
                        }
                    });
                }
            });
        } else {
            var objectContact = {'object': person, 'objectType': table, id: UtilController.randomNumber()};
            db.insert(models.offlineInsertDBModel.name, [objectContact], function () {
                alert(Messages.Offline.offlineMode);
                if (callback != null && typeof callback == "function") {
//                    // workaround sync for sip users
//                    OfflineController.synchronization(function () {
//                        SyncController.sync(function () {
//                            callback();
//                        });
//                    });
                    callback();
                }
            });
        }
    };

    self.update = function (id, firstName, lastName, companyId, title, phones, emails, webs, info, referenceId, visibleTo, companyName, callback) {
        if (connection) {
            if (id) {
                self.findById(id, function (data) {
                    if (data != null && data[0] != null) {
                        var person = data[0];

                        if (firstName) {
                            person['firstName'] = firstName.escape();
                        }
                        if (lastName) {
                            person['lastName'] = lastName.escape();
                        }
                        if (info) {
                            person['info'] = info.escape();
                        } else {
                            person['info'] = "";
                        }

                        if (referenceId) {
                            person['referenceId'] = referenceId.escape();
                        } else {
                            person['referenceId'] = "";
                        }

                        if (emails != null && emails.length > 0) {
                            person['eMailAddresses'] = emails;
                        } else {
                            person['eMailAddresses'] = null;
                        }

                        if (phones != null && phones.length > 0) {
                            person['phoneNumbers'] = phones;
                        } else {
                            person['phoneNumbers'] = null;
                        }

                        if (webs != null && webs.length > 0) {
                            for (i = 0, max = webs.length; i < max; i++) {
                                if (!webs[i].URL.startsWith('http')) {
                                    webs[i].URL = 'http://' + webs[i].URL;
                                }
                            }
                            person['webAddresses'] = webs;
                        } else {
                            person['webAddresses'] = null;
                        }

                        if (companyId != null && companyId != "none") {
                            person['companies'] = [{company: new ObjectId(companyId), position: title}];
                        } else {
                            person['companies'] = null;
                        }

                        person['status'] = "Active";
                        ClientAPI.updateObject(person, function (err, data) {
                            if (err) {
                                if (callback != null && typeof callback == "function") {
                                    callback();
                                }
                            } else {
                                if (data.object) {
                                    var savedPerson = self.encodeItem(data.object, companyName);
                                    if (visibleTo != null) {
                                        ClientAPI.updateACL(data.object._id, visibleTo, function (err, data) {
                                            if (!err) {
                                                PeopleController.findById(id, function (data) {
                                                    savedPerson = PeopleController.encodeItem(data[0]);
                                                    db.deleteObject(table, "id", getTextTypeObject(id, 'integer'), function () {
                                                        db.insert(table, [savedPerson], function () {
                                                            if (data != null && callback != null && typeof callback == "function") {
                                                                callback(savedPerson);
                                                            }
                                                        });
                                                    });
                                                });
                                            } else {
                                                db.deleteObject(table, "id", getTextTypeObject(id, 'integer'), function () {
                                                    db.insert(table, [savedPerson], function () {
                                                        if (data != null && callback != null && typeof callback == "function") {
                                                            callback(savedPerson);
                                                        }
                                                    });
                                                });
                                            }
                                        });
                                    } else {
                                        PeopleController.findById(id, function (data) {
                                            savedPerson = PeopleController.encodeItem(data[0]);
                                            db.deleteObject(table, "id", getTextTypeObject(id, 'integer'), function () {
                                                db.insert(table, [savedPerson], function () {
                                                    if (data != null && callback != null && typeof callback == "function") {
                                                        callback(savedPerson);
                                                    }
                                                });
                                            });
                                        });
                                    }

                                } else {
                                    alert(data['statusDesc']);
                                    if (callback != null && typeof callback == 'function') {
                                        callback();
                                    }
                                }

                            }
                        });
                    } else {
                        if (callback != null && typeof callback == "function") {
                            callback();
                        }
                    }
                });
            } else {
                if (callback != null && typeof callback == "function") {
                    callback();
                }
            }
        } else {
            alert("You are in offline mode. You can't save this person!");
        }
    };

    self.findAll = function (callback) {
        if (connection) {
            ClientAPI.find(objectType, [["modificationDate", -1]], function (err, data) {
                if (err == null) {
                    self.synchronization(data, function (items) {
                        if (callback != null && typeof callback == "function") {
                            callback(items);
                        }
                    });
                } else {
                    if (callback != null && typeof callback == "function") {
                        callback();
                    }
                }
            });
        } else {
            db.selectAll(table, function (data) {
                if (callback != null && typeof callback == "function") {
                    callback(data);
                }
            });
        }
    };

    self.findAllWithIdentity = function (callback) {
        if (connection) {
            ClientAPI.find({objectType: "Person", identity_id: {$exists: true}}, [["modificationDate", -1]], function (err, data) {
                if (err == null) {
                    self.synchronization(data, function (items) {
                        if (callback != null && typeof callback == "function") {
                            callback(items);
                        }
                    });
                } else {
                    if (callback != null && typeof callback == "function") {
                        callback();
                    }
                }
            });
        } else {
            db.selectAll(table, function (data) {
                if (callback != null && typeof callback == "function") {
                    callback(data);
                }
            });
        }
    };

    self.findById = function (id, callback, ignoreIdentity) {
        if (id == null) {
            if (callback != null && typeof callback == "function") {
                callback();
            }
        }

        if (connection) {
            var request = {};
            if (ignoreIdentity) {
                request = {objectType: "Person", _id: new ObjectId(id)};
            } else {
                request = {objectType: "Person", _id: new ObjectId(id), identity_id: {$exists: false}};
            }
            ClientAPI.find({query: request, objectType: 'Person', population: 'companies.company'}, function (err, data) {
                if (err == null && typeof callback == "function") {
                    callback(data);
                } else {
                    if (typeof callback == "function") {
                        callback();
                    }
                }
            });
        } else {
                db.selectByQuery('SELECT * FROM ' + table + ' WHERE id=?',
                        [getTextTypeObject(id, 'integer')],
                        table,
                        function (data) {
                            if (callback != null && typeof callback == "function") {
                                callback(data);
                            }
                        });
        }
    };

    self.delete = function (id, callback) {
        if (connection) {
            ClientAPI.removeObject(id, function (err, data) {
                if (data != null && data.statusCode != null) {
                    if (callback != null && typeof callback == "function") {
                        callback(data.statusCode);
                    } else {
                        callback(-1);
                    }
                } else {
                    callback(-1);
                }
            });
        }
    };

    return self;
}(PeopleController || {}));
