var TaskController = (function (TaskController)
{
    var self = TaskController;
    var table = models.taskDBModel.name;
    var objectType = "Task";
    var countRequest;

    self.synchronization = function (items, callback) {
        if (items != null && items.length != null && items.length > 0) {
            self.encode(items, function (results) {
                var keys = [];
                for (var i = 0, max = results.length; i < max; i++) {
                    keys.push(results[i].id);
                }
                if (keys.length != 0) {
                    db.deleteFromTableByIds(table, 'id', keys, function () {
                        db.insertMultiple(table, [results], function (data) {
                            if (callback != null && typeof callback == "function") {
                                callback(results);
                            }
                        });
                    });
                } else {
                    if (callback != null && typeof callback == "function") {
                        callback([]);
                    }
                }
            });
        } else {
            if (callback != null && typeof callback == "function") {
                callback([]);
            }
        }
    };

    self.syncPartData = function (err, data, callback) {
        if (err == null) {
            if (data != null && data.length > 0) {
                var tasks = [];
                var keys = [];
                var result = [];
                for (var i = 0, max = data.length; i < max; i++) {
                    var task = self.encodeItem(data[i]);
                    if (data[i]['status'] != 'Completed') {
                        keys.push(task.id);
                        tasks.push(task);
                    }
                    result.push(task);
                }

                if (keys.length != 0) {
                    db.deleteFromTableByIds(table, 'id', keys, function () {
                        db.insertMultiple(table, [tasks], function () {
                            if (callback != null && typeof callback == "function") {
                                callback(result);
                            }
                        });
                    });
                } else {

                    if (result.length > 0) {
                        if (callback != null && typeof callback == "function") {
                            callback(result);
                        }
                    } else {
                        if (callback != null && typeof callback == "function") {
                            callback([]);
                        }
                    }
                }
            } else {
                if (callback != null && typeof callback == "function") {
                    callback([]);
                }
            }
        } else {
            if (callback != null && typeof callback == "function") {
                callback([]);
            }
        }
    };

    self.encode = function (items, callback) {
        var result = [];
        var resultItem = {};
        if (items != null) {
            if (items.length != null && items.length > 0) {
                for (var i = 0, max = items.length; i < max; i++) {
                    if (items[i]['status'] != "Completed") {
                        resultItem = self.encodeItem(items[i]);
                        if (resultItem != null) {
                            result.push(resultItem);
                        }
                    }
                }
                if (callback != null && typeof callback == "function") {
                    callback(result);
                }
            } else {
                if (items['status'] != "Completed") {
                    resultItem = self.encodeItem(items);
                    if (resultItem != null) {
                        result.push(resultItem);
                    }
                }

                if (callback != null && typeof callback == "function") {
                    callback(result);
                }
            }
        }
    };

    self.calculateDueStatus = function (date) {
        if (date == null) {
            return "Later";
        }

        if (UtilController.compareDate(new Date(date), new Date()) == 0) {
            return "Today";
        }

        if (UtilController.compareDateTime(new Date(date), new Date()) < 0) {
            return "Overdue";
        }

        var dueDate = new Date(date);
        dueDate = new Date(dueDate.getUTCFullYear(), dueDate.getUTCMonth(), dueDate.getDate(), 0, 0, 0, 0);

        var now = new Date();
        var today = new Date(now.getUTCFullYear(), now.getUTCMonth(), now.getDate(), 23, 59, 0, 0);

        var tomorrow = new Date(now.getTime() + (24 * 60 * 60 * 1000));
        tomorrow = new Date(tomorrow.getUTCFullYear(), tomorrow.getUTCMonth(), tomorrow.getDate(), 23, 59, 0, 0);

        var thisWeek = new Date(now.getTime() + (24 * 60 * 60 * 1000 * (6 - now.getDay())));
        thisWeek = new Date(thisWeek.getUTCFullYear(), thisWeek.getUTCMonth(), thisWeek.getDate(), 23, 59, 0, 0);

        var nextWeek = new Date(now.getTime() + (24 * 60 * 60 * 1000 * (13 - now.getDay())));
        nextWeek = new Date(nextWeek.getUTCFullYear(), nextWeek.getUTCMonth(), nextWeek.getDate(), 23, 59, 0, 0);

        if (dueDate.toDateString() == tomorrow.toDateString()) {
            return "Tomorrow";
        }

        if (dueDate < thisWeek) {
            return "This Week";
        }

        if (dueDate < nextWeek) {
            return "Next Week";
        }

        return "Specific";
    };


    self.encodeItem = function (item) {
        if (item['_id'] != null) {
            var result = {};
            result['id'] = getTextTypeObject(item['_id'], 'integer');
            result['created-at'] = getTextTypeObject(item['creationDate'], 'datetime');
            result['updated-at'] = getTextTypeObject(item['modificationDate'], 'datetime');
            result['due-at'] = getTextTypeObject(item['dueDate'], 'datetime');
            result['alert-at'] = getTextTypeObject(item['dueDate'], 'datetime');
            result['body'] = item['taskDescription'];
            result['frame'] = self.calculateDueStatus(item.dueDate);
            result['searchKeywords'] = item['searchKeywords'];
            result['done-at'] = {"type": "datetime", "nil": "true"};
            result['category-id'] = getTextTypeObject(item['taskCategory'], 'integer');
            result['author-id'] = getTextTypeObject(item['creator'], 'integer');
            result['responsible-id'] = getTextTypeObject(item['assignedTo'], 'integer');
            result['owner-id'] = getTextTypeObject(item['owner'], 'integer');
            result['owner'] = "Owner :" + ALL_USERS[item['owner']].name;
            result['status'] = item.status;
            result['entityId'] = item.entityId;
            result['public'] = {"type": "boolean", "text": "false"};
            for (var i = 0, max = item.accessList.length; i < max; i++) {
                if (item.accessList[i].identity == "Everyone@" + localStorage.getItem('userDomain')) {
                    result['public'] = {"type": "boolean", "text": "true"};
                }
            }

            if (item.taskName) {
                result['name'] = item['taskName'].escape();
            } else {
                result['name'] = "";
            }

            if (item.percentCompleted) {
                result['percent-completed'] = item.percentCompleted;
            } else {
                result['percent-completed'] = 0;
            }

            result['subject-type'] = {"nil": "true"};
            result['subject-name'] = {"nil": "true"};
            result['subjectDes'] = "";
            result['subject-id'] = {"type": "integer", "nil": "true"};
            result['recording-id'] = {"type": "integer", "nil": "true"};
            result['visible-to'] = item.accessList;
            result['priority'] = item.taskPriority;

            if (item['immediateParent'] != null) {
                result['recording-id'] = getTextTypeObject(item.immediateParent, 'integer');
                result['recording-type'] = item.immediateParentObjectType;
            }

            if (item['rootObject'] != null) {
                result['subject-id'] = getTextTypeObject(item['rootObject'], 'integer');

                if (item["immediateParentObjectType"] == "Deal") {
                    result['subject-type'] = item.immediateParentObjectType;
                    result['subject-name'] = item.immediateParentDisplay;
                    result['subjectDes'] = item.immediateParentObjectType + " : " + item.immediateParentDisplay;

                } else if (item.immediateParentObjectType == "Person") {

                    result['subject-type'] = "Party";
                    result['subject-name'] = item.immediateParentDisplay;
                    result['subjectDes'] = "person: " + item.immediateParentDisplay;

                } else if (item["immediateParentObjectType"] == "Company") {

                    result['subject-type'] = "Party";
                    result['subject-name'] = item.immediateParentDisplay;
                    result['subjectDes'] = "company: " + item.immediateParentDisplay;

                } else if (item["immediateParentObjectType"] == "Project") {

                    result['subject-type'] = "Kase";
                    result['subject-name'] = item.immediateParentDisplay;
                    result['subjectDes'] = "case: " + item.immediateParentDisplay;
                } else if (item["immediateParentObjectType"] == "Note") {

                    result['subject-type'] = item.immediateParentObjectType;
                    result['subject-name'] = item.immediateParentDisplay;
                    result['subjectDes'] = "Note : " + item.immediateParentDisplay;

                } else if (item["immediateParentObjectType"] == "Task") {

                    result['subject-type'] = item.immediateParentObjectType;
                    result['subject-name'] = item.immediateParentDisplay;
                    result['subjectDes'] = "Task : " + item.immediateParentDisplay;
                }
            }

            if (item.statusHistory) {
                result["statusHistory"] = item.statusHistory;
            }
            if (item.effortSummary) {
                result.efforts = item.effortSummary;
            }

            return result;
        } else {
            return undefined;
        }
    };

    self.countObjectByParentId = function (id, callback) {
        if (connection) {
            var request = {
                objectType: "Task",
                status: {$ne: "Completed"},
                $or: [{immediateParent: new ObjectId(id)}, {rootObject: new ObjectId(id)}]
            };

            ClientAPI.findCount(request, function (err, data) {
                if (callback != null && typeof callback == "function") {
                    callback(data);
                }
            });
        } else {
            if (callback != null && typeof callback == "function") {
                callback([]);
            }
        }

    };

    self.syncTasksByParentId = function (id, onlyImmediate, sort, start, offset, callBack) {
        var request = {};
        if (connection) {
            if (onlyImmediate) {
                request = {
                    immediateParent: new ObjectId(id),
                    objectType: "Task"
                };

                if (sort != null && sort[0] != null && sort[0][1] != null && sort[0][1] == -1) {
                    request["status"] = {$in: ["Completed", "Canceled", "Verified"]};
                } else {
                    request["status"] = {$nin: ["Completed", "Canceled", "Verified"]};
                }

                ClientAPI.findPaged(request, sort, start / offset + 1, offset, function (err, data) {
                    self.syncPartData(err, data, callBack);
                });
            } else {
                request = {
                    objectType: "Task",
                    $or: [{immediateParent: new ObjectId(id)}, {rootObject: new ObjectId(id)}]
                };

                if (sort != null && sort[0] != null && sort[0][1] != null && sort[0][1] == -1) {
                    request["status"] = {$in: ["Completed", "Canceled", "Verified"]};
                } else {
                    request["status"] = {$nin: ["Completed", "Canceled", "Verified"] };
                }
                ClientAPI.findPaged(request, sort, start / offset + 1, offset, function (err, data) {
                    self.syncPartData(err, data, callBack);
                });
            }
        } else {
            if (callBack != null && typeof callBack == "function") {
                callBack([]);
            }
        }
    };

    self.refresh = function (callback) {
        if (connection) {
            ClientAPI.find({objectType: "Task", modificationDate: {$gte: new Date(window.localStorage.getItem('syncDateAll'))}}, null, function (err, data) {
                self.syncPartData(err, data, callback);
            });
        }
    };

    self.deleteTask = function (id, callback) {
        if (connection) {
            ClientAPI.removeObject(id, function (err, data) {
                if (data != null && data.statusCode != null) {
                    if (callback != null && typeof callback == "function") {
                        if (data.statusCode == 0) {
                            db.deleteObject(table, "id", getTextTypeObject(id, 'integer'), function () {
                                callback(data.statusCode);
                            });
                        } else {
                            callback(data.statusCode);
                        }
                    }
                }
            });
        }
    };

    self.completeTask = function (id, callback) {
        if (connection) {
            ClientAPI.executeTransitionalAction('Complete', 'Task', id, null, function (err) {
                if (!err) {
                    self.findById(id, function (data) {
                        self.syncPartData(err, [data], callback);
                    });
                }
            });
        }
    };

    self.save = function (description, category, dueDate, ownerId, assignedTo, priority, dueFrame, status, rootObject,
    visibleTo, name, percent, callback) {
        var task = null;
        if (typeof description == "object") {
            task = description;
        } else {
            task = {
                'objectType': "Task",
                'taskDescription': description,
                'dueFrame': dueFrame,
                'assignedTo': assignedTo,
                'owner': ownerId,
                'taskPriority': priority,
                'status': status
            };

            if (dueDate != null) {
                task['dueDate'] = dueDate;
            }
            if (name && name != "") {
                task.taskName = name;
            }

            if (percent) {
                task.percentCompleted = percent;
            }

            if (category) {
                task.taskCategory = new ObjectId(category);
            }


            task['accessList'] = visibleTo;
            if (connection) {
                if (task != null && typeof task == "object") {
                    if (rootObject == null) {
                        ClientAPI.insertObject(task, function (err, res) {
                            if (err) {
                                if (category != null && typeof category == "function") {
                                    category();
                                } else if (callback != null && typeof callback == "function") {
                                    callback();
                                }
                            } else {


                                var commentSave = self.encodeItem(res.object);
                                db.insert(table, [commentSave], function () {
                                });
                                if (category != null && typeof category == "function") {
                                    category(commentSave);
                                } else if (res != null && typeof callback == "function") {
                                    callback(commentSave);
                                }
                            }
                        });
                    } else {
                        ClientAPI.attachChildObject(task, rootObject, function (err, res) {
                            if (err) {
                                if (category != null && typeof category == "function") {
                                    category();
                                } else if (callback != null && typeof callback == "function") {
                                    callback();
                                }
                            } else {
                                var commentSave = self.encodeItem(res.object);
                                db.insert(table, [commentSave], function () {
                                });
                                if (category != null && typeof category == "function") {
                                    category(commentSave);
                                } else if (res != null && typeof callback == "function") {
                                    callback(commentSave);
                                }
                            }
                        });
                    }
                }
            } else {
                var objectTask = {'object': task, 'objectType': table, id: UtilController.randomNumber(), 'parent-id': rootObject};
                db.insert(models.offlineInsertDBModel.name, [objectTask], function () {
                    alert(Messages.Offline.offlineMode);
                    if (category != null && typeof category == "function") {
                        category();
                    } else if (callback != null && typeof callback == "function") {
                        var taskObj = {};
                        if (task.taskDescription) {
                            taskObj.body = task.taskDescription;
                        }
                        if (task.taskCategory) {
                            taskObj.id = {};
                            taskObj.id.text = task.taskCategory;
                        }

                        if (task.dueDate) {
                            taskObj['due-at'] = {};
                            taskObj['due-at'].text = task.dueDate;
                        }

                        callback(taskObj);
                    }
                });
            }
        }
    };

    self.update = function (id, description, category, dueDate, ownerId, assignedTo, priority, dueFrame, status,
    subjectId, visibleTo, name, percent, callback) {
        if (connection) {
            if (id != null) {
                self.findById(id, function (itemFromServer) {
                    if (itemFromServer != null) {
                        itemFromServer['taskDescription'] = description;
                        itemFromServer['dueFrame'] = dueFrame;
                        itemFromServer['taskPriority'] = priority;
                        itemFromServer['status'] = status;

                        if (dueDate != null) {
                            itemFromServer['dueDate'] = dueDate;
                        }

                        if (dueFrame == "Later") {
                            itemFromServer['dueDate'] = "";
                        }
                        if (name && name != "") {
                            itemFromServer['taskName'] = name;
                        }

                        if (percent) {
                            itemFromServer['percentCompleted'] = percent;
                        }

                        if (category != null) {
                            itemFromServer['taskCategory'] = new ObjectId(category);
                        }

                        if (assignedTo != null) {
                            itemFromServer['assignedTo'] = assignedTo;
                        }
                        if (ownerId != null) {
                            itemFromServer['owner'] = ownerId;
                        }

                        ClientAPI.updateObject(itemFromServer, function (err, data) {
                            if (!err && data.object != null) {
                                countRequest = 0;
                                if (ownerId != data.object.owner) {
                                    countRequest++;
                                    console.log(countRequest + "changeOwner");
                                    ClientAPI.changeOwner(data.object._id, JSON.stringify(ownerId), function () {
                                        self.findById(data.object._id, function (data) {
                                            countRequest--;
                                            console.log(countRequest + "changeOwner");
                                            if (data != null && data.length == null) {
                                                self.syncPartData(null, [data], function (data) {
                                                    if (countRequest == 0 && callback != null && typeof callback == 'function') {
                                                        callback(data);
                                                    }
                                                });
                                            } else {
                                                if (countRequest == 0 && callback != null && typeof callback == 'function') {
                                                    callback([]);
                                                }
                                            }
                                        });
                                    });
                                }

                                if (assignedTo != data.object.assignedTo) {
                                    countRequest++;
                                    console.log(countRequest + "executeTransitionalAction");
                                    ClientAPI.executeTransitionalAction('Assign', 'Task', data.object._id, {assignedTo: JSON.stringify(assignedTo)}, function () {
                                        self.findById(data.object._id, function (data) {
                                            countRequest--;
                                            console.log(countRequest + "executeTransitionalAction");
                                            self.syncPartData(null, [data], function (data) {
                                                if (countRequest == 0 && callback != null && typeof callback == 'function') {
                                                    callback(data);
                                                }
                                            });
                                        });
                                    });
                                }

                                if (visibleTo) {
                                    countRequest++;
                                    console.log(countRequest + "updateACL");
                                    ClientAPI.updateACL(data.object._id, visibleTo, function () {
                                        countRequest--;
                                        console.log(countRequest + "updateACL");
                                        if (data.object['status'] != "Completed") {
                                            countRequest++;
                                            console.log(countRequest + "attachChildObject");
                                            self.findAndUpdateTask(new ObjectId(data.object._id), subjectId, callback);
                                        }
                                        if (countRequest == 0 && callback != null && typeof callback == 'function') {
                                            callback(data);
                                        }
                                    });
                                } else {
//                                if (data.object['status'] != "Completed" && subjectId != null) {
                                    if (data.object['status'] != "Completed") {
                                        countRequest++;
                                        console.log(countRequest + "attachChildObject");
                                        self.findAndUpdateTask(new ObjectId(data.object._id), subjectId, callback);
                                    }
                                }

                                if (countRequest == 0 && callback != null && typeof callback == 'function') {
                                    callback([self.encodeItem(data.object)]);
                                }
                            } else {
                                alert(data['statusDesc']);
                                if (callback != null && typeof callback == 'function') {
                                    callback();
                                }
                            }
                        });
                    } else {
                        if (callback != null && typeof callback == "function") {
                            callback();
                        }
                    }
                });
            } else {
                if (callback != null && typeof callback == "function") {
                    callback();
                }
            }
        } else {
            alert("You are in offline mode. You can't save this!");
            if (callback != null && typeof callback == "function") {
                callback();
            }
        }
    };

    self.updateDueDate = function (id, dueDate, callback) {
        if (connection) {
            if (id != null) {
                self.findById(id, function (taskObject) {
                    if (dueDate != null) {
                        taskObject['dueDate'] = dueDate;
                    }
                    ClientAPI.updateObject(taskObject, function (err, data) {
                        if (!err) {
                            callback(data);
                        }
                    });
                });
            }
        } else {
            alert("You are in offline mode. Unable to update task!");
            if (callback != null && typeof callback == "function") {
                callback();
            }
        }
    };

    self.updatePercentCompletion = function (id, callback) {
        if (connection) {
            if (id != null) {
                self.findById(id, function (taskObject) {
                    taskObject['percentCompleted'] = 100;
                    ClientAPI.updateObject(taskObject, function (err, data) {
                        if (!err) {
                            callback(data);
                        }
                    });
                });
            }
        }
    };

    self.findAndUpdateTask = function (taskId, subjectId, callback) {
        if (connection) {
            ClientAPI.find({_id: taskId}, null, function (err, data) {
                if (!err) {
                    if (subjectId == null) {
                        console.log(countRequest + " - participantId - null");
                        ClientAPI.promoteToRootObject(data[0]['_id'], data[0].objectType, function () {
                            self.findById(data[0]._id, function (data) {
                                countRequest--;
                                console.log(countRequest + "participantId - null");
                                if (data != null && data.length == null) {
                                    self.syncPartData(null, [data], function (data) {
                                        if (countRequest == 0 && callback != null && typeof callback == 'function') {
                                            callback(data);
                                        }
                                    });
                                } else {
                                    if (countRequest == 0 && callback != null && typeof callback == 'function') {
                                        callback(data);
                                    }
                                }
                            });
                        });
                    } else {
                        ClientAPI.attachChildObject(data[0], subjectId, function () {
                            self.findById(data[0]._id, function (data) {
                                countRequest--;
                                console.log(countRequest + "attachChildObject");
                                if (data != null && data.length == null) {
                                    self.syncPartData(null, [data], function (data) {
                                        if (countRequest == 0 && callback != null && typeof callback == 'function') {
                                            callback(data);
                                        }
                                    });
                                } else {
                                    if (countRequest == 0 && callback != null && typeof callback == 'function') {
                                        callback(data);
                                    }
                                }

                            });
                        });
                    }
                }
            });
        }
    };

    self.findAll = function (callback) {
        if (connection) {
            ClientAPI.find({'objectType': objectType}, [["modificationDate", -1]], function (err, data) {
                if (err == null) {
                    self.synchronization(data, function (items) {
                        if (callback != null && typeof callback == "function") {
                            callback(items);
                        }
                    });
                } else {
                    if (callback != null && typeof callback == "function") {
                        callback();
                    }
                }
            });
        } else {
            db.selectAll(table, function (data) {
                if (callback != null && typeof callback == "function") {
                    callback(data);
                }
            });
        }
    };

    self.findById = function (id, callback) {
        if (id == null) {
            console.log('Id is empty');
            if (callback != null && typeof callback == "function") {
                callback();
            }
        }
        if (connection) {
            ClientAPI.find({'objectType': objectType, _id: new ObjectId(id)}, null, function (err, data) {
                if (!err && data != null && data.length > 0) {
                    if (data[0]['status'] != 'Completed') {
                        var task = self.encodeItem(data[0]);
                        if (err == null && callback != null && typeof callback == "function") {
                            db.deleteObject(table, "id", getTextTypeObject(task.id.text, 'integer'), function () {
                                db.insert(table, [task], function () {
                                    callback(data[0]);
                                });
                            });
                        } else {
                            if (callback != null && typeof callback == "function") {
                                callback([]);
                            }
                        }
                    } else {
                        if (callback != null && typeof callback == "function") {
                            callback(data[0]);
                        }
                    }
                } else {
                    if (callback != null && typeof callback == "function") {
                        callback([]);
                    }
                }
            });
        } else {
            db.selectByColumn(table, "id", getTextTypeObject(id, 'integer'), function (data) {
                if (callback != null && typeof callback == "function") {
                    callback(data);
                }
            });
        }
    };

    self.findByObjectId = function (id, callback) {
        if (id == null) {
            if (callback != null && typeof callback == "function") {
                callback();
            }
        }

        if (connection) {
            ClientAPI.find({objectType: "Task", _id: new ObjectId(id)}, null, function (err, data) {
                if (err == null && typeof callback == "function") {
                    if (typeof callback == "function") {
                        callback(data[0]);
                    }
                } else {
                    if (typeof callback == "function") {
                        callback();
                    }
                }
            });
        } else {
            db.selectByColumn(table, "id", getTextTypeObject(id, 'integer'), function (data) {
                if (callback != null && typeof callback == "function") {
                    callback(data[0].task);
                }
            });
        }
    };

    self.getTasksBySubjectIdAndType = function (id, perentType, start, offset, type, isUser, callback) {
        var query, queryData, request = {};
        type = type[0].toUpperCase() + type.slice(1, type.length);
        if (type == 'Assigned' || type == 'Upcoming' || type == 'Completed') {
            if (type == 'Completed') {
                if (connection) {
                    if (isUser) {
                        request = {objectType: "Task", status: {$in: ["Completed", "Canceled", "Verified"]}, assignedTo: id};
                    } else {
                        request = {objectType: "Task", status: {$in: ["Completed", "Canceled", "Verified"]}, immediateParent: new ObjectId(id)};
                    }

                    ClientAPI.findPaged(request, [["dueDate", -1], ["status", -1]], start / offset + 1, offset,
                            function (err, data) {
                                self.syncPartData(err, data, callback);
                            });
                } else {
                    query = {};
                    queryData = [];
                    if (isUser) {
                        query = 'SELECT * FROM ' + models.taskDBModel.name + ' WHERE "responsible-id" = ? AND ("status" = ? OR "status" = ? OR "status" = ?) ORDER BY [due-at] LIMIT ?, ?';
                        queryData = [getTextTypeObject(id, 'integer'), "Completed" , "Canceled", "Verified" ,start, offset];
                    } else {
                        query = 'SELECT * FROM ' + models.taskDBModel.name + ' WHERE "recording-id" =?  AND "subject-type" = ? AND (status = ? OR status = ? OR status = ?) ORDER BY [due-at] LIMIT ?, ?';
                        queryData = [getTextTypeObject(id, 'integer'), perentType, type, "Canceled", "Verified", start, offset];
                    }

                    db.selectByQuery(query, queryData, table, function (data) {
                        if (callback != null && typeof callback == "function") {
                            callback(data);
                        }
                    });
                }
            } else {
                if (connection) {
                    if (isUser) {
                        request = {objectType: "Task", status: {$nin: ["Completed", "Canceled", "Verified"]}, assignedTo: id};
                    } else {
                        request = {objectType: "Task", immediateParent: new ObjectId(id), status: {$nin: ["Completed", "Canceled", "Verified"]}};
                    }
                    ClientAPI.findPaged(request, [["dueDate", 1], ["status", -1]], start / offset + 1, offset, function (err, data) {
                        self.syncPartData(err, data, callback);
                    });
                } else {
                    query = {};
                    queryData = [];
                    if (isUser) {
                        query = 'SELECT * FROM ' + models.taskDBModel.name + ' WHERE "responsible-id" =? AND status != ? AND status != ? AND status != ? ORDER BY [due-at] LIMIT ?, ?';
                        queryData = [getTextTypeObject(id, 'integer'), "Completed", "Canceled", "Verified" ,start, offset];
                    } else {
                        query = 'SELECT * FROM ' + models.taskDBModel.name + ' WHERE "recording-id" =?  AND "subject-type" = ? AND status != ? AND status != ? AND status != ? ORDER BY [due-at] LIMIT ?, ?';
                        queryData = [getTextTypeObject(id, 'integer'), perentType, "Completed", "Canceled", "Verified", start, offset];
                    }

                    db.selectByQuery(query, queryData, table, function (data) {
                        if (callback != null && typeof callback == "function") {
                            callback(data);
                        }
                    });
                }
            }
        } else {
            if (connection) {
                if (isUser) {
                    request = {objectType: "Task", owner: id};
                } else {
                    request = {objectType: "Task", immediateParent: new ObjectId(id)};
                }
                ClientAPI.find(request, [["dueDate", -1], ["status", -1]], start / offset + 1, offset, function (err, data) {
                    self.syncPartData(err, data, callback);
                });
            } else {
                query = {};
                queryData = [];
                if (isUser) {
                    query = 'SELECT * FROM ' + models.taskDBModel.name + ' WHERE "responsible-id" =? ORDER BY [due-at] LIMIT ?, ?';
                    queryData = [getTextTypeObject(id, 'integer'), start, offset];
                } else {
                    query = 'SELECT * FROM ' + models.taskDBModel.name + ' WHERE "subject-id" =? AND "subject-type" = ? ORDER BY [due-at] LIMIT ?, ?';
                    queryData = [getTextTypeObject(id, 'integer'), perentType, start, offset];
                }
                db.selectByQuery(query, queryData, table, function (data) {
                    if (callback != null && typeof callback == "function") {
                        callback(data);
                    }
                });
            }
        }
    };

    self.findAllFavoriteTask = function(callback) {
      FavoriteController.findByType("Task", function(data) {
          var favorites = [];
          for (var i = 0; i < data.length; i++) {
              favorites.push(data[i].refObject);
          }
          if (connection) {
               self.syncPartData(null, favorites, callback);
          } else {
              if (callback != null && typeof callback == "function") {
                  callback(favorites);
              }
          }
      });
    }

    self.findUpcomingTasks = function(callback){
      if (connection) {
          ClientAPI.findPaged({objectType: "Task", status: {$nin: ["Completed", "Canceled", "Verified"]}},
            [["dueDate", 1]], 1, offset, function (err, items) {
            if (!err) {
                var tasks = [];
                for (var i = 0, max = items.length; i < max; i++) {
                    tasks.push(TaskController.encodeItem(items[i]));
                }
                if (callback != null && typeof callback == "function") {
                    callback(tasks);
                }
            } else {
              if (callback != null && typeof callback == "function") {
                  callback([]);
              }
            }
          });
      } else {
          db.selectByQuery('SELECT * FROM TASK WHERE status = ? OR status = ? OR status = ? ORDER BY [due-at] LIMIT ?, ?',
            ["Created", "Assigned", "InProgress", 0, offset], 'task', function (items) {
              if (callback != null && typeof callback == "function") {
                  callback(items);
              }
          });
      }
    };

    self.findAssignedTasks = function(callback) {
      if (connection) {
          var request = {objectType: "Task"};
          request.$and = [{owner: window.localStorage.getItem('userId')}, {assignedTo: {$ne: window.localStorage.getItem('userId')}}, {assignedTo: {$type: 2}}];
          ClientAPI.findPaged(request, [["dueDate", 1]], 1, offset, function (err, items) {
            var tasks = [];
            if (!err) {
                for (var i = 0, max = items.length; i < max; i++) {
                    tasks.push(TaskController.encodeItem(items[i]));
                }
            }

            if (callback != null && typeof callback == "function") {
                callback(tasks);
            }
          });
      } else {
          db.selectByQuery('SELECT * FROM TASK WHERE status =? AND "owner-id" = ? AND "responsible-id" != ? ORDER BY [due-at] LIMIT ?, ?',
          ["Assigned", getTextTypeObject(window.localStorage.getItem('userId'), 'integer'), getTextTypeObject(window.localStorage.getItem('userId'), 'integer'), 0, offset], 'task', function (items) {
              if (callback != null && typeof callback == "function") {
                  callback(items);
              }
          });
      }
    };

    self.findComplitedTask = function(callback) {
      if (connection) {
        ClientAPI.findPaged({objectType: "Task", status: {$in:["Completed", "Canceled", "Verified"]}}, [["dueDate", -1]], 1, offset, function (err, items) {
            var tasks = [];
            if (!err) {
                for (var i = 0, max = items.length; i < max; i++) {
                    tasks.push(TaskController.encodeItem(items[i]));
                }
            }
            if (callback != null && typeof callback == "function") {
                callback(tasks);
            }
        });
      } else {
        if (callback != null && typeof callback == "function") {
            callback([]);
        }
      }
    };

    self.handleResult = function(err, data){
      var tasks = [];
      if (!err) {
          for (var i = 0, max = items.length; i < max; i++) {
              tasks.push(TaskController.encodeItem(items[i]));
          }
      }
      if (callback != null && typeof callback == "function") {
          callback(tasks);
      }
    };

    self.findMyComplitedTask = function(callback) {
      if (connection) {
        var request = {objectType: "Task"};
        request.$or = [{assignedTo: window.localStorage.getItem('userId')}, {
                assignedTo: {$exists: false},
                owner: window.localStorage.getItem('userId')
            }, {assignedTo: {$type: 10}, owner: window.localStorage.getItem('userId')}];
        request.status = {$in: ["Completed", "Canceled", "Verified"]};

        ClientAPI.findPaged(request, [["dueDate", -1]], 1, offset, function (err, items) {
            var tasks = [];
            if (!err) {
                for (var i = 0, max = items.length; i < max; i++) {
                    tasks.push(TaskController.encodeItem(items[i]));
                }
            }
            if (callback != null && typeof callback == "function") {
                callback(tasks);
            }
        });
      } else {
        db.selectByQuery('SELECT * FROM TASK WHERE status =? AND "owner-id" = ? ORDER BY [due-at] LIMIT ?, ?',
        ["Completed", getTextTypeObject(window.localStorage.getItem('userId'), 'integer'), 0, offset], 'task', function (items) {
          if (callback != null && typeof callback == "function") {
              callback(items);
          }
        });
      }
    };


    self.findMyUpcomingTask = function(callback){
      if (connection) {
          var request = {objectType: "Task"};
          request.$or = [{assignedTo: window.localStorage.getItem('userId')}, {
                  assignedTo: {$exists: false},
                  owner: window.localStorage.getItem('userId')
              }, {assignedTo: {$type: 10}, owner: window.localStorage.getItem('userId')}];
          request.status = {$nin: ["Completed", "Canceled", "Verified"]};

          ClientAPI.findPaged(request, [["dueDate", 1]], 1, offset, function (err, items) {
            var tasks = [];
            if (!err) {
                for (var i = 0, max = items.length; i < max; i++) {
                    tasks.push(TaskController.encodeItem(items[i]));
                }
            }
            if (callback != null && typeof callback == "function") {
                callback(tasks);
            }
          });
      } else {
          db.selectByQuery('SELECT * FROM TASK WHERE (status = ? OR status = ?)  AND "owner-id" = ? ORDER BY [due-at] LIMIT ?, ?', ["Created", "InProgress", getTextTypeObject(window.localStorage.getItem('userId'), 'integer'), 0, offset], 'task', function (items) {
            if (callback != null && typeof callback == "function") {
                callback(items);
            }
          });
      }
    };

    self.findMyUpcomingTaskForParentObject = function(parent, callback) {
        if (!parent) {
          if (callback != null && typeof callback == "function") {
              callback([]);
              return;
          }
        }
        if (connection) {
            var myTasksFilter = [{assignedTo: window.localStorage.getItem('userId')}, {
                    assignedTo: {$exists: false},
                    owner: window.localStorage.getItem('userId')
                }, {assignedTo: {$type: 10}, owner: window.localStorage.getItem('userId')}];
            var completedTasks = ["Completed", "Canceled", "Verified"];
            var request = {objectType: "Task", immediateParent: new ObjectId(parent.id.text)};
            request.$or = myTasksFilter;
            request.status = {$nin: completedTasks};

            ClientAPI.findPaged(request, [["dueDate", 1]], 1, offset, function (err, items) {
                var tasks = [];
                if (!err) {
                    for (var i = 0; i < items.length; i++) {
                        tasks.push(TaskController.encodeItem(items[i]));
                    }
                }
                if (callback != null && typeof callback == "function") {
                    callback(tasks);
                }
            });

        } else {
            var query = 'SELECT * FROM ' + models.taskDBModel.name + ' WHERE "subject-type" =? AND "subject-name"= ? AND "recording-id" = ? AND "responsible-id" = ? AND status != ? AND status != ? AND status != ?';
            var queryData = [parent.objectType, parent.name, getTextTypeObject(parent.id.text, 'integer'), getTextTypeObject(window.localStorage.getItem('userId'), 'integer'), "Completed", "Canceled", "Verified"];
            db.selectByQuery(query, queryData, 'task', function (items) {
              if (callback != null && typeof callback == "function") {
                  callback(items);
              }
            });
        }
    };

    self.findCustomTask = function(filterQuery, callback) {
      if (connection){
        if(filterQuery[0].$match.immediateParent){
            filterQuery[0].$match.immediateParent = new ObjectId(filterQuery[0].$match.immediateParent.val);
        }
        ClientAPI.aggregate(filterQuery, function(err, items){
          var tasks = [];
          if (!err) {
              for (var i = 0, max = items.length; i < max; i++) {
                  tasks.push(TaskController.encodeItem(items[i]));
              }
          }
          if (callback != null && typeof callback == "function") {
              callback(tasks);
          }
      	});
      } else {
        if (callback != null && typeof callback == "function") {
            callback([]);
        }
      }
    };

    self.findAllFilters = function(callback) {
        if (connection) {
          ClientAPI.findSettings({query:{settingType: 'CustomFilter',
          creator: window.localStorage.getItem('userId'), refObjectType: 'Task' }}, function(err, data){
            if (callback != null && typeof callback == "function") {
                callback(data);
            }
          });
        } else {
          if (callback != null && typeof callback == "function") {
              callback([]);
          }
        }
    };

    self.findRecentlyAdded = function(item, callback){
      if (connection) {
          ClientAPI.findPaged({objectType: "Task", immediateParent: new ObjectId(item.id.text)}, [["dueDate", -1]], 1, offset, function (err, items) {
              var tasks = [];
              if (!err) {
                  for (var i = 0; i < items.length; i++) {
                      tasks.push(TaskController.encodeItem(items[i]));
                  }
              }
              if (callback != null && typeof callback == "function") {
                  callback(tasks);
              }
          });
      } else {
          var query = 'SELECT * FROM ' + models.taskDBModel.name + ' WHERE "recording-id" =?  AND "subject-type" = ? ORDER BY [due-at] DESC LIMIT ?, ?';
          var queryParams = [getTextTypeObject(item.id.text, 'integer'), item.objectType, 0, offset];
          db.selectByQuery(query, queryParams, 'task', function (items) {
            if (callback != null && typeof callback == "function") {
                callback(items);
            }
          });
      }
    };

    return self;
}(TaskController || {}));
