var UserController = (function (UserController)
{
    var self = UserController;
    var maxItems = 1000;

    self.getAllUser = function (callback) {
        db.selectAll(models.userDBModel.name, function (data) {
            var result = [];
            $(data).each(function (index, value) {
                result.push(value.user);
            });
            if (callback != null && typeof callback == "function") {
                callback(result);
            }
        });
    };

    self.synchronization = function (callback) {
        if (connection) {
            ClientAPI.getAppUsersAndGroupsCount(null, function (err, count) {
                if(err){
                  return callback();
                }
                db.deleteAllFromTable(models.userDBModel.name, function () {
                    db.deleteAllFromTable(models.groupsDBModel.name, function () {
                        GROUPS = [];
                        var max = Math.floor(count / maxItems);
                        var from = 0, to = maxItems, asyn;
                        if (count % maxItems != 0) {
                            max++;
                        }

                        asyn = max;
                        for (var i = 0; i < max; i++) {
                            ClientAPI.getAppUsersAndGroups(null, from, to, function (err, items) {
                                if (items != null && items.length > 0  && Array.isArray(items)){
                                    var users = items.filter(function (item) {
                                        return item.identityType == "User";
                                    });

                                    var groups = items.filter(function (item) {
                                        return item.identityType == "Group";
                                    });

                                    GroupController.encode(groups, function (results) {
                                        db.insertMultiple(models.groupsDBModel.name, [results], function () {});
                                        for (var i = 0, max = results.length; i < max; i++) {
                                            GROUPS.push(results[i]);
                                        }
                                        self.syncMembership(GROUPS);
                                    });

                                    self.encode(users, function (results) {
                                        asyn--;
                                        db.insertMultiple(models.userDBModel.name, [results], function(data) {});
                                        if (asyn == 0 && callback != null && typeof callback == "function") {
                                            getAllUsers();
                                            callback(results);
                                        }
                                    });
                                } else {
                                    getAllUsers();
                                    asyn--;
                                    if (asyn == 0 && callback != null && typeof callback == "function") {
                                        callback();
                                    }
                                }
                            });

                            from = to;
                            to += maxItems;
                        }
                    });
                });
            });
        } else {
            getAllUsers();
        }
    };

    self.syncMembership = function (groups) {
        db.deleteAllFromTable(models.membershipDBModel.name, function () {
            var customGroups = [];
            for (var i = 0; i < groups.length; i++) {
                if (groups[i].name != "ContentAdmins" && groups[i].name != "Everyone") {
                    customGroups.push(groups[i]);
                }
            }
            for (var i = 0; i < customGroups.length; i++) {
                (function (num) {
                    ClientAPI.getGroupMembers(customGroups[i].name, 1, 100, function (err, data) {
                        if (!err) {
                            if (data && data.length > 0) {
                                var memberItems = [];
                                for (var j = 0; j < data.length; j++) {
                                    var memberItem = {};
                                    memberItem['id'] = UtilController.randomNumber();
                                    memberItem['group-id'] = customGroups[num].id;
                                    memberItem['group'] = customGroups[num].name;
                                    memberItem['user-id'] = data[j]['_id'];
                                    memberItems.push(memberItem);
                                }
                                db.insertMultiple(models.membershipDBModel.name, [memberItems], function () {
                                });
                            }
                        }
                    });
                })(i);
            }
        });
    };

    self.encode = function (items, callback) {
        var result = [];
        if (items != null) {
            if (items.length != null && items.length > 0) {
                for (var i = 0, max = items.length; i < max; i++) {
                    var resultItem = self.encodeItem(items[i]);
                    if (resultItem != null) {
                        result.push(resultItem);
                    }
                }
                if (callback != null) {
                    callback(result);
                }
            } else {
                var resultItem = self.encodeItem(items);
                if (resultItem != null) {
                    result.push(resultItem);
                }
                if (callback != null) {
                    callback(result);
                }
            }
        }
    };

    self.encodeItem = function (item) {
        if (item['_id'] != null) {
            var result = new Object();
            result['id'] = getTextTypeObject(item['_id'], 'integer');
            result['admin'] = getTextTypeObject(false, 'boolean');
            result['created-at'] = getTextTypeObject(new Date(), 'datetime');
            result['updated-at'] = getTextTypeObject(new Date(), 'datetime');
            result['name'] = item.firstName + " " + item.lastName;
            result['email-address'] = item['e_mail'] ? item['e_mail'] : "";
            result['person'] = item['person'] ? item['person'] : "";
            return result;
        } else {
            return undefined;
        }
    };

    self.findUserByName = function(name, callback) {
      var queryUser;
      var queryUserData = [];
      if (name) {
          queryUser = 'SELECT * FROM user WHERE name LIKE ?';
          queryUserData.push('%' + name + '%');
      } else {
          queryUser = 'SELECT * FROM user';
      }
      console.log('queryUser');

      db.selectByQuery(queryUser, queryUserData, 'sip', function (sipData) {
          if (callback != null && typeof callback == "function") {
              callback(sipData);
          }
      });
    }

    self.findUserByNameOffline = function(name, callback) {
        name = name;
        var result = [];
        ALL_USERS.forEach(function(el){
          if (name) {
            if (el.name.toLowerCase().match(name.toLowerCase())) {
              result.push(el);
            }
          } else {
            result.push(el);
          }
        });

        if (callback != null && typeof callback == "function") {
            callback(result);
        }
    };


    self.findUserById = function(id, callback) {
      if (connection) {
        ClientAPI.getIdentitiesInfo([id], function(err, data){
          if (!err) {
            if (callback != null && typeof callback == "function") {
                callback(data);
            }
          } else {
            if (callback != null && typeof callback == "function") {
                callback([]);
            }
          }

        });
      } else {
        var queryUser = "SELECT * FROM user WHERE id=?";
        var queryUserData = [getTextTypeObject(id, 'integer')];
        db.selectByQuery(queryUser, queryUserData, 'sip', function (user) {
            if (callback != null && typeof callback == "function") {
                callback(user);
            }
        });
      }
    };

    self.loadSipUserFromServer = function (callback) {
      if (connection) {
          var url = "https://gsvba.com/m/getOnline";
          var tokenData = {
              "ConnectAuth": localStorage.getItem('userDomain') + "=" + localStorage.getItem('userToken') + "=" + localStorage.getItem('deviceId')
          }
          $.ajax({
            type: "POST",
            contentType: "application/json",
            url: url,
            data: JSON.stringify(tokenData),
            dataType: "json",
            success: function (data) {
              if (callback) {
                callback(data);
              }
            }
         });
      }
    };

    self.setPhoneNumberToUsers = function() {
      PeopleController.findAllWithIdentity(function(data){
        if (data) {
            if (connection) {
              data.forEach(function(element){
                self.setPhoneNumberToUsersIfOnline(element);
              });
            } else {
              var personNumber = {};
              data.forEach(function(element){
                var number = self.setPhoneNumberToUsersIfOffline(element.people);
                if (number) {
                  personNumber[element.people.id.text] = number;
                }
              });
              ALL_USERS.forEach(function(element){
                if (personNumber[element.person]) {
                  element['phone'] = personNumber[element.person];
                }
              });
            }
        }
      });
    }

    self.setPhoneNumberToUsersIfOnline = function(element) {
      if (element['contact-data'] && element['contact-data']['phone-numbers'] &&
        element['contact-data']['phone-numbers']['phone-number']) {
          if(Object.prototype.toString.call(
              element['contact-data']['phone-numbers']['phone-number']) === '[object Array]') {
                element['contact-data']['phone-numbers']['phone-number'].forEach(function(phone){
                    if (phone.location && phone.location == "Work") {
                      ALL_USERS[element.identity_id]['phone'] = phone.number;
                    }
                });
          } else {
            if (element['contact-data']['phone-numbers']['phone-number']['location'] &&
              element['contact-data']['phone-numbers']['phone-number']['location'] == "Work") {
                ALL_USERS[element.identity_id]['phone'] = element['contact-data']['phone-numbers']['phone-number'].number;
            }
          }
      }
    }

    self.setPhoneNumberToUsersIfOffline = function(element) {
      var phoneNumber = null;
      if (element['contact-data'] && element['contact-data']['phone-numbers'] &&
        element['contact-data']['phone-numbers']['phone-number']) {
          if(Object.prototype.toString.call(
              element['contact-data']['phone-numbers']['phone-number']) === '[object Array]') {
                element['contact-data']['phone-numbers']['phone-number'].forEach(function(phone){
                    if (phone.location && phone.location == "Work") {
                        phoneNumber = phone.number;
                    }
                });
          } else {
            if (element['contact-data']['phone-numbers']['phone-number']['location'] &&
              element['contact-data']['phone-numbers']['phone-number']['location'] == "Work") {
                  phoneNumber = element['contact-data']['phone-numbers']['phone-number']['number'];
            }
          }
      }
      return phoneNumber;
    }



    return self;
}(UserController || {}));
