function AppDatabase() {

    // basic database parameters
    var dbName = 'mConneCT';
    var dbLatestVer = '1.6';
    var dbDesc = 'mConneCT DB';

    var dbSize = 20 * 1024 * 1024; // 20MB
    // object that is used in transactions
    var db = null;
    var that = this;

    if (!window.localStorage.getItem('databaseVersionActive')) {
        window.localStorage.setItem('databaseVersionActive', dbLatestVer);
    }

    var databaseQueries = [
        {version: "0.2", query: "ALTER TABLE phoneContact ADD COLUMN avatar TEXT;"},
        {version: "0.3", query: "ALTER TABLE task ADD COLUMN [recording-type] TEXT;"},
        {version: "0.4", query: "ALTER TABLE kase ADD COLUMN [party] TEXT;"},
        {version: "0.5", query: "ALTER TABLE kase ADD COLUMN [party-id] TEXT;"},
        {version: "0.6", query: "ALTER TABLE kase ADD COLUMN [subjectDes] TEXT;"},
        {version: "0.7", query: "ALTER TABLE deal ADD COLUMN [statusHistory] TEXT;"},
        {version: "0.8", query: "ALTER TABLE kase ADD COLUMN [statusHistory] TEXT;"},
        {version: "0.9", query: "ALTER TABLE task ADD COLUMN [statusHistory] TEXT;"},
        {version: "1.0", query: "ALTER TABLE kase ADD COLUMN [due-at] TEXT;"},
        {version: "1.1", query: "ALTER TABLE deal ADD COLUMN [link] TEXT;"},
        {version: "1.2", query: "ALTER TABLE kase ADD COLUMN [link] TEXT;"},
        {version: "1.3", query: "ALTER TABLE task ADD COLUMN [name] TEXT;"},
        {version: "1.4", query: "ALTER TABLE task ADD COLUMN [percent-completed] TEXT;"},
        {version: "1.5", query: "ALTER TABLE offlineInsert ADD COLUMN [parent-id] TEXT;"},
        {version: "1.6", query: "ALTER TABLE groups ADD COLUMN [description] TEXT;"},
        {version: "1.7", query: "ALTER TABLE user ADD COLUMN [person] TEXT;"},
        {version: "1.8", query: "ALTER TABLE deal ADD COLUMN [entityId] TEXT;"},
        {version: "1.9", query: "ALTER TABLE kase ADD COLUMN [entityId] TEXT;"},
        {version: "2.0", query: "ALTER TABLE task ADD COLUMN [entityId] TEXT;"},
        {version: "2.1", query: "ALTER TABLE deal ADD COLUMN [salesStage] TEXT;"}
    ];

    this.createTable = function (table, callback) {
        db.transaction(
                function (tx) {
                    var columns = "";
                    var col;
                    for (col in table.data) {
                        if (table.data.hasOwnProperty(col)) {
                            columns += "[" + col + "] " + table.data[col] + " ,";
                        }
                    }
                    tx.executeSql(
                            'CREATE TABLE IF NOT EXISTS ' + table.name + ' (' + columns.substring(0, columns.length - 1) + ')',
                            [],
                            function () {
                            },
                            function (tx, err) {
                                console.log("Error sql createTable: " + err.code + " - " + err.message);
                            }
                    );

                },
                function (tx, err) {
                    // transaction error
                    console.log("Error transaction createTable, err: " + JSON.stringify(err));
                    if (callback != null) {
                        callback();
                    }
                },
                function () {
                    // transaction success
                    if (callback != null) {
                        callback();
                    }
                    return undefined;
                }
        );
    };

    /*
     * main function
     */
    this.openDatabase = function () {
        this.initMainDatabase();
        return db;
    };

    /*
     * private function for initializing database
     */
    this.initMainDatabase = function () {
        // INIT Main database
        db = window.openDatabase(dbName, "0.1", dbDesc, dbSize);
        var table;
        for (table in models) {
            this.createTable(models[table]);
        }
        this.versionDatabase();
        return db;
    };

    /*
     * parameters: * table - name of the table * arrayValues - array of JSON objects * callback -
     * optional callback function
     */
    this.insertMultiple = function (table, arrayValues, callback) {
        db.transaction(
                function (tx) {
                    if (arrayValues.length != null && arrayValues[0] != null && arrayValues[0].length != null) {
                        $(arrayValues[0]).each(function (index, value) {
                            var fields = "";
                            var fieldValuesString = "";
                            var fieldValues = [];
                            var separator = ",";
                            var obj = value;
                            for (var key in obj) {
                                //var dbkey = key.replace(/\-/g, "_");
                                fields += '[' + key + ']' + separator;
                                fieldValuesString += "?,";
                                if (obj.hasOwnProperty(key)) {
                                    fieldValues.push(JSON.stringify(obj[key]));
                                }
                            }
                            var sql = 'INSERT INTO '
                                    + table
                                    + ' ('
                                    + fields.substring(0, fields.length - 1)
                                    + ') VALUES ('
                                    + fieldValuesString.substring(0, fieldValuesString.length - 1) + ')';
                            tx.executeSql(
                                    sql,
                                    fieldValues,
                                    function () {
                                    },
                                    function (tx, err) {
                                        console.log("Error sql insertMultiple: " + err.code + " - " + err.message);
                                    }
                            );
                        });
                    } else {
                        that.insert(table, [arrayValues], callback);
                    }

                },
                function (tx, err) {
                    // transaction error
                    console.log("Error transaction insertMultiple, err: " + JSON.stringify(err));
                    if (callback != null) {
                        callback();
                    }
                },
                function () {
                    if (callback != null) {
                        callback();
                    }
                }
        );
    };

    /*
     * parameters: * table - name of the table * arrayValues - array of JSON objects * callback -
     * optional callback function
     */
    this.insertOrReplaceMultiple = function (table, arrayValues, callback) {
        db.transaction(
                function (tx) {
                    if (arrayValues.length != null && arrayValues[0] != null && arrayValues[0].length != null) {
                        $(arrayValues[0]).each(function (index, value) {
                            var fields = "";
                            var fieldValuesString = "";
                            var fieldValues = [];
                            var separator = ",";
                            var obj = value;
                            for (var key in obj) {
                                //var dbkey = key.replace(/\-/g, "_");
                                fields += '[' + key + ']' + separator;
                                fieldValuesString += "?,";
                                if (obj.hasOwnProperty(key)) {
                                    fieldValues.push(JSON.stringify(obj[key]));
                                }
                            }
                            var sql = 'INSERT OR REPLACE INTO '
                                    + table
                                    + ' ('
                                    + fields.substring(0, fields.length - 1)
                                    + ') VALUES ('
                                    + fieldValuesString.substring(0, fieldValuesString.length - 1) + ')';
                            tx.executeSql(
                                    sql,
                                    fieldValues,
                                    function () {
                                    },
                                    function (tx, err) {
                                        console.log("Error sql insertMultiple: " + err.code + " - " + err.message);
                                    }
                            );
                        });
                    } else {
                        that.insert(table, [arrayValues], callback);
                    }

                },
                function (tx, err) {
                    // transaction error
                    console.log("Error transaction insertMultiple, err: " + JSON.stringify(err));
                    if (callback != null) {
                        callback();
                    }
                },
                function () {
                    if (callback != null) {
                        callback();
                    }
                }
        );
    };

    /*
     * parameters: * table - name of the table * values - JSON object * callback -
     * optional callback function
     */
    this.insert = function (table, values, callback) {
        db.transaction(
                function (tx) {
                    var fields = "", fieldValuesString = "", fieldValues = [], separator = ",", obj = values[0], key;
                    for (key in obj) {
                        //var dbkey = key.replace(/\-/g, "_");
                        if (obj.hasOwnProperty(key)) {
                            fields += '[' + key + ']' + separator;
                            fieldValuesString += "?,";
                            fieldValues.push(JSON.stringify(obj[key]));
                        }
                    }
                    var sql = 'INSERT INTO '
                            + table
                            + ' ('
                            + fields.substring(0, fields.length - 1)
                            + ') VALUES ('
                            + fieldValuesString.substring(0, fieldValuesString.length - 1) + ')';
                    tx.executeSql(
                            sql,
                            fieldValues,
                            function () {
                            },
                            function (tx, err) {
                                console.log("Error sql insert: " + err.code + " - " + err.message);
                            }
                    );
                },
                function (tx, err) {
                    // transaction error
                    console.log("Error transaction insert, err: " + JSON.stringify(err));
                    if (callback != null) {
                        callback();
                    }
                },
                function () {
                    if (callback != null) {
                        callback();
                    }
                }
        );
    };

    /*
     * parameters: * table - name of the table * key - id of updated record *
     * value - value in where condition * object - object in JSON format *
     * callback - optional callback function
     */
    this.update = function (table, key, value, object, callback) {
        db.transaction(
                function (tx) {
                    // prepare fields
                    var fields = "";
                    var fieldValues = [];
                    var separator = "=?,";
                    var obj = object[table];
                    for (var key1 in obj) {
                        //var dbkey = key1.replace(/\-/g, "_");
                        if (obj.hasOwnProperty(key1)) {
                            fields += '[' + key1 + ']' + separator;
                            fieldValues.push(JSON.stringify(obj[key1]));
                        }
                    }
                    fieldValues.push(JSON.stringify(value));
                    var sql = 'UPDATE ' + table + ' SET '
                            + fields.substring(0, fields.length - 1) + ' WHERE ' + key
                            + ' =?';
                    tx.executeSql(
                            sql,
                            fieldValues,
                            function () {
                            },
                            function (tx, err) {
                                console.log("Error sql update: " + err.code + " - " + err.message);
                            }
                    );
                },
                function (tx, err) {
                    // transaction error
                    console.log("Error transaction update, err: " + JSON.stringify(err));
                    if (callback != null) {
                        callback();
                    }
                },
                function () {
                    if (callback != null) {
                        callback();
                    }
                }
        );
    };

    /*
     * parameters: * table - name of the table * column - where condition column
     * name * value - where condition value * callback - optional callback
     * function
     */
    this.deleteObject = function (table, column, value, callback) {
        db.transaction(
                function (tx) {
                    tx.executeSql(
                            "DELETE FROM " + table + " WHERE [" + column + "]=?",
                            [JSON.stringify(value)],
                            function (tx) {},
                            function (tx, err) {
                                console.log("Error sql deleteObject: " + err.code + " - " + err.message);
                            }
                    );
                },
                function (tx, err) {
                    // transaction error
                    console.log("Error transaction deleteObject, err: " + JSON.stringify(err));
                    if (callback != null) {
                        callback();
                    }
                },
                function () {
                    if (callback != null) {
                        callback();
                    }
                }
        );
    };


    this.deleteMultipleObject = function (table, column, value, callback) {
        db.transaction(
                function (tx) {
                    var sql = '';
                    var param = [];
                    if (column.length != null && column.length > 1) {
                        var p = '', k;
                        for (k in column) {
                            if (column.hasOwnProperty(k)) {
                                p += "[" + column[k] + "]=? AND ";
                            }
                        }
                        sql = "DELETE FROM " + table + " WHERE " + p;
                        sql = sql.substring(0, sql.length - 5);
                    } else {
                        sql = "DELETE FROM " + table + " WHERE [" + column + "]=?";
                    }
                    $(value).each(function (index, val) {
                        param.push(JSON.stringify(val));
                    });
                    tx.executeSql(
                            sql,
                            param,
                            function () {
                            },
                            function (tx, err) {
                                console.log("Error sql deleteMultipleObject: " + err.code + " - " + err.message);
                            }
                    );
                },
                function (tx, err) {
                    // transaction error
                    console.log("Error transaction deleteMultipleObject, err: " + JSON.stringify(err));
                    if (callback != null) {
                        callback();
                    }
                },
                function () {
                    if (callback != null) {
                        callback();
                    }
                }
        );
    };

    /*
     * parameters: * table - name of the table * callback - optional callback
     * function
     */
    this.selectAll = function (table, callback) {
        var that = this;
        db.transaction(
                function (tx) {
                    tx.executeSql(
                            "SELECT * FROM " + table,
                            [],
                            function (tx, results) {
                                if (callback != null) {
                                    var data = [], len = results.rows.length, i;
                                    for (i = 0; i < len; i++) {
                                        data.push(that.convertObjectToJSON(table, results.rows.item(i)));
                                    }
                                    callback(data);
                                }
                            },
                            function (tx, err) {
                                console.log("Error sql selectAll: " + err.code + " - " + err.message);
                            }
                    );
                },
                function (tx, err) {
                    // transaction error
                    console.log("Error transaction selectAll, err: " + JSON.stringify(err));
                    if (callback != null) {
                        callback();
                    }
                },
                function () {
                }
        );
    };

    /*
     * * parameters: * table - name of the table * column - where condition column
     * name * value - where condition value * callback - optional callback
     * function
     * function
     */
    this.selectCount = function (table, column, value, callback) {
        var that = this;
        db.transaction(
                function (tx) {
                    tx.executeSql(
                            "SELECT [" + column + "], COUNT(*) as COUNT FROM " + table + " WHERE [collection-id] = ? GROUP BY [" + column + "]",
                            [JSON.stringify(value)],
                            function (tx, results) {
                                if (callback != null) {
                                    var data = [], len = results.rows.length, i;
                                    for (i = 0; i < len; i++) {
                                        data.push(that.convertObjectToJSON(table, results.rows.item(i)));
                                    }
                                    callback(data);
                                }
                            },
                            function (tx, err) {
                                console.log("Error sql selectCount: " + err.code + " - " + err.message);
                            }
                    );
                },
                function (tx, err) {
                    // transaction error
                    console.log("Error transaction selectCount, err: " + JSON.stringify(err));
                    if (callback != null) {
                        callback();
                    }
                },
                function (tx) {
                }
        );
    };

    this.selectCountByColumn = function (table, column, value, callback) {
        if (table != null && column != null && value != null) {
            var sql = 'SELECT COUNT(*), [' + column + '] FROM ' + table + ' WHERE [' + column + '] in (';
            var param = [];
            if (value.length != null) {
                $(value).each(function (index, val) {
                    param.push(JSON.stringify(val));
                    sql += "?, ";
                });
                sql = sql.substring(0, sql.length - 2) + ")  GROUP BY [" + column + "]";
            } else {
                sql += "?)  GROUP BY [" + column + "]";
                param.push(JSON.stringify(value));
            }

            db.transaction(
                    function (tx) {
                        tx.executeSql(
                                sql,
                                param,
                                function (tx, results) {
                                    var data = [];
                                    var length = results.rows.length;
                                    for (var i = 0; i < length; i++) {
                                        data.push({
                                            id: JSON.parse(results.rows.item(i)['parent-id']).text,
                                            count: results.rows.item(i)['COUNT(*)']
                                        });
                                    }
                                    if (callback != null) {
                                        callback(data);
                                    }
                                },
                                function (tx, err) {
                                    console.log("Error sql selectCountByColumn: " + err.code + " - " + err.message);
                                }
                        );
                    },
                    function (tx, err) {
                        // transaction error
                        console.log("Error transaction selectCountByColumn, err: " + JSON.stringify(err));
                        if (callback != null) {
                            callback();
                        }
                    },
                    function (tx) {

                    }
            );

        }
    };

    /*
     * parameters: * table - name of the table * name - where condition column
     * name * value - where condition value * callback - callback function with
     * results in JSON format
     */
    this.selectByColumn = function (table, name, value, callback) {
        var that = this;
        db.transaction(
                function (tx) {
                    tx.executeSql(
                            "SELECT * FROM " + table + " WHERE [" + name + "]=?",
                            [JSON.stringify(value)],
                            function (tx, results) {
                                if (callback != null) {
                                    var data = [];
                                    var len = results.rows.length;
                                    for (var i = 0; i < len; i++) {
                                        data.push(that.convertObjectToJSON(table, results.rows.item(i)));
                                    }
                                    callback(data);
                                }
                            },
                            function (tx, err) {
                                console.log("Error sql selectByColumn: " + err.code + " - " + err.message);
                            }
                    );
                },
                function (tx, err) {
                    // transaction error
                    console.log("Error transaction selectByColumn, err: " + JSON.stringify(err));
                    if (callback != null) {
                        callback();
                    }
                },
                function () {
                }
        );
    };

    /*
     * parameters: * Query -
     * parameters: * value - where condition value
     * callback - callback function with
     * objectName -
     * results in JSON format
     */
    this.selectByQuery = function (query, value, objectName, callback) {
        var that = this;
        db.transaction(
                function (tx) {
                    var fieldValues = [];
                    var i, max;
                    for (i = 0, max = value.length; i < max; i++) {
                        if (typeof value[i] == "string" && value[i].indexOf("%") != -1) {
                            fieldValues.push(value[i]);
                        } else {
                            fieldValues.push(JSON.stringify(value[i]));
                        }
                    }
                    tx.executeSql(
                            query,
                            fieldValues,
                            function (tx, results) {
                                if (callback != null) {
                                    var data = [], len = results.rows.length, i;
                                    for (i = 0; i < len; i++) {
                                        data.push(that.convertObjectToJSON(objectName, results.rows.item(i))[objectName]);
                                    }
                                    callback(data);
                                }
                            },
                            function (tx, err) {
                                console.log("Error sql selectByQuery: " + err.code + " - " + err.message);
                            }
                    );
                },
                function (tx, err) {
                    // transaction error
                    console.log("Error transaction selectByQuery, err: " + JSON.stringify(err));
                    if (callback != null) {
                        callback();
                    }
                },
                function () {
                }
        );
    };

    this.selectByIds = function (table, column, value, callback) {
        var that = this;
        if (table != null && column != null && value != null) {
            var sql = 'SELECT * FROM ' + table + ' WHERE [' + column + '] in (';
            var param = [];
            if (value.length != null) {
                $(value).each(function (index, val) {
                    param.push(JSON.stringify(val));
                    sql += "?, ";
                });
                sql = sql.substring(0, sql.length - 2) + ")";
            } else {
                sql += "?)";
                param.push(JSON.stringify(value));
            }

            db.transaction(
                    function (tx) {
                        tx.executeSql(
                                sql,
                                param,
                                function (tx, results) {
                                    var data = [];
                                    var length = results.rows.length;
                                    for (var i = 0; i < length; i++) {
                                        data.push(that.convertObjectToJSON(table, results.rows.item(i)));
                                    }
                                    if (callback != null) {
                                        callback(data);
                                    }
                                },
                                function (tx, err) {
                                    console.log("Error sql selectByIds: " + err.code + " - " + err.message);
                                }
                        );
                    },
                    function (tx, err) {
                        // transaction error
                        console.log("Error transaction selectByIds, err: " + JSON.stringify(err));
                        if (callback != null) {
                            callback();
                        }
                    },
                    function () {
                    }
            );

        }
    };


    /*
     * parameters: * table - name of the table * name - where condition column
     * name * value - where condition value * callback - callback function with
     * JSON object
     */
    this.selectSingleRecord = function (table, name, value, callback) {
        var that = this;
        db.transaction(
                function (tx) {
                    tx.executeSql(
                            "SELECT * FROM " + table + " WHERE [" + name + "]=?",
                            [JSON.stringify(value)],
                            function (tx, results) {
                                if (callback != null) {
                                    if (results.rows.length > 0) {
                                        callback(that.convertObjectToJSON(table, results.rows.item(0)));
                                    } else {
                                        callback();
                                    }
                                }
                            },
                            function (tx, err) {
                                console.log("Error sql selectSingleRecord: " + err.code + " - " + err.message);
                            }
                    );
                },
                function (tx, err) {
                    // transaction error
                    console.log("Error transaction selectSingleRecord, err: " + JSON.stringify(err));
                    if (callback != null) {
                        callback();
                    }
                },
                function () {
                }
        );
    };

    /*
     * parameters: * table - name of the table * callback - optional callback
     * function
     */
    this.dropTable = function (table, callback) {
        db.transaction(
                function (tx) {
                    tx.executeSql(
                            'DROP TABLE IF EXISTS ' + table,
                            [],
                            function () {
                            },
                            function (tx, err) {
                                console.log("Error sql dropTable: " + err.code + " - " + err.message);
                            }
                    );
                },
                function (tx, err) {
                    // transaction error
                    console.log("Error transaction dropTable, err: " + JSON.stringify(err));
                    if (callback != null) {
                        callback();
                    }
                },
                function () {
                    if (callback != null) {
                        callback();
                    }
                }
        );
    };

    this.dropAllTables = function () {
        var table;
        for (table in models) {
            if (models.hasOwnProperty(table)) {
                this.dropTable(models[table].name);
            }
        }
    };

    this.deleteServerItemFromTableFavorite = function (data, callback) {
        // DELETE FROM favorite WHERE refObjectType != '"PhoneNumber"';
        var query = "DELETE FROM favorite WHERE refObjectType!='\"PhoneNumber\"'";

        db.transaction(
                function (tx) {
                    tx.executeSql(
                            query,
                            [],
                            function () {
                            },
                            function (tx, err) {
                                console.log("Error sql deleteServerItemFromTableFavorite: " + err.code + " - " + err.message);
                            }
                    );
                },
                function (tx, err) {
                    // transaction error
                    console.log("Error transaction deleteServerItemFromTableFavorite, err: " + JSON.stringify(err));
                    if (callback != null) {
                        callback();
                    }
                },
                function () {
                    if (callback != null) {
                        callback();
                    }
                }
        );
    };


    this.deleteAllFromTable = function (name, callback) {
        db.transaction(
                function (tx) {
                    tx.executeSql(
                            "DELETE FROM " + name,
                            [],
                            function () {
                            },
                            function (tx, err) {
                                console.log("Error sql deleteAllFromTable: " + err.code + " - " + err.message);
                            }
                    );
                },
                function (tx, err) {
                    // transaction error
                    console.log("Error transaction deleteAllFromTable, err: " + JSON.stringify(err));
                    if (callback != null) {
                        callback();
                    }
                },
                function () {
                    if (callback != null) {
                        callback();
                    }
                }
        );
    };

    this.deleteAllFromTablesInDB = function (callback) {
        var counter = 0, table;
        for (table in models) {
            if (models.hasOwnProperty(table)) {
                if (models[table].name != "user" && models[table].name != "dealCategory" &&
                        models[table].name != "taskCategory" && models[table].name != "recent" &&
                        models[table].name != "domain" && models[table].name != "draft" &&
                        models[table].name != "favorite" && models[table].name != "groups") {
                    counter++;
                    this.deleteAllFromTable(models[table].name, function () {
                        counter--;
                        if (counter == 0) {
                            if (callback != null) {
                                callback();
                            }
                        }
                    });
                }
            }
        }
    };



    this.deleteFromTableByIds = function (table, colomn, values, callback) {
        var sql = "DELETE FROM " + table + " WHERE " + colomn + " IN (";
        var i, max;
        for (i = 0, max = values.length; i < max; i++) {
            sql += "?,";
        }
        sql = sql.substring(0, sql.length - 1) + ")";
        var valueForDelete = [];
        for (i = 0, max = values.length; i < max; i++) {
            valueForDelete.push(JSON.stringify(values[i]));
        }
        db.transaction(
                function (tx) {
                    tx.executeSql(
                            sql,
                            valueForDelete,
                            function () {
                            },
                            function (tx, err) {
                                console.log("Error sql deleteFromTableByIds: " + err.code + " - " + err.message);
                            }
                    );
                },
                function (tx, err) {
                    // transaction error
                    console.log("Error transaction deleteFromTableByIds, err: " + JSON.stringify(err));
                    if (callback != null) {
                        callback();
                    }
                },
                function () {
                    if (callback != null) {
                        callback();
                    }
                }
        );
    };

    this.updateObject = function (table, key, value, object, updateAll, callback) {
        db.transaction(function (tx) {
            var query = "UPDATE " + table + " SET";
            var queryParams = [];

            for (var key1 in object) {
                if (key1 != "id") {
                    query += " [" + key1 + "] = ?,";
                    queryParams.push(JSON.stringify(object[key1]));
                }
            }
            query = query.substring(0, query.length-1);

            if(!updateAll){
                query += " WHERE [" + key + "] = ?";
                queryParams.push(JSON.stringify(value));
            }

            console.log("Update Object");
            console.log("#1 Query");
            console.log(query);
            console.log("#2 Query Params");
            console.log(queryParams);

            tx.executeSql(query, queryParams, function (tx, results) {
                if (callback && typeof callback == "function") {
                    callback();
                }

            }, function (tx, err) {
                console.log("Error sql selectByQuery: " + err.code + " - " + err.message);
            });
        }, function (tx, err) {
            // transaction error
            console.log("Error transaction selectByQuery, err: " + JSON.stringify(err));
            if (callback != null) {
                callback();
            }
        });
    };

    /*
     * parameters: dbRecord - object that is read from database
     */
    this.convertObjectToJSON = function (table, dbRecord) {
        var obj = {}, attr;
        for (attr in dbRecord) {
            if (dbRecord[attr] != null && dbRecord[attr] != "undefined") {
                if (dbRecord.hasOwnProperty(attr)) {
                    try {
                        obj[attr] = JSON.parse(dbRecord[attr]);
                    } catch (e) {
                        var replaced = dbRecord[attr].replace('""', ' ');
                        obj[attr] = JSON.parse(replaced);
                    }
                }
            }
        }

        var obj2 = {};
        obj2[table] = obj;
        return obj2;
    };

    this.versionDatabase = function (callback) {
        var count = databaseQueries.length;
        var databaseVersionActive = window.localStorage.getItem('databaseVersionActive');
        $(databaseQueries).each(function (index, value) {
            db.transaction(
                    function (tx) {
                        if (parseFloat(value.version) > databaseVersionActive) {
                            tx.executeSql(
                                    value.query,
                                    [],
                                    function () {
                                    },
                                    function (tx, err) {
                                        console.log("Error sql versionDatabase: " + err.code + " - " + err.message);
                                    }
                            );
                            databaseVersionActive = value.version;
                            window.localStorage.setItem('databaseVersionActive', databaseVersionActive);
                        }
                    },
                    function (tx, err) {
                        // transaction error
                        console.log("Error transaction versionDatabase, err: " + JSON.stringify(err));
                        if (callback != null) {
                            callback();
                        }
                    },
                    function () {
                        count--;
                        if (count == 0) {
                            if (callback != null) {
                                callback();
                            }
                        }
                    }
            );
        });
    };
}
