var EffortService = (function (EffortService) {
    var self = EffortService;

    self.validateEffort = function (efforts, relatedCreationDate) {
        for (var i in efforts) {
            if (!(/^[0-9]{1,2}$/i.test(efforts[i].hours)) || efforts[i].description == "") {
                showAlertDialog(Messages.Effort.invalidHoursOrDescription, Messages.Effort.alert, Messages.Effort.ok, null);
                return false;
            }
            if (new Date(efforts[i].effortDate) > new Date()) {
                showAlertDialog(Messages.Effort.addEffortInFutureError, Messages.Effort.alert, Messages.Effort.ok, null);
                return false;
            }
            if (new Date(efforts[i].effortDate) < new Date(relatedCreationDate)) {
                showAlertDialog(Messages.Effort.addEffortToRelatedObjectError, Messages.Effort.alert, Messages.Effort.ok, null);
                return false;
            }
        }
        return true;
    };

    self.calculateEfforts = function (efforts) {
        var sum = 0;
        for (var key in efforts) {
            sum += efforts[key].hours;
        }
        return sum;
    };

    self.loadEffort = function (relatedId, relatedType, pageId, tableId, fromPage, callback) {
        ClientAPI.findEfforts({query: {immediateParent: relatedId}, relatedType: relatedType}, function (err, effort) {
            if (err) {
                showAlertDialog(Messages.Effort.errorLoadingEffort, Messages.Effort.alert, Messages.Effort.ok, null);
                if (callback) {
                    callback();
                }
            } else {
                if (effort && effort.length > 0) {
                    if(fromPage){
                        self.populateEffortTable(effort[0].efforts, pageId, tableId, callback);
                    } else {
                        self.loadEffortViewTable(pageId, effort[0].efforts, callback);
                    }
                } else {
                  if (callback) {
                      callback();
                  }
                }
            }
        });
    };

    self.createOrUpdateEffort = function (relatedId, relatedType, displayName, relatedCreationDate, tableId, cb) {
        ClientAPI.findEfforts({query: {immediateParent: relatedId}, relatedType: relatedType}, function (err, effort) {
            // load efforts from form
            var taskEfforts = [];
            var userProfile = JSON.parse(window.localStorage.getItem('userProfile'));
            var submitter = userProfile.first_name + " " + userProfile.last_name;
            if (typeof tableId != "function") {
              var taskEffortRows = $('#' + tableId + ' .task-effort-row');
              for (var i = 0; i < taskEffortRows.length; i++) {
                  var currentDate = new Date();
                  taskEfforts.push({hours: parseInt($.trim($(taskEffortRows[i]).find('.effort-hours').val())),
                      description: $.trim($(taskEffortRows[i]).find('.effort-desc').val()),
                      submitter: submitter,
                      submitter_id: userProfile.identity_id,
                      effort_creationDate: new Date(),
                      effortDate: new Date(parseInt($(taskEffortRows[i]).find('.date-time-picker').val().split(".")[2].split(" ")[0]),
                              parseInt($(taskEffortRows[i]).find('.date-time-picker').val().split(".")[1] - 1),
                              parseInt($(taskEffortRows[i]).find('.date-time-picker').val().split(".")[0]),
                              currentDate.getHours(), currentDate.getMinutes(), currentDate.getSeconds())
                  });
              }
            } else {
              cb = tableId;
            }

            var testEfforts = self.validateEffort(taskEfforts, relatedCreationDate);
            if (testEfforts) {
                if (err) {
                    return cb(err);
                } else {
                    if (effort && effort.length > 0) {
                        effort[0].efforts = taskEfforts;
                        ClientAPI.updateEfforts(effort[0], function (err, data) {
                            if (err) {
                                return cb(err);
                            } else {
                                return cb(null, data);
                            }
                        });
                    } else {
                        var effortObject = {
                            immediateParent: relatedId,
                            immediateParentObjectType: relatedType,
                            immediateParentDisplay : displayName,
                            efforts: taskEfforts
                        };
                        if (taskEfforts.length > 0) {
                            ClientAPI.addEfforts(effortObject, function (err, data) {
                                if (err) {
                                    return cb(err);
                                } else {
                                    return cb(null, data);
                                }
                            });
                        } else {

                            return cb();
                        }
                    }
                }
            }
        });
    };

    self.populateEffortTable = function (efforts, pageId, tableId, callback) {
        var html = "";
        for (var i in efforts) {
            var date = efforts[i].effortDate.split("T")[0];
            var effortDate = date.split("-")[2] + '.' + date.split("-")[1] + '.' + date.split("-")[0];
            html += '<div class="task-effort-row bottom-margin-10">'
                + '<div><input class="date-time-picker" value="' + effortDate +'" readonly/>'
                + '</div>'
                + '<div style="display:inline-block; width:15%;">'
                + '<input class="effort-hours" type="number" min="1" data-mini="true" data-inset="true" class="full-width" value="' + efforts[i].hours  + '"/>'
                + '</div>'
                + '<div style="display:inline-block; width:70%;">'
                + '<input class="effort-desc" type="text" data-mini="true" data-inset="true" class="full-width" value="' + efforts[i].description +'"/>'
                + '</div>'
                + '<div style="display:inline-block; width:10%;position: relative;top: 15px;margin-left: 12px"><span class="remove-effort-link remove-button-link" href=""> <i class="md-remove  md-2x"></i></span>'
                + '</div>'
                + '</div>';

        }
        $('#' + pageId + ' #' + tableId).append(html);
        $('#' + pageId + ' #' + tableId).trigger('create');
        initDateTimePicker(".date-time-picker");
        $('.date-time-picker').attr('type', "");
        if (callback) {
          callback();
        }
    };

    self.drawEffortTable = function (pageId, tableId, date) {
      var htmlDateValue = "";
      if (date) {
        htmlDateValue = dateFormat(date);
      }
      $('#' + pageId + ' #' + tableId).append(
              '<div class="task-effort-row bottom-margin-10">'
              + '<div><input class="date-time-picker" value="'+ htmlDateValue + '" data-i18n="[placeholder]effort.effortDate" readonly/>'
              + '</div>'
              + '<div style="display:inline-block; width:15%;"><input class="effort-hours" type="number" min="1" data-mini="true" data-inset="true" value="" class="full-width" data-i18n="[placeholder]effort.hours"/>'
              + '</div>'
              + '<div style="display:inline-block; width:70%;"><input class="effort-desc" type="text" data-mini="true" data-inset="true" value="" class="full-width" data-i18n="[placeholder]effort.description"/>'
              + '</div>'
              + '<div style="display:inline-block; width:10%;position: relative;top: 15px;margin-left: 12px"><span class="remove-effort-link remove-button-link" href=""> <i class="md-remove  md-2x"></i></span>'
              + '</div>'
              + '</div>'
              );
        $('#' + pageId + ' #' + tableId).i18n();

        // Trigger create table
        $('#' + pageId + ' #' + tableId).trigger('create');
        $('.date-time-picker').attr('type', "");
    };

    self.removeEffortLink = function (evt) {
        $(evt.currentTarget).parent().parent().remove();
    };

    self.loadEffortViewTable = function (pageId, efforts, callback) {
        var html = "";
        for (var i in efforts) {
            html += "<li>"
                    + "<span class='table-el-five align-text'>"
                    + efforts[i].hours
                    + "</span>"
                    + "<span class='table-el-thirty align-text'>"
                    + efforts[i].description
                    + "</span>"
                    + "<span class='table-el-twenty align-text'>"
                    + efforts[i].submitter
                    + "</span>"
                    + "<span class='table-el-thirty align-text'>"
                    + dateFormat(new Date(efforts[i].effort_creationDate))
                    + "</span>";
        }
        $('#' + pageId + ' .effort-list').append(html);
        if (callback) {
          callback();
        }
    };

    return self;
}(EffortService || {}));
