function GoogleMap() {

    this.initialize = function () {
        if (latitude && longitude) {
            var mapObjects = [];
            ClientAPI.findLocations({position: {type: 'Point', coordinates: [longitude, latitude]}, objectType: 'Note'}, function (err, data) {
                if (data && data.locations && data.locations.length > 0) {
                    var userLocations = data.locations;
                    for (var i = 0; i < userLocations.length; i++) {
                        var mapObj = {};
                        mapObj.coordinates = userLocations[i].location.coordinates;
                        mapObj.userId = userLocations[i].userId;
                        mapObj.name = userLocations[i].parentDisplay;
                        mapObj.avatarUrl = ClientAPI.rootURL + '/api/db/downloadimg?objId=' + encodeURIComponent(userLocations[i].parentId) + '&fieldName=' + encodeURIComponent('picture');
                        mapObjects.push(mapObj);
                    }
                }
                var map = showMap();
                addMarkersToMap(map, mapObjects);
            });
        } else {
            showMap();
        }
    };

    var showMap = function () {
        var mapOptions = {
            zoom: 6,
            mapTypeId: google.maps.MapTypeId.ROADMAP
        };
        if (latitude && longitude) {
            mapOptions.center = new google.maps.LatLng(latitude, longitude);
        } else {
            mapOptions.center = new google.maps.LatLng(43.871920, 18.413810);
        }

        var map = new google.maps.Map(document.getElementById("map_canvas"), mapOptions);

        return map;
    };

    var addMarkersToMap = function (map, mapArray) {
        if (mapArray && mapArray.length > 0) {
            for (var i = 0; i < mapArray.length; i++) {
                var coordinates = new google.maps.LatLng(mapArray[i].coordinates[1], mapArray[i].coordinates[0]);
                var overlay = new CustomMarker(coordinates, map, mapArray[i].avatarUrl, mapArray[i].name);
                (function (num) {
                    overlay.addListener('click', function () {
                        gotoPage({url: "editTask", param: {ownerId: mapArray[num].userId}}, {url: "map", param: null});
                    });
                }(i));

            }
        }
    };

    this.getUserAddress = function (callback) {
        var geocoder = new google.maps.Geocoder();
        var coordinates = new google.maps.LatLng(latitude, longitude);
        geocoder.geocode({'location': coordinates}, function (results, status) {
            if (status == google.maps.GeocoderStatus.OK) {
                if (results[0]) {
                    if (callback && typeof callback == "function") {
                        callback(results[0].formatted_address, status);
                    }
                } else {
                    callback(null, status);
                }
            } else {
                callback(null, status);
            }
        });

    };
}
;

function CustomMarker(latlng, map, avatarUrl, name) {
    this.latlng_ = latlng;
    this.avatar = avatarUrl;
    this.username = name;
    // Once the LatLng and text are set, add the overlay to the map.  This will
    // trigger a call to panes_changed which should in turn call draw.
    this.setMap(map);
}

CustomMarker.prototype = new google.maps.OverlayView();

CustomMarker.prototype.draw = function () {
    var me = this;
    this.imageSrc = this.avatar;
    // Check if the div has been created.
    var div = this.div_;
    if (!div) {
        // Create a overlay text DIV
        div = this.div_ = document.createElement('DIV');
        // Create the DIV representing our CustomMarker
        div.style.position = 'absolute';
        div.style.paddingLeft = '0px';
        div.style.cursor = 'pointer';

        var img = document.createElement("img");
        img.src = this.imageSrc;
        img.className = "map-avatar";
        div.appendChild(img);
        var label = this.span_ = document.createElement("span");
        this.span_.innerHTML = this.username;
        label.className = "map-title";
        div.appendChild(label);

        google.maps.event.addDomListener(div, "click", function (event) {
            google.maps.event.trigger(me, "click");
        });

        // Then add the overlay to the DOM
        var panes = this.getPanes();
        panes.overlayImage.appendChild(div);
    }

    // Position the overlay 
    var point = this.getProjection().fromLatLngToDivPixel(this.latlng_);
    if (point) {
        div.style.left = point.x + 'px';
        div.style.top = point.y + 'px';
    }
};

CustomMarker.prototype.remove = function () {
    // Check if the overlay was on the map and needs to be removed.
    if (this.div_) {
        this.div_.parentNode.removeChild(this.div_);
        this.div_ = null;
    }
};

CustomMarker.prototype.getPosition = function () {
    return this.latlng_;
};


