function getTextTypeObject(text, type) {
    var ob = new Object();
    if (type != null) {
        ob["type"] = type;
    }
    if (text != null && text != 'undefined') {
        ob["text"] = text;
    }
    return ob;
}

function getTagObject(id, name) {
    var ob = new Object();
    ob["name"] = name;
    return ob;
}

function getSubjectData(subjectFieldId, value) {
    var ob = new Object();
    ob["subject_data"] = new Object();
    ob["subject_data"]["subject_field_id"] = getTextTypeObject(subjectFieldId, 'integer');
    ob["subject_data"]["value"] = value;
    return ob;
}

function getPhoneNumberObject(id, location, number) {
    var ob = new Object();
    ob["phone-number"] = new Object();
    ob["phone-number"]["location"] = location;
    ob["phone-number"]["number"] = number;
    return ob;
}

function getEmailAddressObject(title, body, subjectId, subjectType) {
    var ob = new Object();
    ob['email'] = new Object();
    ob['email']["title"] = title;
    ob['email']["subject-id"] = getTextTypeObject(subjectId, "integer");
    ob['email']["subject-type"] = subjectType;
    return ob;
}

function getCommentObject(body, parentId) {
    var ob = new Object();
    ob["comment"] = new Object();
    ob["comment"]["body"] = body;
    ob["comment"]['parent-id'] = parentId;
    return ob;
}


function getAttachmentObject(id, url, name, size) {
    var ob = new Object();
    ob["attachment"] = new Object();
    ob["attachment"]["id"] = getTextTypeObject(id, "integer");
    ob["attachment"]["url"] = url;
    ob["attachment"]["name"] = name;
    ob["attachment"]["size"] = size;
    return ob;
}

function getNoteObject(body, subjectId, subjectType, visibleTo, groupId) {
    var ob = new Object();
    ob["note"] = new Object();
    ob["note"]["body"] = body;

    if (subjectId != null) {
        ob["note"]["subject-id"] = getTextTypeObject(subjectId, "integer");
        if (subjectType == "Party" || subjectType == "Deal" || subjectType == "Kase") {
            ob["note"]["subject-type"] = subjectType;
        }
    }

    if (visibleTo != null) {
        if (visibleTo == "justMe") {
            ob["note"]["visible-to"] = "Owner";
            ob["note"]["group-id"] = {'type': 'integer', 'nil': 'true'};
        } else if (visibleTo == "everyone") {
            ob["note"]["visible-to"] = "Everyone";
            ob["note"]["group-id"] = {'type': 'integer', 'nil': 'true'};
        } else if (visibleTo == "mePlusGroup") {
            if (groupId != null) {
                ob["note"]["visible-to"] = "NamedGroup";
                ob["note"]["group-id"] = getTextTypeObject(groupId, "integer");
            }
        }
    }

    return ob;
}

function getKaseObject(name, visibleTo, closedAt) {
    var ob = new Object();
    ob['kase'] = new Object();
    ob['kase']['name'] = name;

    if (visibleTo != null) {
        if (visibleTo == "justMe") {
            ob["kase"]["visible-to"] = "Owner";
            ob["kase"]["group-id"] = "{'type':'integer', 'nil':'true'}";
        } else if (visibleTo == "everyone") {
            ob["kase"]["visible-to"] = "Everyone";
            ob["kase"]["group-id"] = "{'type':'integer', 'nil':'true'}";
        } else if (visibleTo == "mePlusGroup") {
            if (groupId != null) {
                ob["kase"]["visible-to"] = "NamedGroup";
                ob["kase"]["group-id"] = {"type": "integer", "text": groupId};
            }
        }
    }

    if (closedAt != null) {
        ob["kase"]["closed-at"] = getTextTypeObject(closedAt, "datetime");
    }

    return ob;
}

function getGroupObject(name) {
    var ob = new Object();
    ob["group"] = new Object();
    ob["group"]["name"] = name;
    return ob;
}

function getCustomFieldObject(label) {
    var ob = new Object();
    ob["subject-field"] = new Object();
    ob["subject-field"]["label"] = label;
    return ob;
}

function getTaskObject(body, frame, subjectType, subjectId, categoryId, recordingId, ownerId, publics, notify, dueAt) {
    var ob = new Object();
    ob["task"] = new Object();
    ob["task"]["body"] = body;
    ob["task"]["frame"] = frame;

    if (frame == 'specific') {
        ob["task"]['due-at'] = new Object();
        ob["task"]['due-at']['text'] = dueAt;
        ob["task"]['due-at']['type'] = 'datetime';
    }

    if (subjectType == "Party" || subjectType == "Company" ||
            subjectType == "Kase" || subjectType == "Deal") {
        ob["task"]["subject-type"] = subjectType;
    } else {
        ob["task"]["subject-type"] = {'nil': 'true'};
    }

    if (subjectId != null) {
        ob["task"]["subject-id"] = subjectId;
    } else {
        ob["task"]["subject-id"] = {'type': 'integer', 'nil': 'true'};
    }

    if (categoryId != null) {
        ob["task"]["category-id"] = getTextTypeObject(categoryId, "integer");
    }
    if (recordingId != null) {
        ob["task"]["recording-id"] = getTextTypeObject(recordingId, "integer");
    }
    if (ownerId != null) {
        ob["task"]["owner-id"] = getTextTypeObject(ownerId, "integer");
    }
    if (publics != null) {
        ob["task"]["public"] = getTextTypeObject(publics, "boolean");
    }
    if (notify != null) {
        ob["task"]["notify"] = getTextTypeObject(notify, "boolean");
    }
    return ob;
}

function getDealObject(name, partyId, visibleTo, groupId, ownerId, responsiblePartyId, categoryId, background, currency, price, priceType, duration) {
    var ob = new Object();
    ob["deal"] = new Object();
    ob["deal"]['name'] = name;

    if (partyId != null) {
        ob["deal"]["party-id"] = getTextTypeObject(partyId, "integer");
    }

    if (visibleTo != null && visibleTo.toUpperCase() == "Everyone".toUpperCase()) {
        ob["deal"]["visible-to"] = "Everyone";
    } else if (visibleTo != null && visibleTo.toUpperCase() == "justMe".toUpperCase()) {
        ob["deal"]["visible-to"] = "Owner";
    } else if (visibleTo != null && visibleTo.toUpperCase() == "NamedGroup".toUpperCase()) {
        ob["deal"]["visible-to"] = "NamedGroup";
    }

    if (groupId != null) {
        ob["deal"]["group-id"] = getTextTypeObject(groupId, "integer");
    }
    if (ownerId != null) {
        ob["deal"]["owner-id"] = getTextTypeObject(ownerId, "integer");
    }
    if (responsiblePartyId != null) {
        ob["deal"]["responsible-party-id"] = getTextTypeObject(responsiblePartyId, "integer");
    }

    if (categoryId != null) {
        ob["deal"]["category-id"] = getTextTypeObject(categoryId, "integer");
    } else {
        ob["deal"]["category-id"] = {'type': 'integer', 'nil': 'true'};
    }

    if (background != null) {
        ob["deal"]["background"] = background;
    }
    if (currency != null) {
        ob["deal"]["currency"] = currency;
    }
    if (price != null) {
        ob["deal"]["price"] = getTextTypeObject(price, "integer");
    }
    if (priceType != null) {
        ob["deal"]["price-type"] = priceType;
    }
    if (duration != null) {
        ob["deal"]["duration"] = getTextTypeObject(duration, "integer");
    }
    return ob;
}

var companyXML = '<company><id type="integer" nil="true"></id><name nil="true"></name><background nil="true"></background><created-at type="datetime" nil="true"></created-at>' +
        '<updated-at type="datetime" nil="true"></updated-at><visible-to nil="true"></visible-to><owner-id type="integer" nil="true"></owner-id>' +
        '<group-id type="integer" nil="true"></group-id><author-id type="integer" nil="true"></author-id><contact-data nil="true"><email-addresses nil="true"><email-address>' +
        '<id type="integer"></id><address></address><location></location>/email-address></email-addresses><phone-numbers nil="true"><phone-number><id type="integer"></id>' +
        '<number></number><location></location></phone-number><phone-number><id type="integer" nil="true"></id><number></number><location>Fax</location></phone-number>' +
        '</phone-numbers></contact-data><subject_datas type="array" nil="true"><subject_data><id type="integer"></id><value></value><subject_field_id type="integer">' +
        '</subject_field_id><subject_field_label></subject_field_label></subject_data></subject_datas><tags type="array" nil="true">' +
        '<tag><id type="integer"></id><name></name></tag></tags</company>';

var peopleXML = '<person><id type="integer" nil="true"></id><first-name nil="true"></first-name><last-name nil="true"></last-name><title nil="true"></title>' +
        '<background nil="true"></background><linkedin-url nil="true"></linkedin-url><avatar-url nil="true"></avatar-url><company-id type="integer" nil="true"></company-id>' +
        '<company-name nil="true"></company-name><created-at type="datetime" nil="true"></created-at><updated-at type="datetime" nil="true"></updated-at>' +
        '<visible-to nil="true"></visible-to><owner-id type="integer" nil="true"></owner-id><group-id type="integer" nil="true"></group-id><author-id type="integer" nil="true">' +
        '</author-id><contact-data nil="true"></contact-data><subject_datas type="array" nil="true"></subject_datas><tags type="array" nil="true"></tags></person>';