$(document).on('pageshow', '#edit-case', function () {
    EditCaseFunctions.init();
    EditCaseFunctions.bindElementControls();
    //initiate richtext editor
    if (tinymce.editors.length > 0) {
        tinymce.editors = [];
    }
    tinymce.init({
        mode: "exact",
        selector: "#edit-case-body",
        autoresize_min_height: 250,
        menubar: false,
        content_css: "styles/tinymce-custom.css",
        statusbar: false,
        skin: 'light',
        plugins: "autoresize",
        toolbar: [
            "bold italic underline bullist numlist outdent indent"
        ]
    });
});

$(document).on('pagehide', '#edit-case', function () {
    EditCaseFunctions.unbindElementControls();
});

var EditCaseFunctions = (function (EditCaseFunctions)
{
    var self = EditCaseFunctions;

    var $caseGroupContainer = null;
    var $casePeopleContainer = null;
    var $editCaseStatus = null;
    var $editCaseBody = null;
    var $editCaseName = null;
    var $autoCompleteOwner = null;
    var $caseFrame = null;
    var $dateTimePicker = null;
    var $dateTimePickerContainer = null;
    var caseObject = null;
    var caseParentId = null;
    var caseParentType = null;
    var visibilityObjectId = null;
    var visibilityObjectType = null;

    self.init = function () {
        PermissionController.init('#edit-case');
        AutoCompleteController.init('#edit-case');
        AutoCompleteController.addViewId('autocomplete-owner', 'autocomplete-owner-list', 'autocomplete-owner');
        AutoCompleteController.addViewId('autocomplete-case-contact', 'autocomplete-case-contact-list', 'autocomplete-case-contact', function (data) {
            setTimeout(function () {
                if (data) {
                    visibilityObjectId = data.id;
                    visibilityObjectType = data.objectType;
                }
                if (visibilityObjectId && visibilityObjectType) {
                    showActionDialog('Take visibility of parent object?', 'Yes', self.updateACL, "No", function () {
                    });
                }
            }, 2000);
        });
        self.selectElement();
        self.initDateTimePicker();

        if (pageObject && pageObject.caseId) {
            $('#case-edit-title').show();
            var caseId = pageObject.caseId;
            CaseController.findById(caseId, function (data) {
                if (data[0] == null) {
                    data = CaseController.encodeItem(data);
                    caseObject = data;

                } else {
                    data = data[0].kase;
                    caseObject = data;
                }
                //load contact object for case
                if (caseObject.party) {
                    if (caseObject.party.id && caseObject.party.type) {
                        var name = "";
                        if (caseObject.party.type.toLowerCase() == "person") {
                            var names = caseObject.party.name.split(" ");
                            if (names[1]) {
                                name += names[1] + " ";
                            }
                            if (names[0]) {
                                name += names[0];
                            }
                        } else if (caseObject.party.type.toLowerCase() == "company") {
                            name = caseObject.party.name;
                        }
                        self.setContact(caseObject.party.id, name);
                    }
                }
                self.loadCase();
                self.changeCaseDateTime();
            });
        } else {
            $('#case-add-title').show();
            self.setDefaultTime();
            if (pageObject && pageObject.parentId) {
                // get parent object
                caseParentId = pageObject.parentId;
                caseParentType = pageObject.parentType;
                var accessList = pageObject.accessList;
                var parentName = pageObject.parentName;
                if (accessList) {
                    caseObject = {"visible-to": accessList};
                    PermissionController.setData(caseObject["visible-to"]);
                }
                //load contact object for case
                self.setContact(caseParentId, parentName);
            }
        }
        pageObject = null;
        try {
            AutoCompleteController.setObject('responsible', window.localStorage.getItem('userId'));
            $autoCompleteResponsible.val(ALL_USERS[window.localStorage.getItem('userId')].name);
        } catch (e) {
            console.log('Responsible:' + JSON.stringify(e));
        }

        try {
            AutoCompleteController.setObject('owner', window.localStorage.getItem('userId'));
            $autoCompleteOwner.val(ALL_USERS[window.localStorage.getItem('userId')].name);
        } catch (e) {
            console.log('Responsible:' + JSON.stringify(e));
        }

    };

    self.updateACL = function () {
            var aclObjectVisibility = [];
            var request = {_id: new ObjectId(visibilityObjectId)};
            request['$or'] = [{'objectType': "Person"}, {'objectType': "Company"}];
            if (connection) {
                ClientAPI.find(request, null, function (err, data) {
                    if (data && data.length > 0) {
                        aclObjectVisibility = data[0]["accessList"];
                        if (aclObjectVisibility.length > 0) {
                            PermissionController.changeACL = true;
                            PermissionController.setData(aclObjectVisibility);
                        }
                    }
                });
            }
    };

    self.selectElement = function () {
        $caseGroupContainer = $('#case-group-container');
        $casePeopleContainer = $("#case-people-container");
        $editCaseStatus = $('#edit-case-status');
        $editCaseBody = $('#edit-case-body');
        $editCaseName = $('#edit-case-name');
        $autoCompleteOwner = $('#autocomplete-owner');
        $caseFrame = $('#edit-case #case-frame');
        $dateTimePicker = $('#edit-case #date-time-picker');
        $dateTimePickerContainer = $('#edit-case #case-date-time-picker-container');
    };

    self.bindElementControls = function () {
        $(document).on('click', '#case-save-link', self.saveCaseEvent);
        $(document).on('click', '#back-link', self.goBackEvent);
        $(document).on('click', '#edit-case-more-link', self.openEditCasePopup);
        $(document).on('click', '.ui-input-clear', '#case-contact-id-group', self.clearContactData);
        $('#edit-case .popup-menu-backdrop').on('click', function () {
            $('#edit-case .advanced-menu').hide();
            $(this).hide();
        });
        $(document).on('change', '#edit-case #case-frame', self.changeCaseDateTime);
        $(document).on('click', '#edit-case #add-case-link', self.addCaseLink);
        $(document).on('click', '#edit-case .remove-case-link', self.removeCaseLink);
        $(document).on('click', '#edit-case .additional-options', self.saveCaseEvent);
    };

    self.unbindElementControls = function () {
        $(document).off('click', '#case-save-link');
        $(document).off('click', '#back-link');
        $(document).off('click', '#edit-case-more-link');
        $(document).off('click', '.ui-input-clear', '#case-contact-id-group');
        $('#edit-case .popup-menu-backdrop').off('click');
        $(document).off('change', '#edit-case #case-frame');
        $(document).off('click', '#edit-case #add-case-link');
        $(document).off('click', '#edit-case .remove-case-link');
        $(document).off('click', '#edit-case .additional-options');
        caseObject = null;
        caseParentId = null;
        caseParentType = null;
    };

    self.addCaseLink = function () {
        $('#edit-case-link-table').append(
                '<tr class="case-link-row">'
                + '<td>'
                + '<select class="case-link-type" data-mini="true" data-theme="d">'
                + '<option value="Link" selected="selected" data-i18n="case.link"></option>'
                + '<option value="Video" data-i18n="case.video"></option>'
                + '</select>'
                + '</td>'
                + '<td><input class="case-link-val" type="text" data-mini="true" data-inset="true" value="" class="full-width" data-i18n="[placeholder]case.link"/>'
                + '</td>'
                + '<td><input class="case-link-desc" type="text" data-mini="true" data-inset="true" value="" class="full-width" data-i18n="[placeholder]case.description"/>'
                + '</td>'
                + '<td><a class="remove-case-link" href=""> <i class="md-remove  md-2x"></i></a></td>'
                + '</tr>');
        $("#edit-case-link-table").i18n();

        // Trigger create table
        $('#edit-case-link-table').trigger('create');
    };

    self.removeCaseLink = function (evt) {
        $(evt.currentTarget).parent().parent().remove();
    };

    self.changeCaseDateTime = function () {
        var now = new Date();
        var today = new Date(now.getUTCFullYear(), now.getUTCMonth(), now.getDate(), 23, 59, 0, 0);
        if ($caseFrame.val() == 'Specific') {
            $dateTimePickerContainer.show();
            $dateTimePicker.show();
        } else {
            $dateTimePickerContainer.hide();
            // $dateTimePicker.hide();
            if ($caseFrame.val() == "Today") {
                $dateTimePicker.val(UtilController.dateToStringMobiScroll(today));
            } else if ($caseFrame.val() == "Tomorrow") {
                var tomorrow = new Date(now.getTime() + (24 * 60 * 60 * 1000));
                tomorrow = new Date(tomorrow.getUTCFullYear(), tomorrow.getUTCMonth(), tomorrow.getDate(), 23, 59, 0, 0);
                $dateTimePicker.val(UtilController.dateToStringMobiScroll(tomorrow));
            } else if ($caseFrame.val() == "ThisWeek") {
                var thisWeek = new Date(now.getTime() + (24 * 60 * 60 * 1000 * (6 - now.getDay())));
                thisWeek = new Date(thisWeek.getUTCFullYear(), thisWeek.getUTCMonth(), thisWeek.getDate(), 23, 59, 0, 0);
                $dateTimePicker.val(UtilController.dateToStringMobiScroll(thisWeek));
            } else if ($caseFrame.val() == "NextWeek") {
                var nextWeek = new Date(now.getTime() + (24 * 60 * 60 * 1000 * (13 - now.getDay())));
                nextWeek = new Date(nextWeek.getUTCFullYear(), nextWeek.getUTCMonth(), nextWeek.getDate(), 23, 59, 0, 0);
                $dateTimePicker.val(UtilController.dateToStringMobiScroll(nextWeek));
            } else if ($caseFrame.val() == "Later") {
                $dateTimePicker.val('');
            }
        }
    };

    self.initDateTimePicker = function () {
        if (navigator.userAgent.match(/(iPad|iPhone)/)) {
            $dateTimePicker.attr('type', 'datetime-local');
        } else {
            $('#edit-case #disable-date-time').removeClass("disable-date-time-class");
            $dateTimePicker.mobiscroll().date({
                theme: 'android',
                display: 'modal',
                mode: 'scroller',
                dateOrder: 'dd MM yy',
                dateFormat: 'dd.mm.yy',
                timeFormat: 'HH:ii',
                timeWheels: 'HH:ii',
                maxDate: new Date(2025, 1, 1),
                minDate: new Date(2010, 1, 1)
            });
        }
    };

    self.setDefaultTime = function () {
        $dateTimePickerContainer.hide();
        $caseFrame.i18n();
        $caseFrame.val("Today").selectmenu("refresh");
        var now = new Date();
        var today = new Date(now.getUTCFullYear(), now.getUTCMonth(), now.getDate(), 23, 59, 0, 0);
        $dateTimePicker.val(UtilController.dateToStringMobiScroll(today));
    };

    self.openEditCasePopup = function () {
        $('#edit-case .popup-menu-backdrop').toggle();
        $('#edit-case .advanced-menu').css('z-index', '20000');
        $('#edit-case .advanced-menu').slideToggle('4000', 'swing');
    };

    self.saveCaseEvent = function () {
        $("#edit-case .advanced-menu").hide();
        $("#edit-case .popup-menu-backdrop").hide();
        if (window.localStorage.getItem("enableSave") == "true") {
            showActionDialog(Messages.Case.saveCaseMessage, "Yes", self.saveCase, "No", function () {
            });
        } else {
            self.saveCase();
        }
    };

    self.setContact = function (id, name) {
        AutoCompleteController.setObject('contact', id);
        if (name != "") {
            $("#autocomplete-case-contact").val(name);
        }
    };

    self.clearContactData = function () {
        AutoCompleteController.setObject('contact', null);
    };

    self.goBackEvent = function () {
        if ($("#edit-case #edit-case-name").val() != "" || tinymce.activeEditor.getContent() != "") {
            if (caseObject == null) {
                showActionDialog("There is unsaved data left, would you like to save it?", "Yes", function () {
                    self.saveCase();
                }, "No", function () {
                    goBack();
                });
            } else {
                if (caseObject.name != $("#edit-case #edit-case-name").val() || htmlStrip(caseObject.background) != htmlStrip(tinymce.activeEditor.getContent())
                        || caseObject["owner-id"].text != AutoCompleteController.getObject('owner')
                        || caseObject["party-id"].text != AutoCompleteController.getObject('contact')) {
                    showActionDialog("There is unsaved data left, would you like to save it?", "Yes", function () {
                        self.saveCase();
                    }, "No", function () {
                        goBack();
                    });
                } else {
                    goBack();
                }
            }
        } else {
            goBack();
        }
    };

    self.loadCase = function () {
        if (caseObject != null) {
            if (typeof caseObject.background == "string") {
                setTimeout(function(){
                    tinymce.activeEditor.setContent(caseObject.background);
                }, 300);
            }

            if (caseObject['due-at'] != null && caseObject['due-at'].text != null) {
                var caseDate = new Date(caseObject['due-at'].text);
                caseDate.setHours(0, 0, 0, 0);
                var now = new Date();
                now.setHours(0, 0, 0, 0);
                var tomorrow = new Date(now.getTime() + (24 * 60 * 60 * 1000));
                tomorrow.setHours(0, 0, 0, 0);
                var thisWeek = new Date(now.getTime() + (6 - now.getDay()) * 24 * 60 * 60 * 1000);
                thisWeek.setHours(0, 0, 0, 0);
                var nextWeek = new Date(now.getTime() + (24 * 60 * 60 * 1000 * (13 - now.getDay())));
                nextWeek.setHours(0, 0, 0, 0);
                if (caseDate.valueOf() == now.valueOf()) {
                    $caseFrame.val("Today").selectmenu("refresh");
                } else if (caseDate.valueOf() == tomorrow.valueOf()) {
                    $caseFrame.val("Tomorrow").selectmenu("refresh");
                } else if (caseDate.valueOf() == thisWeek.valueOf()) {
                    $caseFrame.val("ThisWeek").selectmenu("refresh");
                } else if (caseDate.valueOf() == nextWeek.valueOf()) {
                    $caseFrame.val("NextWeek").selectmenu("refresh");
                } else {
                    $caseFrame.val("Specific").selectmenu("refresh");
                }

                var selectDate = "";
                if (navigator.userAgent.match(/(iPad|iPhone)/)) {
                    selectDate = caseObject['due-at'].text.replace('Z', '');
                } else {
                    var date = caseObject['due-at'].text.split('T')[0];
                    selectDate = date.split("-")[2] + '.' + date.split("-")[1] + '.' + date.split("-")[0];
                }

                $dateTimePicker.val(selectDate);
            } else {
                $caseFrame.val("Later").selectmenu("refresh");
            }

            $editCaseName.val(caseObject.name);
            if (caseObject['owner-id'] != null) {
                AutoCompleteController.setObject('owner', caseObject['owner-id'].text);
                $autoCompleteOwner.val(ALL_USERS[caseObject['owner-id'].text].name);
            }
            PermissionController.setData(caseObject['visible-to']);
        } else {
            $("#case-add-title").show();
            AutoCompleteController.setObject('owner', window.localStorage.getItem('userId'));
            $autoCompleteOwner.val(ALL_USERS[window.localStorage.getItem('userId')].name);
        }

        if (caseObject.link) {
            var caseLinkHtml = "";
            if (caseObject.link instanceof  Array) {
                for (var i = 0; i < caseObject.link.length; i++) {
                    caseLinkHtml += "<tr class='case-link-row'>"
                            + "<td>"
                            + "<select class='case-link-type' data-mini='true' data-theme='d'>";
                    switch (caseObject.link[i].urlType) {
                        case "link":
                            caseLinkHtml += "<option value='Link' selected='selected'>Link</option>"
                                    + "<option value='Video'>Video</option>";
                            break;
                        case "video":
                            caseLinkHtml += "<option value='Link'>Link</option>"
                                    + "<option value='Video' selected='selected'>Video</option>";
                            break;
                    }
                    caseLinkHtml += "</select>"
                            + "</td>"
                            + "<td><input type='text' class='case-link-val' value='" + caseObject.link[i].url + "'/></td>"
                            + "<td><input type='text' class='case-link-desc' value='" + caseObject.link[i].urlDesc + "'/></td>"
                            + "<td><a class='remove-case-link' href=''> <i class='md-remove  md-2x'></i></a></td>"
                            + "</tr>";
                }
            } else {
                caseLinkHtml += "<tr class='case-link-row'>"
                        + "<td>"
                        + "<select class='case-link-type' data-mini='true' data-theme='d'>";
                switch (caseObject.link.urlType) {
                    case "link":
                        caseLinkHtml += "<option value='Link' selected='selected'>Link</option>"
                                + "<option value='Video'>Video</option>";
                        break;
                    case "video":
                        caseLinkHtml += "<option value='Link'>Link</option>"
                                + "<option value='Video' selected='selected'>Video</option>";
                        break;
                }
                caseLinkHtml += "</select>"
                        + "</td>"
                        + "<td><input type='text' class='case-link-val' value='" + caseObject.link.url + "'/></td>"
                        + "<td><input type='text' class='case-link-desc' value='" + caseObject.link.urlDesc + "'/></td>"
                        + "<td><a class='remove-case-link' href=''> <i class='md-remove  md-2x'></i></a></td>"
                        + "</tr>";
            }
            $('#edit-case #edit-case-link-table').append(caseLinkHtml).trigger('create');
        }
    };

    self.saveCase = function () {
        showLoadingIndikator();
        playSound("www/libs/blop.mp3");
        var caseName = $.trim($editCaseName.val());
        var caseBody = $.trim(tinymce.activeEditor.getContent());
        var caseOwnerId = AutoCompleteController.getObject('owner');
        var participantId = AutoCompleteController.getObject('contact');
        var dueAt = null;
        var caseLinks = [];
        var caseLinkRows = $("#edit-case .case-link-row");
        console.log("Case participant ID");
        console.log(participantId);
        $(this).addClass('not-clickable');

        if (caseBody == "") {
            alert(Messages.Case.backgroundInfoErr);
            hideLoadingIndikator();
            return;
        }

        if ($caseFrame.val() != "Later") {
            if (navigator.userAgent.match(/(iPad|iPhone)/)) {
                dueAt = new Date($dateTimePicker.val());
            } else {
                dueAt = new Date(parseInt($dateTimePicker.val().split(".")[2].split(" ")[0]),
                        parseInt($dateTimePicker.val().split(".")[1] - 1),
                        parseInt($dateTimePicker.val().split(".")[0]), 23, 59, 0);
            }
        }

        for (var i = 0; i < caseLinkRows.length; i++) {
          if (UtilController.validateURL($.trim($(caseLinkRows[i]).find('.case-link-val').val()))) {
              caseLinks.push({url: $.trim($(caseLinkRows[i]).find('.case-link-val').val()), urlDescription: $.trim($(caseLinkRows[i]).find('.case-link-desc').val()), urlType: $.trim($(caseLinkRows[i]).find('.case-link-type option:selected').val().toLowerCase())});
          } else {
            alert("Link is not valid!");
            hideLoadingIndikator();
            $(this).removeClass('not-clickable');
            return;
          }
        }

        if (caseName != "") {
            var groupId = null;
            var visibleTo = PermissionController.permission;

            if (caseObject && caseObject.id) {
                if (connection) {
                    if (!PermissionController.changeACL) {
                        visibleTo = null;
                    }
                    CaseController.update(caseObject.id.text, caseName, caseBody, caseOwnerId, visibleTo, null, participantId, dueAt, $caseFrame.val(), caseLinks, function (data) {
                        hideLoadingIndikator();
                        goBack();
                    });
                }
            } else {
                CaseController.save(caseName, caseBody, caseOwnerId, visibleTo, null, participantId, dueAt, caseLinks, function (data) {
                    hideLoadingIndikator();
                    if (data != null) {
                        if (data._id != null) {
                            var caseObj = CaseController.encodeItem(data);
                        } else {
                            if (data[0] != null) {
                                caseObj = data[0];
                            } else {
                                caseObj = data;
                            }
                        }
                        gotoPage({url: "viewCase", param: {caseId: caseObj.id.text}});
                    } else {
                        goBack();
                    }
                });
            }
        } else {
            alert('Case name is empty!!!');
        }
        caseObject = null;
    };

    return self;
}(EditCaseFunctions || {}));
