$(document).on('pageinit', '#preview-contact', function () {
    PreviewContactFunctions.init();
});

$(document).on('pageshow', '#preview-contact', function () {
    PreviewContactFunctions.bindElementControls();
});

$(document).on('pagehide', '#preview-contact', function () {
    PreviewContactFunctions.unbindElementControls();
});


var PreviewContactFunctions = (function (PreviewContactFunctions)
{
    var self = PreviewContactFunctions;
    var emailLocation = null;
    var phoneLocation = null;
    var webLocation = null;
    var $editContactPhoneTable = null;
    var $editContactEmailTable = null;
    var $editContactWebTable = null;
    var $autocompleteCompany = null;
    var previewContact = null;
    var previewType = null;
    var vCard = null;

    self.init = function () {
        PermissionController.init('#preview-contact');
        AutoCompleteController.init('#preview-contact');
        AutoCompleteController.addViewId('autocomplete-company', 'autocomplete-company-list', 'autocomplete-company');

        $editContactPhoneTable = $('#edit-contact-phone-table');
        $editContactEmailTable = $('#edit-contact-email-table');
        $editContactWebTable = $('#edit-contact-web-table');
        $autocompleteCompan = $('#autocomplete-company');

        $('#title-box').hide();

        if (pageObject) {
            vCard = pageObject.vCard;
            if (vCard) {
                var t, e, w;
                previewContact = {};
                previewContact.organizations = [];

                var phoneNumbers = [];
                var email = [];
                var webs = [];

                if ((vCard['n']['firstName'] != null && $.trim(vCard['n']['firstName']) != "") || (vCard['n']['lastName'] != null && $.trim(vCard['n']['lastName']) != "")) {
                    previewType = 'Person';

                    var name = {};
                    name['givenName'] = vCard['n']['firstName'];
                    name['familyName'] = vCard['n']['lastName'];
                    previewContact.name = name;

                    previewContact.organizations.push({
                        title: vCard.title,
                        name: vCard.org,
                        type: 'work'
                    });
                } else {
                    previewType = 'company';
                    previewContact['displayName'] = vCard.org;
                }

                for (t in vCard.tel) {
                    phoneNumbers.push({
                        type: vCard.tel[t].type,
                        value: vCard.tel[t].value
                    });
                }

                for (e in vCard.emails) {
                    email.push({
                        type: vCard.emails[e].type,
                        value: vCard.emails[e].value
                    });
                }


                for (w in vCard.webs) {
                    webs.push({
                        type: vCard.webs[w].type,
                        value: vCard.webs[w].value
                    });
                }

                previewContact.note = vCard.note;
                previewContact.phoneNumbers = phoneNumbers;
                previewContact.emails = email;
                previewContact.webs = webs;
                self.drawViewContact();

            } else {
                var parentId = pageObject.parentId;
                var accessList = pageObject.accessList;
                var phoneContactId = pageObject.phoneContactId;
                previewType = pageObject.contactType;

                if (phoneContactId) {
                    ContactsController.findPhoneContactById(phoneContactId, function (data) {
                        previewContact = data;
                        console.log(JSON.stringify(previewContact));
                        self.drawViewContact();
                    });
                }

                if (parentId) {
//                    if (COMPANY) {
//                        var company = COMPANY.filter(function (item) {
//                            return item.id.text == parentId;
//                        });
//                        if (company[0] != null) {
//                            $('#autocomplete-company').val(company[0].name);
//                            AutoCompleteController.setObject('Company', parentId);
//                        }
//                    }
                }

                if (accessList) {
                    contact = {people: {"visible-to": accessList}};
                    PermissionController.setData(contact.people["visible-to"]);
                }
            }
        }
        pageObject = null;
    };

    self.unbindElementControls = function () {
        $(document).off('click', '#back-link');
        $(document).off('click', '#edit-contact-phone-table .remove-contact-phone');
        $(document).off('click', '#edit-contact-phone-table .remove-contact-email');
        $(document).off('click', '#edit-contact-phone-table .remove-contact-web');
        $(document).off('click', '#save-link');
        $(document).off('click', '#add-contact-phone');
        $(document).off('click', '#preview-contact #preview-contact-more-link');
        $(document).off('click', '#preview-contact .popup-menu-backdrop');
        $(document).off('click', '#add-contact-email');
        previewContact = null;
    };

    self.bindElementControls = function () {
        $(document).on('click', '#back-link', goBack);
        $(document).on('click', '.remove-contact-phone', self.removeContactAddress);
        $(document).on('click', '.remove-contact-email', self.removeContactAddress);
        $(document).on('click', '.remove-contact-web', self.removeContactAddress);
        $(document).on('click', '#preview-contact #preview-contact-more-link', self.openOptionsPopup);
        $(document).on('click', '#preview-contact .popup-menu-backdrop', function(){
            $("#preview-contact #preview-contact-popup").hide();
            $(this).hide();
        });

        $('#save-link').on('click', self.contactSaveButton);
        $('#add-contact-phone').on('click', self.addContactPhone);
        $('#add-contact-email').on('click', self.addContactEmail);
        $('#add-contact-web').on('click', self.addContactWebAddress);
    };

    self.removeContactAddress = function (evt) {
        $(evt.currentTarget).parent().parent().remove();
    };

    // event dodavanje tel broja
    self.addContactPhone = function () {
        $editContactPhoneTable.append(
                '<tr class="contact-phone-row">'
                + '<td class="locationClass">' + phoneLocation + '</td>'
                + '<td><input class="contact-phone-number" type="tel" data-mini="true" value="" class="full-width"/></td>'
                + '<td><a class="remove-contact-phone" href=""><i class="md-remove  md-2x"></i></a></td>'
                + '</tr>');

        // Trigger create table
        $editContactPhoneTable.trigger('create');
    };

    // event dodavanje email-a
    self.addContactEmail = function () {
        $editContactEmailTable.append(
                '<tr class="contact-email-row">'
                + '<td class="locationClass">' + emailLocation + '</td>'
                + '<td><input class="contact-email-adress" type="email" data-mini="true" value=""/></td>'
                + '<td><a class="remove-contact-email" href=""><i class="md-remove  md-2x"></i></a></td>'
                + '</tr>');

        // Trigger create table
        $editContactEmailTable.trigger('create');
    };

    // event dodavanje web addresse
    self.addContactWebAddress = function () {
        $editContactWebTable.append(
                '<tr class="contact-web-row">'
                + '<td>' + webLocation + '</td>'
                + '<td><input class="contact-web-adress" type="text" data-mini="true" value="" class="full-width"/></td>'
                + '<td><a class="remove-contact-web" href=""><i class="md-remove  md-2x"></i></a></td>'
                + '</tr>');

        // Trigger create table
        $editContactWebTable.trigger('create');
    };

    self.contactSaveButton = function () {
        if (ViewContactFunctions.sharedContactForOpen != null) {
            ViewContactFunctions.sharedContactForOpen.new = true;
        } else {
            ViewContactFunctions.sharedContactForOpen = {};
            ViewContactFunctions.sharedContactForOpen.new = true;
        }
        EditContactFunctions.savePerson();
        return false;
    };

    self.drawViewContact = function () {
        var i, j;
        phoneLocation =
                '<select class="contact-phone-location" data-mini="true" data-theme="d">'
                + '<option value="Work">Work</option>'
                + '<option value="Mobile">Mobile</option>'
                + '<option value="Fax">Fax</option>'
                + '<option value="Pager">Pager</option>'
                + '<option value="Home">Home</option>'
                + '<option value="Skype">Skype</option>'
                + '<option value="Other">Other</option>'
                + '</select>';

        emailLocation =
                '<select class="contact-email-location" data-mini="true" data-theme="d">'
                + '<option value="Work">Work</option>'
                + '<option value="Home">Home</option>'
                + '<option value="Other">Other</option>'
                + '</select>';

        webLocation =
                '<select class="contact-web-location" data-mini="true" data-theme="d">'
                + '<option value="Work">Work</option>'
                + '<option value="Personal">Personal</option>'
                + '<option value="Other">Other</option>'
                + '</select>';

        if (previewType === 'Person') {
            if (previewContact.givenName) {
                previewContact.name = {
                    givenName: previewContact.givenName,
                    familyName: previewContact.familyName
                };
            };

            if (previewContact.name != null) {
                $('#contact-first-name').val(previewContact.name.givenName);
                $('#contact-last-name').val(previewContact.name.familyName);
            }
            var title = "";
            var organization = "";
            if (previewContact.organizations != null) {
                for (j = 0; j < previewContact.organizations.length; j++) {
                    title = (previewContact.organizations[j].title == null) ? "" : previewContact.organizations[j].title;
                    organization = (previewContact.organizations[j].name == null) ? "" : previewContact.organizations[j].name;
                }
            }

            if (previewContact.info != null && previewContact.info != "") {
                previewContact.note = previewContact.info;
            }

            $('#contact-background').val(previewContact.note);
            $('#edit-contact-person').show();
            for (i = 0; i < COMPANY.length; i++) {
                if (organization == COMPANY[i].name) {
                    $('#autocomplete-company').val(COMPANY[i].name);
                    AutoCompleteController.setObject('Company', COMPANY[i].id.text);
                    $('#contact-title').val(title);
                }
            }
        } else if (previewType === 'company') {
            $('#contact-name').val(previewContact.displayName);
            $('#contact-background').val(previewContact.note);
            $('#edit-contact-company').show();
        }

        // dinamicko kreiranje tabele tel brojeva
        var phones = [];
        if (previewContact != null) {
            phones = previewContact.phoneNumbers;
            if (previewContact['phone-number'] != null) {
                phones = previewContact['phone-number'];
            }
        }
        if (phones != null) {
            for (i = 0; i < phones.length; i++) {

                if (phones[i].phoneNumber) {
                    phones[i].value = phones[i].phoneNumber;
                }

                $('#edit-contact-phone-table').append(
                        '<tr class="contact-phone-row">'
                        + '<td>' + phoneLocation + '</td>'
                        + '<td><input class="contact-phone-number" class="full-width" type="tel" data-mini="true" data-theme="d" value="' + phones[i].value + '"/></td>'
                        + '<td><a class="remove-contact-phone" href=""><i class="md-remove  md-2x"></i></a></td>'
                        + '</tr>');
            }
        }

        var contactPhoneLocations = $('.contact-phone-location');
        if (phones != null) {
            for (i = 0; i < phones.length; i++) {
                if (phones[i].numberType) {
                    phones[i].type = phones[i].numberType;
                }
                // default selekcija tel lokacije
                $(contactPhoneLocations[i]).val(self.capitalize(phones[i].type));
            }
        }

        // dinamicko kreiranje tabele email-ova
        var emails = [];
        if (previewContact != null) {
            emails = previewContact.emails;
            if (previewContact['email-address'] != null) {
                emails = previewContact['email-address'];
            }
        }

        if (emails != null) {
            for (i = 0; i < emails.length; i++) {

                if (emails[i].eMailAddress) {
                    emails[i].value = emails[i].eMailAddress;
                }

                $('#edit-contact-email-table').append(
                        '<tr class="contact-email-row">'
                        + '<td>' + emailLocation + '</td>'
                        + '<td><input class="contact-email-adress" type="email" data-mini="true" data-theme="d" value="' + emails[i].value + '"/></td>'
                        + '<td><a class="remove-contact-email" data-role="button" data-iconpos="notext">Remove</a></td>'
                        + '</tr>');
            }
        }
        var contactEmailLocations = $('.contact-email-location');
        if (emails != null) {
            for (i = 0; i < emails.length; i++) {
                // default selekcija email lokacije
                if (emails[i].eMailAddressType) {
                    emails[i].type = emails[i].eMailAddressType;
                }
                $(contactEmailLocations[i]).val(self.capitalize(emails[i].type));
            }
        }

        // dinamicko kreiranje tabele email-ova
        var webs = [];
        if (previewContact) {
            webs = previewContact.webs || previewContact.urls;
        }
        if (webs != null) {
            for (i = 0; i < webs.length; i++) {
                $('#edit-contact-web-table').append(
                        '<tr class="contact-web-row">'
                        + '<td>' + webLocation + '</td>'
                        + '<td><input class="contact-web-adress" type="text" data-mini="true" data-theme="d" value="' + webs[i].value + '"/></td>'
                        + '<td><a class="remove-contact-web" data-role="button" data-iconpos="notext">Remove</a></td>'
                        + '</tr>');
            }
        }

        var contactWebLocations = $('.contact-web-location');
        if (webs != null) {
            for (i = 0; i < webs.length; i++) {
                // default selekcija email lokacije
                $(contactWebLocations[i]).val(self.capitalize(webs[i].type));
            }
        }

        // Trigger create content
        $('#edit-contact-content').trigger('create').show();

        hideLoadingIndikator();
    };

    self.capitalize = function (txt) {
        if (txt != null && txt.charAt(0) != null) {
            return txt.charAt(0).toUpperCase() + txt.slice(1);
        } else {
            return txt;
        }
    };
    
    self.openOptionsPopup = function(){
        $('#preview-contact .popup-menu-backdrop').toggle();
        $('#preview-contact .advanced-menu').css('z-index', '20000');
        $('#preview-contact .advanced-menu').slideToggle('4000', 'swing');
    };

    return self;
}(PreviewContactFunctions || {}));
