$(document).on('pageinit', '#edit-deal', function () {
    EditDealFunctions.init();
});

$(document).on('pageshow', '#edit-deal', function () {
    EditDealFunctions.bindElementControls();
    //initiate richtext editor
    if (tinymce.editors.length > 0) {
        tinymce.editors = [];
    }
    tinymce.init({
        mode: "exact",
        selector: "#deal-description",
        autoresize_min_height: 250,
        menubar: false,
        content_css: "styles/tinymce-custom.css",
        statusbar: false,
        skin: 'light',
        plugins: "autoresize",
        toolbar: [
            "bold italic underline bullist numlist outdent indent"
        ]
    });
});

$(document).on('pagehide', '#edit-deal', function () {
    EditDealFunctions.unbindElementControls();
});

var EditDealFunctions = (function (EditDealFunctions)
{
    var self = EditDealFunctions;
    var deal = null;
    var $autoCompleteOwner = null;
    var $dateTimePickerDeal = null;
    var $dealName = null;
    var $probability = null;
    var $dealDescription = null;
    var $dealCurrency = null;
    var $dealCategoryId = null;
    var $dealPrice = null;
    var $dealContactId = null;
    var $editDealContent = null;
    var $autoCompleteContact = null;
    var html, i, max;
    var parentId = null;
    var parentType = null;
    var visibilityObjectId = null;
    var userOwnerId = null;
    var dealParentName = null;

    self.init = function () {
        self.selecteElement();
        PermissionController.init('#edit-deal');
        AutoCompleteController.init('#edit-deal');
        AutoCompleteController.addViewId('autocomplete-owner', 'autocomplete-owner-list', 'autocomplete-owner');
        AutoCompleteController.addViewId('autocomplete-contact', 'autocomplete-contact-list', 'autocomplete-contact', function (data) {
            setTimeout(function () {
                visibilityObjectId = data.id;
                showActionDialog('Take visibility of parent object?', 'Yes', self.updateACL, "No", function () {
                });
            }, 2000);
        });

        if (navigator.userAgent.match(/(iPad|iPhone)/)) {
            $dateTimePickerDeal.attr('type', 'datetime-local');
        } else {
            $('#disable-date-time-deal').removeClass("disable-date-time-class");
            $dateTimePickerDeal.mobiscroll().date({
                theme: 'android',
                display: 'modal',
                mode: 'scroller',
                dateOrder: 'dd MM yy',
                dateFormat: 'dd.mm.yy',
                maxDate: new Date(2025, 1, 1),
                minDate: new Date(2010, 1, 1)
            });
        }
        if (pageObject && pageObject.id) {
            $('#deal-edit-title').show();
            DealController.findById(pageObject.id, function (data) {
                deal = data;
                //load contact object
                if (deal.party) {
                    if (deal.party.id && deal.party.type) {
                        var name = "";
                        switch (deal.party.type.toLowerCase()) {
                            case "person":
                                var names = deal.party.name.split(" ");
                                if (names[1]) {
                                    name += names[1] + " ";
                                }
                                if (names[0]) {
                                    name += names[0];
                                }
                                break;
                            case "company":
                                name = deal.party.name;
                                break;
                        }
                        self.setContact(deal.party.id, name);
                    }
                }
                self.loadDeal();
            });
        } else {
            if (pageObject && pageObject.parentId) {
                $('#deal-add-title').show();
                parentId = pageObject.parentId;
                parentType = pageObject.parentType;
                dealParentName = pageObject.parentName;
                self.loadDeal();
            } else if (pageObject && pageObject.ownerId) {
                userOwnerId = pageObject.ownerId;
                pageObject = null;
            } else {
                $('#deal-add-title').show();
                self.loadDealCategory();
                self.setToCurrentDateTime();
            }
        }
        pageObject = null;

        try {
            AutoCompleteController.setObject('responsible', window.localStorage.getItem('userId'));
            $autoCompleteResponsible.val(ALL_USERS[window.localStorage.getItem('userId')].name);
        } catch (e) {
            console.log('Responsible:' + JSON.stringify(e));
        }

        try {
            if (userOwnerId) {
                AutoCompleteController.setObject('owner', userOwnerId);
                $autoCompleteOwner.val(ALL_USERS[userOwnerId].name);
            } else {
                AutoCompleteController.setObject('owner', window.localStorage.getItem('userId'));
                $autoCompleteOwner.val(ALL_USERS[window.localStorage.getItem('userId')].name);
            }

        } catch (e) {
            console.log('Responsible:' + JSON.stringify(e));
        }


    };

    self.loadDealCategory = function () {
        var html = '';
        for (i = 0, max = dealCategories.length; i < max; i++) {
            var key;
            switch (dealCategories[i].name) {
                case "Print project":
                    key = "deal.printProject";
                    break;
                case "Support":
                    key = "deal.support";
                    break;
                case "Professional services":
                    key = "deal.professionalServices";
                    break;
                case "Third-party resseling":
                    key = "deal.thirdPartyResseling";
                    break;
                case "Own product resseling":
                    key = "deal.ownProductResseling";
                    break;
                case "Education":
                    key = "deal.education";
                    break;
                    default:
                    key = null;
                    break;
            }
            if (key) {
              html += '<option value="' + dealCategories[i].id.text + '" data-i18n="' + key + '"> </option>';
            } else {
              html += '<option value="' + dealCategories[i].id.text + '">' + dealCategories[i].name + '</option>';
            }
        }
        $dealCategoryId.empty().append(html);
        $dealCategoryId.i18n();
        $dealCategoryId.selectmenu('refresh');
    };

    self.updateACL = function () {
            var aclObjectVisibility = [];
            var request = {_id: new ObjectId(visibilityObjectId)};
            request['$or'] = [{'objectType': "Person"}, {'objectType': "Company"}];
            if (connection) {
                ClientAPI.find(request, null, function (err, data) {
                    if (data && data.length > 0) {
                        aclObjectVisibility = data[0]['accessList'];
                        if (aclObjectVisibility.length > 0) {
                            PermissionController.changeACL = true;
                            PermissionController.setData(aclObjectVisibility);
                        }
                    }
                });
            }
    };

    self.selecteElement = function () {
        $autoCompleteOwner = $('#autocomplete-owner');
        $dateTimePickerDeal = $('#date-time-picker-deal');
        $dealName = $('#deal-name');
        $probability = $('#probability');
        $dealDescription = $('#deal-description');
        $dealCurrency = $('#deal-currency');
        $dealCategoryId = $('#deal-category-id');
        $dealPrice = $('#deal-price');
        $dealContactId = $('#deal-contact-id');
        $editDealContent = $('#edit-deal-content');
        $autoCompleteContact = $('#autocomplete-contact');
    };

    self.unbindElementControls = function () {
        $('#deal-save-button').off('click');
        $dealPrice.off('keypress');
        $(document).off('click', '#deal-edit-back-button');
        $(document).off('click', '#edit-deal-more-link');
        $('#edit-deal .popup-menu-backdrop').off('click');
        $(document).off('click', '#edit-deal #add-deal-link');
        $(document).off('click', '#edit-deal .remove-deal-link');
        $(document).off('click', '#edit-deal .additional-options');
        deal = null;
        parentId = null;
        parentType = null;
        userOwnerId = null;
        dealParentName = null;
    };

    self.bindElementControls = function () {
        $('#deal-save-button').on('click', self.dealSaveEvent);
        $dealPrice.on('keypress', EditContactFunctions.validationTel);
        $(document).on('click', '#deal-edit-back-button', self.goBackLinkEvent);
        $(document).on('click', '.ui-input-clear', '#deal-contact-id-group', self.clearContactData);
        $(document).on('click', '#edit-deal-more-link', self.openeditDealPopup);
        $('#edit-deal .popup-menu-backdrop').on('click', function () {
            $('#edit-deal .advanced-menu').hide();
            $(this).hide();
        });
        $(document).on('click', '#edit-deal #add-deal-link', self.addDealLink);
        $(document).on('click', '#edit-deal .remove-deal-link', self.removeDealLink);
        $(document).on('click', '#edit-deal .additional-options', self.dealSaveEvent);
    };

    self.addDealLink = function () {
        $('#edit-deal-link-table').append(
                '<tr class="deal-link-row">'
                + '<td>'
                + '<select class="deal-link-type" data-mini="true" data-theme="d">'
                + '<option value="Link" selected="selected">Link</option>'
                + '<option value="Video">Video</option>'
                + '</select>'
                + '</td>'
                + '<td><input class="deal-link-val ui-input-text ui-body-c" type="text" data-mini="true" data-inset="true" value="" class="full-width" data-i18n="[placeholder]deal.link"/>'
                + '</td>'
                + '<td><input class="deal-link-desc ui-input-text ui-body-c" type="text" data-mini="true" data-inset="true" value="" class="full-width" data-i18n="[placeholder]deal.description"/>'
                + '</td>'
                + '<td><a class="remove-deal-link" href=""> <i class="md-remove  md-2x"></i></a></td>'
                + '</tr>').i18n();
        // Trigger create table
        $('#edit-deal-link-table').trigger('create');
    };

    self.removeDealLink = function (evt) {
        $(evt.currentTarget).parent().parent().remove();
    };

    self.openeditDealPopup = function () {
        $('#edit-deal .popup-menu-backdrop').toggle();
        $('#edit-deal .advanced-menu').css('z-index', '20000');
        $('#edit-deal .advanced-menu').slideToggle('4000', 'swing');
    };

    self.clearContactData = function () {
        AutoCompleteController.setObject('contact', null);
    };

    self.goBackLinkEvent = function () {
        if ($("#edit-deal #deal-name").val() != "") {
            if (deal == null) {
                showActionDialog("There is unsaved data left, would you like to save it?", "Yes", function () {
                    self.dealSaveEvent();
                }, "No", function () {
                    goBack();
                });
            } else {
                if (deal.name != $("#edit-deal #deal-name").val() || htmlStrip(deal.background) != htmlStrip(tinymce.activeEditor.getContent())) {
                    showActionDialog("There is unsaved data left, would you like to save it?", "Yes", function () {
                        self.dealSaveEvent();
                    }, "No", function () {
                        goBack();
                    });
                } else {
                    goBack();
                }
            }
        } else {
            goBack();
        }
    };

    self.dealSaveEvent = function () {
        playSound("www/libs/blop.mp3");
        var dealTmpObject = deal;
        var name = null;
        var info = null;
        var categoryId = null;
        var currency = null;
        var value = null;
        var valueType = null;
        var assignedTo = null;
        var participantId = AutoCompleteController.getObject('contact');
        var ownerId = AutoCompleteController.getObject('owner');
        var probability = null;
        var closeDate = null;
        var tempDate = $dateTimePickerDeal.val();
        var referenceId = $('#reference-id', '#edit-deal').val();
        var dealLinks = [];
        var dealLinkRows = $("#edit-deal .deal-link-row");
        $(this).addClass('not-clickable');

        // get deal links
        for (var i = 0; i < dealLinkRows.length; i++) {
          if (UtilController.validateURL($.trim($(dealLinkRows[i]).find('.deal-link-val').val()))) {
              dealLinks.push({url: $.trim($(dealLinkRows[i]).find('.deal-link-val').val()), urlDescription: $.trim($(dealLinkRows[i]).find('.deal-link-desc').val()), urlType: $.trim($(dealLinkRows[i]).find('.deal-link-type option:selected').val().toLowerCase())});
          } else {
            alert("Link is not valid!");
            hideLoadingIndikator();
            $(this).removeClass('not-clickable');
            return;
          }
        }

        $("#edit-deal .advanced-menu").hide();
        $("#edit-deal .popup-menu-backdrop").hide();
        if (!isNaN($probability.val())) {
            probability = $probability.val();
        }
        if (tempDate && $.trim(tempDate) != "") {
            if (navigator.userAgent.match(/(iPad|iPhone)/)) {
                closeDate = new Date(tempDate);
            } else {
                closeDate = new Date(parseInt(tempDate.split(".")[2].split(" ")[0]),
                        parseInt(tempDate.split(".")[1] - 1),
                        parseInt(tempDate.split(".")[0]),
                        0, 0, 0);
            }
        }

        if ($.trim($dealName.val()) == '') {
            alert("Please provide a name for this deal!");
            return false;
        } else {
            name = $dealName.val();
        }

        info = tinymce.activeEditor.getContent();

        if ($dealCategoryId.val() != "None") {
            categoryId = $dealCategoryId.val();
        }

        if ($dealPrice.val() != null && $dealPrice.val() != "") {
            value = $dealPrice.val();
        }

        currency = $dealCurrency.val();
        valueType = 'Fixed';

        var visibleTo = PermissionController.permission;
        if (window.localStorage.getItem("enableSave") == "true") {
            showActionDialog('Save deal?', 'Yes', function () {
                if (dealTmpObject && dealTmpObject.id) {

                    if (!PermissionController.changeACL) {
                        visibleTo = null;
                    }
                    DealController.update(dealTmpObject.id.text, name, info, categoryId, currency, value, valueType, assignedTo, ownerId, participantId, visibleTo, probability, closeDate, referenceId, dealLinks, function (data) {
                        goBack();
                    });
                } else {
                    DealController.save(name, info, categoryId, currency, value, valueType, assignedTo, ownerId, participantId, visibleTo, probability, closeDate, referenceId, dealLinks, function (data) {
                        var dealObj = DealController.encodeItem(data[0]);
                        var dealId = dealObj.id.text;
                        gotoPage({url: "viewDeal", param: {id: dealId}});
                    });
                }
            }, "No", function () {
            });
        } else {
            if (deal && deal.id) {
                if (!PermissionController.changeACL) {
                    visibleTo = null;
                }
                DealController.update(deal.id.text, name, info, categoryId, currency, value, valueType, assignedTo, ownerId, participantId, visibleTo, probability, closeDate, referenceId, dealLinks, function (data) {
                    goBack();
                });
            } else {
                DealController.save(name, info, categoryId, currency, value, valueType, assignedTo, ownerId, participantId, visibleTo, probability, closeDate, referenceId, dealLinks, function (data) {
                    if (data != null && data[0] != null) {
                        var dealObj = DealController.encodeItem(data[0]);
                        var dealId = dealObj.id.text;
                        gotoPage({url: "viewDeal", param: {id: dealId}});
                    } else {
                        goBack();
                    }
                });
            }
        }

        deal = null;
    };



    self.loadDeal = function () {
        $dealName.empty();
        $dealPrice.empty();

        var userId;
        if (ViewContactFunctions.sharedContactForOpen != null && ViewContactFunctions.sharedContactForOpen.item != null && ViewContactFunctions.sharedContactForOpen.item.identity_id != null) {
            userId = ViewContactFunctions.sharedContactForOpen.item.identity_id;
        } else {
            userId = window.localStorage.getItem("userId");
        }
        AutoCompleteController.setObject('owner', userId);
        $autoCompleteOwner.val(ALL_USERS[userId].name);

        // deal kategorije
        self.loadDealCategory();

        if (!deal) {
            //init object visibility
            deal = {"visible-to": pageObject.accessList};
            PermissionController.setData(deal['visible-to']);

            //init parent Object
            if (parentId && parentType && dealParentName) {
                self.setContact(parentId, dealParentName);
            }
        } else {
            if (deal['probability'] && !isNaN(deal['probability'])) {
                $probability.val(deal['probability']).selectmenu("refresh");
            } else {
                $probability.val(0).selectmenu("refresh");
            }

            if (deal['closeDate']) {
                if (navigator.userAgent.match(/(iPad|iPhone)/)) {
                    $dateTimePickerDeal.val(deal['closeDate'].replace('Z', ''));
                } else {
                    $dateTimePickerDeal.val(UtilController.dateToStringMobiScroll(new Date(deal['closeDate']), true));
                }
            } else {
                self.setToCurrentDateTime();
            }

            if (deal['referenceId']) {
                $('#reference-id', '#edit-deal').val(deal.referenceId);
            }

            if (deal.link) {
                var dealLinkHtml = "";
                if (deal.link instanceof  Array) {
                    for (var i = 0; i < deal.link.length; i++) {
                        dealLinkHtml += "<tr class='deal-link-row'>"
                                + "<td>"
                                + "<select class='deal-link-type' data-mini='true' data-theme='d'>";
                        switch (deal.link[i].urlType) {
                            case "link":
                                dealLinkHtml += "<option value='Link' selected='selected'>Link</option>"
                                        + "<option value='Video'>Video</option>";
                                break;
                            case "video":
                                dealLinkHtml += "<option value='Link'>Link</option>"
                                        + "<option value='Video' selected='selected'>Video</option>";
                                break;
                        }
                        dealLinkHtml += "</select>"
                                + "</td>"
                                + "<td><input type='text' class='deal-link-val' value='" + deal.link[i].url + "'/></td>"
                                + "<td><input type='text' class='deal-link-desc' value='" + deal.link[i].urlDesc + "'/></td>"
                                + "<td><a class='remove-deal-link' href=''> <i class='md-remove  md-2x'></i></a></td>"
                                + "</tr>";
                    }
                } else {
                    dealLinkHtml += "<tr class='deal-link-row'>"
                            + "<td>"
                            + "<select class='deal-link-type' data-mini='true' data-theme='d'>";
                    switch (deal.link.urlType) {
                        case "link":
                            dealLinkHtml += "<option value='Link' selected='selected'>Link</option>"
                                    + "<option value='Video'>Video</option>";
                            break;
                        case "video":
                            dealLinkHtml += "<option value='Link'>Link</option>"
                                    + "<option value='Video' selected='selected'>Video</option>";
                            break;
                    }
                    dealLinkHtml += "</select>"
                            + "</td>"
                            + "<td><input type='text' class='deal-link-val' value='" + deal.link.url + "'/></td>"
                            + "<td><input type='text' class='deal-link-desc' value='" + deal.link.urlDesc + "'/></td>"
                            + "<td><a class='remove-deal-link' href=''> <i class='md-remove  md-2x'></i></a></td>"
                            + "</tr>";
                }
                $('#edit-deal #edit-deal-link-table').append(dealLinkHtml).trigger('create');
            }

            $dealName.val(deal.name);

            if (deal.background != null && deal.background.nil != null) {
                tinymce.activeEditor.setContent("");
            } else {
                tinymce.activeEditor.setContent(deal.background);
            }

            $dealCurrency.val(deal.currency).selectmenu('refresh');

            if (deal.price != null && deal.price.text != null) {
                $dealPrice.val(deal.price.text);
            }

            $('#deal-price-type').val(deal['price-type']).selectmenu('refresh');

            if (deal.category == null) {
                $dealCategoryId.val('None');
            } else {
                $dealCategoryId.i18n();
                $dealCategoryId.val(deal.category.id.text).selectmenu('refresh');
            }

            AutoCompleteController.setObject('owner', deal['owner-id'].text);
            $autoCompleteOwner.val(ALL_USERS[deal['owner-id'].text].name);

            PermissionController.setData(deal['visible-to']);
        }
    };

    self.setContact = function (id, parentName) {
        AutoCompleteController.setObject('contact', id);
        $autoCompleteContact.val(parentName);
    };

    self.setToCurrentDateTime = function () {
        if (navigator.userAgent.match(/(iPad|iPhone)/)) {
            $dateTimePickerDeal.val(new Date().toISOString().replace('Z', ''));
        } else {
            $dateTimePickerDeal.val(UtilController.dateToStringMobiScroll(new Date(), true));
        }
    };



    return self;
}(EditDealFunctions || {}));
