$(document).on('pageinit', '#effort', function () {
    EffortFunctions.init();

});

$(document).on('pageshow', '#effort', function () {
    EffortFunctions.bindElementControls();
});

$(document).on('pagehide', '#effort', function () {
    EffortFunctions.unbindElementControls();
});

var EffortFunctions = (function (EffortFunctions) {
    var self = EffortFunctions;
    var relatedId = null;
    var relatedType = null;
    var relatedCreationDate = null;
    var displayName = null;
    var $dateTimePicker = null;

    self.init = function () {
        if(pageObject){
            relatedId = pageObject.relatedId;
            relatedType = pageObject.relatedType;
            relatedCreationDate = pageObject.relatedCreationDate;
            displayName = pageObject.displayName;
            pageObject = null;
        }
        EffortService.loadEffort(relatedId, relatedType, "effort", "edit-effort-link-table", true, function(){
            self.addEffortRow();
            $("#effort-name").html(displayName);
        });
        $('.date-time-picker').attr('type', "");

    };

    self.keyboardShowHandler = function(e){
        $('#effort .additional-options').css('bottom', '190px');
    };

    self.keyboardHideHandler = function(e){
        $('#effort .additional-options').css('bottom', '30px');
    };

    self.bindElementControls = function () {
        $(document).on("click", "#effort #effort-back-link", goBack);
        $(document).on("click", "#effort #effort-more-link", self.openOptions);
        $('#effort .popup-menu-backdrop').on('click', function () {
            $("#effort .advanced-menu").hide();
            $(this).hide();
        });
        $(document).on("click", "#effort #add-effort-link", self.addEffortRow);
        $(document).on("click", "#effort .remove-effort-link", EffortService.removeEffortLink);
        $(document).on("click", "#effort #effort-save-link", self.saveEffort);
        $(document).on("click", "#effort .additional-options", self.saveEffort);
        if (navigator.userAgent.match(/(iPad|iPhone)/)) {
            window.addEventListener('native.showkeyboard', self.keyboardShowHandler);
            window.addEventListener('native.hidekeyboard', self.keyboardHideHandler);
        }
    };

    self.unbindElementControls = function () {
        $(document).off("click", "#effort #effort-back-link");
        $(document).off("click", "#effort #effort-more-link");
        $('#effort .popup-menu-backdrop').off('click');
        $(document).off("click", "#effort #add-effort-link");
        $(document).off("click", "#effort .remove-effort-link");
        $(document).off("click", "#effort #effort-save-link");
        $(document).off("click", "#effort .additional-options");
        if (navigator.userAgent.match(/(iPad|iPhone)/)) {
            window.removeEventListener('native.showkeyboard', self.keyboardShowHandler);
            window.removeEventListener('native.hidekeyboard', self.keyboardHideHandler);
        }
    };

    self.saveEffort = function () {
        EffortService.createOrUpdateEffort(relatedId, relatedType, displayName, relatedCreationDate, "effort", function (err, data) {
            if (err) {
                alert("Effort related error!");
            } else {
                goBack();
            }
        });
    };

    self.openOptions = function(){
        openOptionsPopup("effort");
    };

    self.addEffortRow = function () {
        EffortService.drawEffortTable('effort', 'edit-effort-link-table', new Date());
        $dateTimePicker = $("#effort .date-time-picker");
        self.initDateTimePicker();
        $('.date-time-picker').attr('type', "");
    };

    self.initDateTimePicker = function () {
        $dateTimePicker.off();
        var type = "android";
        if (navigator.userAgent.match(/(iPad|iPhone)/)) {
            type = "ios";
        }
        $('#disable-date-time').removeClass("disable-date-time-class");
        $dateTimePicker.mobiscroll().date({
            theme: type,
            display: 'modal',
            mode: 'scroller',
            dateOrder: 'dd MM yy',
            dateFormat: 'dd.mm.yy',
            timeFormat: 'HH:ii',
            timeWheels: 'HH:ii',
            maxDate: new Date(2025, 1, 1),
            minDate: new Date(2010, 1, 1)
        });
        $('.date-time-picker').attr('type', "");
    };

    return self;
}(EffortFunctions || {}));
