$(document).on('pageinit', '#overview-effort', function () {
    hideLoadingIndikator();
});

$(document).on('pageshow', '#overview-effort', function () {
    OverviewEffortFunctions.bindElementControls();
    OverviewEffortFunctions.init();
    OverviewEffortFunctions.setConnectionMode();
});

$(document).on('pagehide', '#overview-effort', function () {
    OverviewEffortFunctions.unbindElementControls();
});

var OverviewEffortFunctions = (function (OverviewEffortFunctions)
{
    var self = OverviewEffortFunctions;
    var FILTER_TYPE = {THIS_MONTH: 0, PREVIOUS_MONTH: 1, LAST_3_MONTHS: 2, THIS_YEAR: 3};
    var currentTaskFilterType = FILTER_TYPE.THIS_MONTH;
    var $totalEffort = null;
    var $filterEffortType = null;

    self.init = function () {
        self.filterEvent();
        $totalEffort = $("#total-effort");
        $filterEffortType = $("#filter-effort-type");
        $filterEffortType.i18n();
    };

    self.bindElementControls = function () {
        $(document).on('click', '#back-link', goBack);
        $('#filter-effort-type', '#overview-effort').on('change', self.filterEvent);
        $(document).on('click', '#overview-effort #total-effort li', self.allTaskItemEvent);
    };

    self.unbindElementControls = function () {
        $(document).off('click', '#back-link');
        $('#filter-effort-type', '#overview-effort').off('change');
        $(document).off('click', '#overview-effort #total-effort li');
    };

    self.filterEvent = function(event) {
      var filterValue;
      if (event) {
          filterValue = $(event.currentTarget).find('option:selected').val();
      } else {
          if (currentTaskFilterType === FILTER_TYPE.THIS_MONTH) {
              filterValue = "filter-this-month";
          } else if (currentTaskFilterType === FILTER_TYPE.PREVIOUS_MONTH) {
              filterValue = "filter-previous-month";
          } else if (currentTaskFilterType === FILTER_TYPE.LAST_3_MONTHS) {
              filterValue = "filter-last-three-month";
          } else if (currentTaskFilterType == FILTER_TYPE.THIS_YEAR) {
              filterValue = "filter-this-year";
          }
      }

     var startDate, endDate, now = new Date();

      switch (filterValue) {
        case "filter-this-month":
          currentTaskFilterType = FILTER_TYPE.THIS_MONTH;
          startDate = new Date(now.getFullYear(), now.getMonth(), 1, 0, 0, 0, 0);
          break;
        case "filter-previous-month":
          currentTaskFilterType = FILTER_TYPE.PREVIOUS_MONTH;
          if(now.getMonth() > 0){
              var daysInPreviousMonth = OverviewController.daysInMonth(now.getMonth(), now.getFullYear());
              startDate = new Date(now.getFullYear(), now.getMonth()-1, 1, 0, 0, 0, 0);
              endDate = new Date(now.getFullYear(), now.getMonth()-1, daysInPreviousMonth, 23, 59, 59, 0);
          }
          break;
        case "filter-last-three-month":
          currentTaskFilterType = FILTER_TYPE.LAST_3_MONTHS;
          if(now.getMonth() < 3){
              startDate = new Date(now.getFullYear(), 0, 1, 0, 0, 0, 0);
          }else{
              startDate = new Date(now.getFullYear(), now.getMonth()-3, 1, 0, 0, 0, 0);
          }
          break;
        case "filter-this-year":
          currentTaskFilterType = FILTER_TYPE.THIS_YEAR;
          startDate = new Date(now.getFullYear(), 0, 1, 0, 0, 0, 0);
          break;
        default:
      }
      self.loadEffortData(startDate, endDate);
    };

    self.loadEffortData = function(startDate, endDate) {
        showLoadingIndikator();
        if (connection) {
          self.findEfforts(startDate, endDate, function(err, data){
              if (!err) {
                  self.drawEffort(data);
              } else {
                hideLoadingIndikator();
              }
          });
        } else {
          hideLoadingIndikator();
          OverviewEffortFunctions.setConnectionMode();
        }
    };

    self.drawEffort = function(data) {
      var html = "";
        if (data) {
            for (var i = 0, max = data.length; i < max; i++) {
               html += self.drawEffortItem(data[i]);
            }
        }
        hideLoadingIndikator();
        if (html != "") {
          $totalEffort.empty().append(html)
        } else {
          $totalEffort.empty().append(emptyListHtml);
        }
    };

    self.drawEffortItem = function(item) {
        var htmlItem = "";
        var taskid =""
        var taskName = "";
        if (item && item.immediateParent && item.immediateParent._id) {
          taskid="taskid='" + item.immediateParent._id + "'";
          if (item.immediateParent.entityId) {
            taskName += "(" + item.immediateParent.entityId + ") ";
          }
          taskName += item.immediateParent.taskName;
        }

        htmlItem +=
                "<li " + taskid + " class='border-bottom'><div class='overview-effort-task-list'>"
                +"<div>"
                  + "<h3>" + taskName + "</h3>"
                + "</div>"
                + "<div class='overview-effort-description'>"
                  + "<span><b>Effort description:</b></span><br/>"
                  + "<span>" + item.efforts.description + "</span>"
                + "</div>"
                + "<div>"
                  + "<span><b>Hours:</b></span>"
                  + "<span>" + item.efforts.hours + "h</span>"
                + "</div>"
                + "<div>"
                  + "<span><b>Effort date:</b></span>"
                  + "<span>" + dateFormat(new Date(item.efforts.effortDate)) + "</span> | "
                  + "<span><b>Creation date:</b></span>"
                  + "<span>" + dateFormat(new Date(item.efforts.effort_creationDate)) + "</span>"
                + "</div>"
                + "</div></li>"
        return htmlItem;
    };

    self.allTaskItemEvent = function () {
        gotoPage({url: "viewTask", param: {id: $(this).attr('taskid')}}, {url: "overviewEffort", param: null});
    };

    self.findEfforts = function(startDate, endDate, callback){
        var effortsQuery = [
            {$match:  { type: "Effort" }
            },{$unwind: "$efforts"},
            { $match: { 'efforts.submitter_id': localStorage.getItem('userId')} }];

        if (startDate && !endDate) {
            effortsQuery[2].$match['efforts.effortDate'] = {
                $gte: startDate
            };
        }

        if(endDate && !startDate){
            effortsQuery[2].$match['efforts.effortDate'] = {
                $lte: endDate
            };
        }

        if(startDate && endDate){
            effortsQuery[2].$match['efforts.effortDate'] = {
                $gte: startDate,
                $lte: endDate
            };
        }
        ClientAPI.findEfforts({
            query: {pipeline: effortsQuery, sort: [["efforts.effortDate ", -1]]},
            sort: [ ["efforts.effortDate", -1 ]],
            relatedType: 'Task',
            aggregated: true
        }, callback);

      };

      self.setConnectionMode = function () {
        var $offlineMessageElement = $('#overview-effort #offline-dashboard-message');
          if (connection) {
              if ($offlineMessageElement.is(':empty') == false) {
                  $offlineMessageElement.empty();
              }
          } else {
              $offlineMessageElement
              .html('<p class="center"><img src="' + rootPath + '/images/noWifi.png" onerror="imgError(this);"></p>'
                      + '<p style="text-align:center;">' + Messages.Login.offlineMessage + '</p></div>');
              if ($totalEffort) {
                  $totalEffort.empty();
              }
          }
      };


    return self;
}(OverviewEffortFunctions || {}));
