$(document).on('pageinit', '#overview', function () {
    hideLoadingIndikator();
});

$(document).on('pageshow', '#overview', function () {
    removeCallListForIOS("main-nav-list", "overview", "call-list-link");
    OverviewFunctions.bindElementControls();
    OverviewFunctions.init();
    OverviewFunctions.setConnectionMode();
});

$(document).on('pagehide', '#overview', function () {
    OverviewFunctions.unbindElementControls();
});

var OverviewFunctions = (function (OverviewFunctions)
{
    var self = OverviewFunctions;
    var $myEffort = null;
    var $wonOpportunities = null;
    var $openOpportunities = null;
    var $closedTasks = null;

    self.init = function () {
        removeCallListForIOS("main-nav-list", "overview", "call-list-link");
        NavigationController.loadUserInfo('overview');
        self.initValues();
        if (connection) {
          self.initMyEffort();
          self.initMyOpenOpportunities();
          self.initMyWonOpportunities();
          self.initMyClosedTasks();
          self.initMyOverViewOpportunitiesAndProject();
          self.initTopSales();
          self.initTopClients();
        }
    };

    self.initValues =  function() {
        $myEffort = $('#my-effort');
        $wonOpportunities = $('#won-opportunities');
        $openOpportunities = $('#open-opportunities');
        $closedTasks = $("#closed-tasks");
    }

    self.bindElementControls = function(){
        $(document).on('click', '#menu-link', self.menuLinkEvent);
        $(document).on('click', '.backdrop', self.menuLinkEvent);
        $(document).on('click', '#owerview-link', self.menuLinkEvent);
        $(document).on('click', '#dashboard-link', self.dashboardLinkEvent);
        $(document).on('click', '#contacts-link', self.contactsLinkEvent);
        $(document).on('click', '#deals-link', self.dealsLinkEvent);
        $(document).on('click', '#case-link', self.caseLinkEvent);
        $(document).on('click', '#tasks-link', self.tasksLinkEvent);
        $(document).on('click', '#drafts-link', self.draftLinkEvent);
        $(document).on('click', '#mygroup-link', self.myGroupLinkEvent);
        $(document).on("click", "#app-calendar", self.calendarLinkEvent);
        $(document).on('click', '#call-list-link', self.gotoCallList);
        $(document).on('click', '#about-link', self.gotoAbout);
        $(document).on('click', '#settings-link', self.settingsLinkEvent);
        $(document).on('click', '#my-effort', self.overviewEffortLinkEvent);
    };

    self.unbindElementControls = function() {
        $(document).off('click', '#menu-link');
        $(document).off('click', '.backdrop');
        $(document).off('click', '#owerview-link');
        $(document).off('click', '#dashboard-link');
        $(document).off('click', '#contacts-link');
        $(document).off('click', '#deals-link');
        $(document).off('click', '#case-link');
        $(document).off('click', '#tasks-link');
        $(document).off('click', '#drafts-link');
        $(document).off('click', '#mygroup-link');
        $(document).off("click", "#app-calendar");
        $(document).off('click', '#call-list-link');
        $(document).off('click', '#about-link');
        $(document).off('click', '#settings-link');
        $(document).off('click', '#my-effort');
        $('html').removeClass('sidebar-left-in');
    };

    self.initMyClosedTasks = function(){
      OverviewController.findNumberOfClosedTaskPerProject(function(err, data){
        if (data && data.length > 0){
          $closedTasks.removeClass("overiview-item-hide").i18n();
          $('#closed-tasks-value', $closedTasks).text(data.length);
          var dataValues = [];
          for (var i = 0, max = data.length; i < max; i++){
            dataValues.push(data[i]['count']);
          }
          self.sparklineBar('stats-bar-3', dataValues, '100px', 5, 'blue', 5);
        }
      });
    };

  self.initMyOverViewOpportunitiesAndProject = function() {
      OverviewController.collectDataForDealProject(null, null, ['Deal', 'Project'], function(err, data){
        if (data) {
          var $project = $('#overview-project');
          var $opportunities = $('#overview-opportunities');
          if (data.projectStats && data.projectStats.projectData) {
            $('#project-active-val', $project).text(data.projectStats.active);
            $('#project-created-val', $project).text(data.projectStats.created);
            $('#project-complited-val', $project).text(data.projectStats.closed);
            $('#project-total-val', $project).text(data.projectStats.total);
            self.drawChartPie('donut-chart', data.projectStats.projectData);
          } else {
            $project.addClass('overiview-item-hide');
          }
          if (data.dealStats && data.dealStats.dealData) {
            $('#opport-active-val', $opportunities).text(data.dealStats.active);
            $('#opport-won-val', $opportunities).text(data.dealStats.won);
            $('#opport-lost-val', $opportunities).text(data.dealStats.lost);
            $('#opport-total-val', $opportunities).text(data.dealStats.total);
            self.drawChartPie('donut-chart-1', data.dealStats.dealData);
          } else {
            $opportunities.addClass('overiview-item-hide');
          }
        }
      });
    };

    self.initMyOpenOpportunities = function() {
      OverviewController.findAmountPerOpenOpportunities(function(err, data){
        if (data && data.length > 0){
          $openOpportunities.removeClass("overiview-item-hide").i18n();
          $('#open-tasks', $openOpportunities).text(data.length);
          self.sparklineBar('stats-bar', data, '100px', 5, 'blue', 5);
        }
      });
    };

    self.initMyWonOpportunities = function() {
      OverviewController.findAmountPerWonOpportunities(function(err, data){
        if (data && data.length > 0) {
          $wonOpportunities.removeClass("overiview-item-hide").i18n();
          self.sparklineBar('stats-bar-2', data, '100px', 5, 'green', 5);
          $('#won-tasks', $wonOpportunities).text(data.length);
        }
      });
    };

    self.initMyEffort = function() {
      var now = new Date();
      var startDate = new Date(now.getFullYear(), now.getMonth(), 1, 0, 0, 0, 0);
      OverviewController.findMyEfforts(localStorage.getItem('userId'), startDate, function(err, data){
          if (!err) {
            $myEffort.removeClass('overiview-item-hide').i18n();
            var numberOfWorkingHours = OverviewController.getNumberOfWorkingDays(
              now.getMonth(), now.getFullYear()) * 8;
            var numberOfHours = 0;
            if (data && data[0] && data[0].totalValue) {
              numberOfHours = data[0].totalValue;
            }
            $("#hours-month").text(numberOfHours + "h");
            var procent  = Math.round((numberOfHours * 100) / numberOfWorkingHours);
            $('.second.circle').circleProgress({
                startAngle: -Math.PI / 4 * 2,
                value: procent/100,
                lineCap: 'round',
                fill: { color: 'green' }
            }).on('circle-animation-progress', function(event, progress) {
                $(this).find('strong').html(parseInt(procent * progress) + '<i>%</i>');
            });
          }
      });
    }

    self.initTopSales = function() {
        OverviewController.topSales(null, null, function(err, data) {
          var $topSales = $("#top-sales");
          if (!err && data && data.length > 0) {
              self.drawPeople("top-sales-list", data);
              $topSales.removeClass('overiview-item-hide');
          } else {
              $topSales.addClass('overiview-item-hide');
          }
        });
    };

    self.initTopClients = function() {
      OverviewController.topClients(null, null, function(err, data) {
        var $topClients = $("#top-clients");
        if (!err && data && data.length > 0) {
            self.drawPeople("top-clients-list", data);
            $topClients.removeClass('overiview-item-hide');
        } else {
            $topClients.addClass('overiview-item-hide');
        }
      });
    };

    self.menuLinkEvent = function () {
        $("html").toggleClass("sidebar-left-in");
    };

    self.dashboardLinkEvent = function () {
        gotoPage({url: "dashboard", param: null});
    };

    self.contactsLinkEvent = function () {
        gotoPage({url: "contacts", param: null});
    };

    self.dealsLinkEvent = function () {
        gotoPage({url: "deals", param: null});
    };

    self.caseLinkEvent = function () {
        gotoPage({url: "case", param: null});
    };

    self.tasksLinkEvent = function () {
        gotoPage({url: "tasks", param: null});
    };

    self.draftLinkEvent = function () {
        gotoPage({url: "draft", param: null}, {url: "overview", param: null});
    };

    self.myGroupLinkEvent = function () {
        gotoPage({url: "myGroup", param: null});
    };

    self.calendarLinkEvent = function() {
          gotoPage({url: "calendar", param: null}, {url: "overview", param: null});
    };

    self.gotoCallList = function () {
        gotoPage({url: "callLog", param: null}, {url: "overview", param: null});
    };

    self.gotoAbout = function () {
        gotoPage({url: "about", param: null}, {url: "overview", param: null});
    };

    self.settingsLinkEvent = function () {
        gotoPage({url: "settings", param: null}, {url: "overview", param: null});
    };

    self.overviewEffortLinkEvent = function () {
        gotoPage({url: "overviewEffort", param: null}, {url: "overview", param: null});
    };

    self.setConnectionMode = function () {
      var $offlineMessageElement = $('#overview #offline-dashboard-message');
        if (connection) {
            if ($offlineMessageElement.is(':empty') == false) {
                $offlineMessageElement.empty();
            }
        } else {
            $offlineMessageElement
            .html('<p class="center"><img src="' + rootPath + '/images/noWifi.png" onerror="imgError(this);"></p>'
                    + '<p style="text-align:center;">' + Messages.Login.offlineMessage + '</p></div>');
            if ($myEffort) {
                $myEffort.empty();
            }
            if ($wonOpportunities) {
              $wonOpportunities.empty();
            }
            if ($openOpportunities){
              $openOpportunities.empty();
            }
        }
    };


    self.drawAvatarItem = function(item) {
        var html = "";
        var avatar = null;
        if (connection && item['user'] && item['user']['avatar-url'] != null) {
            avatar = window.localStorage.getItem('folderPath') + item['user']['avatar-url'];
        }
        if (avatar) {
            var fileUrl = ClientAPI.rootURL + '/api/db/downloadimg?objId=' + encodeURIComponent(item.user.person) + '&fieldName=' + encodeURIComponent('picture');
            html +=  '<div class="overview-avatar is-profile-pic" style="background-image:url(' + fileUrl + ')"></div>';
        } else {
            html += '<div class="overview-avatar-default"><i class="md-person  md-3x"></i></div>';
        }
        return html;
    };

    self.drawPeopleItem = function (item) {
        var html = "<div class='padding-ten'>"
                  + self.drawAvatarItem(item)
              + "<div class='overview-name-container'>" + item['user'].name + "</div>"
              + "<div class='overview-count-container'>" + item.count + "</div>"
              + "</div>";
            return html;
    };

    self.drawPeople = function(id, data) {
        var html = "";
        for (var i = 0, max = data.length; i < max; i++) {
            html += self.drawPeopleItem(data[i]);
        }
        if (html != "") {
            $("#" + id).empty().append(html);
        }

    };

    self.sparklineBar = function (id, values, height, barWidth, barColor, barSpacing) {
        $('.'+id).sparkline(values, {
            type: 'bar',
            height: height,
            barWidth: barWidth,
            barColor: barColor,
            barSpacing: barSpacing
        })
    };

    self.drawChartPie = function(id, data) {
      $.plot('#' + id, data, {
          series: {
              pie: {
                  innerRadius: 0.5,
                  show: true,
                  stroke: {width: 2},
              },
          },
          legend: {
              container: '.flc-donut',
              backgroundOpacity: 0.5,
              noColumns: 0,
              backgroundColor: "white",
              lineWidth: 0
          }
      });
    };

    return self;
}(OverviewFunctions || {}));
