$(document).on('pageinit', '#calendar', function () {
    CalendarFunctions.selectElement();
});

$(document).on('pageshow', '#calendar', function () {
    CalendarFunctions.bindElementControls();
    CalendarFunctions.init();
    $("#calendar #calendar-elem").fullCalendar({
        height: $("#calendar .overthrow").height(),
        eventClick: function (events) {
            CalendarFunctions.updateEvent(events);
        }
    });
});

$(document).on('pagehide', '#calendar', function () {
    CalendarFunctions.unbindElementControls();
});

var CalendarFunctions = (function (CalendarFunctions) {
    var self = CalendarFunctions;
    var $calendar = null;

    self.selectElement = function () {
        $calendar = $("#calendar #calendar-elem");
    };

    self.bindElementControls = function () {
        $(document).on('click', '#calendar #back-link', function () {
            goBack();
        });
        $(document).on('click', '#calendar #add-calendar-event', self.navigateEditEvent);
    };

    self.unbindElementControls = function () {
        $(document).off('click', '#calendar #back-link');
        $(document).off('click', '#calendar #add-calendar-event');
    };

    self.init = function () {
        self.getEvents(function (events) {
            if (events.length > 0) {
                $calendar.fullCalendar('addEventSource', events);
            }
        });
    };

    self.getEvents = function (callback) {
        if (connection) {
            ClientAPI.find({objectType: "Event"}, [["modificationDate", -1]], function (err, data) {
                if (!err) {
                    var events = [];
                    for (var i = 0; i < data.length; i++) {
                        var eventObject = {};
                        eventObject.title = data[i].eventName;
                        if (data[i].eventInfo) {
                            eventObject.description = data[i].eventInfo;
                        }
                        if (data[i].startDate) {
                            eventObject.start = data[i].startDate;
                        } else {
                            eventObject.start = new Date();
                        }
                        if (data[i].endDate) {
                            eventObject.end = data[i].endDate;
                        }
                        switch (data[i].priority) {
                            case "Low":
                                eventObject.color = "green";
                                break;
                            case "Medium":
                                eventObject.color = "blue";
                                break;
                            case "High":
                                eventObject.color = "red";
                                break;
                            default :
                                eventObject.color = "blue";
                                break;
                        }
                        events.push(eventObject);
                    }
                    if (callback && typeof callback == "function") {
                        callback(events);
                    }
                } else {
                    if (callback && typeof callback == "function") {
                        callback([]);
                    }
                }
            });
        }
    };

    self.updateEvent = function (event) {
        var object = {objectType: "Event"};
        object.eventName = event.title;
        if (event.description) {
            object.eventInfo = event.description;
        }
        if (event.start) {
            object.startDate = event.start._d;
        }
        if (event.end) {
            object.endDate = event.end._d;
        }

        ClientAPI.find(object, [["modificationDate", -1]], function (err, data) {
            if (data && data.length > 0) {
                gotoPage({url:"editCalendar", param: {calendarEvent: data[0]}}, {url: "calendar", param: null});
            } else {
            }
        });
    };
    
    self.navigateEditEvent = function(){
        gotoPage({url: "editCalendar", param: null}, {url: "calendar", param: null});
    };
    
    return self;
})(CalendarFunctions || {});