$(document).on('pageinit', '#edit-calendar', function () {
    EditCalendarFunctions.selectElement();
});

$(document).on('pageshow', '#edit-calendar', function () {
    EditCalendarFunctions.bindElementControls();
    EditCalendarFunctions.init();
});

$(document).on('pagehide', '#edit-calendar', function () {
    EditCalendarFunctions.unbindElementControls();
});


var EditCalendarFunctions = (function (EditCalendarFunctions) {

    var self = EditCalendarFunctions;
    var $subject = null;
    var $description = null;
    var $startDate = null;
    var $endDate = null;
    var $priority = null;
    var $internalGroup = null;
    var $externalGroup = null;
    var eventFromDatabase = null;

    self.selectElement = function () {
        $subject = $("#edit-calendar #subject");
        $description = $("#edit-calendar #description");
        $startDate = $("#edit-calendar #start-date");
        $endDate = $("#edit-calendar #end-date");
        $priority = $("#edit-calendar #calendar-priority");
        $internalGroup = $("#edit-calendar #internal-group-users");
        $externalGroup = $("#edit-calendar #external-group-users");
    };

    self.init = function () {
        CalendarAttendees.init("#edit-calendar");
        self.initiateDatePicker();
        if (pageObject) {
            eventFromDatabase = pageObject.calendarEvent;
            pageObject = null;
            self.loadFormData();
            $("#edit-calendar .title-h1").text("Edit Event");
        } else {
            $("#edit-calendar .title-h1").text("Add Event");
        }
    };

    self.bindElementControls = function () {
        $(document).on('click', '#edit-calendar #back-link', self.goBackEvent);
        $(document).on('click', '#edit-calendar #calendar-more-options', self.openMoreOptionsPopup);
        $(document).on('click', '#edit-calendar #save-link', self.save);
        $(document).on('click', '#edit-calendar .additional-options', self.save);
        $(document).on('click', '#edit-calendar .popup-menu-backdrop', function () {
            $('#edit-calendar .advanced-menu').hide();
            $(this).hide();
        });
    };

    self.unbindElementControls = function () {
        $(document).off('click', '#edit-calendar #back-link');
        $(document).off('click', '#edit-calendar #calendar-more-options');
        $(document).off('click', '#edit-calendar .popup-menu-backdrop');
        $(document).off('click', '#edit-calendar #save-link');
        $(document).off('click', '#edit-calendar .additional-options');
    };

    self.loadFormData = function () {
        $subject.val(eventFromDatabase.eventName);
        if (eventFromDatabase.eventInfo) {
            $description.val(eventFromDatabase.eventInfo);
        }
        if (eventFromDatabase.startDate) {
            var start = eventFromDatabase.startDate;
            start = start.split("T");
            start = normalizeDateFormat(reverseDate(start[0]));
            $startDate.val(start);
        }
        if (eventFromDatabase.endDate) {
            var endDate = eventFromDatabase.endDate;
            endDate = endDate.split("T");
            endDate = normalizeDateFormat(reverseDate(endDate[0]));
            $endDate.val(endDate);
        }
        if (eventFromDatabase.priority) {
            $priority.val(eventFromDatabase.priority);
        } else {
            $priority.val("Low");
        }
        $priority.selectmenu('refresh');

        if (eventFromDatabase.internalAttendees.length > 0 && eventFromDatabase.externalAttendees.length > 0) {
            self.drawAttendees(eventFromDatabase.internalAttendees, eventFromDatabase.externalAttendees);
        }

    };

    self.save = function () {
        if (eventFromDatabase) {
            eventFromDatabase.eventName = $subject.val();
            eventFromDatabase.eventInfo = $description.val();
            eventFromDatabase.priority = $priority.val();
            eventFromDatabase.startDate = self.formatDate($startDate.val());
            eventFromDatabase.endDate = self.formatDate($endDate.val());
            var externalIds = [];
            var internalIds = [];
            for (var i = 0; i < CalendarAttendees.externalIds.length; i++) {
                externalIds.push({person_id: new ObjectId(CalendarAttendees.externalIds[i].id)});
            }
            for (var i = 0; i < CalendarAttendees.internalIds.length; i++) {
                internalIds.push({person_id: new ObjectId(CalendarAttendees.internalIds[i].id)});
            }
            eventFromDatabase.externalAttendees = externalIds;
            eventFromDatabase.internalAttendees = internalIds;
            ClientAPI.updateObject(eventFromDatabase, function (err, data) {
                if (!err) {
                    self.goBackEvent();
                }
            });
        } else {
            var calendarItem = {
                objectType: "Event",
                priority: $priority.val()
            };
            calendarItem.externalAttendees = [];
            calendarItem.internalAttendees = [];
            if ($subject.val() == "") {
                alert("Subject field is empty!");
            } else {
                calendarItem.eventName = $subject.val();
                if ($description.val() != "") {
                    calendarItem.eventInfo = $description.val();
                }
                if ($startDate.val() != "") {
                    calendarItem.startDate = self.formatDate($startDate.val());
                }
                if ($endDate.val() != "") {
                    calendarItem.endDate = self.formatDate($endDate.val());
                }

                if (CalendarAttendees.externalIds.length > 0) {
                    for (var i = 0; i < CalendarAttendees.externalIds.length; i++) {
                        calendarItem.externalAttendees.push({person_id: new ObjectId(CalendarAttendees.externalIds[i].id)});
                    }
                } else {
                    delete calendarItem.externalAttendees;
                }

                if (CalendarAttendees.internalIds.length > 0) {
                    for (var i = 0; i < CalendarAttendees.internalIds.length; i++) {
                        calendarItem.internalAttendees.push({person_id: new ObjectId(CalendarAttendees.internalIds[i].id)});
                    }
                } else {
                    delete calendarItem.internalAttendees;
                }

                ClientAPI.insertObject(calendarItem, function (err, data) {
                    if (!err && data) {
                        self.goBackEvent();
                    }
                });
            }
        }
    };

    self.openMoreOptionsPopup = function () {
        $('#edit-calendar .popup-menu-backdrop').toggle();
        $('#edit-calendar .advanced-menu').css('z-index', '20000');
        $('#edit-calendar .advanced-menu').slideToggle('4000', 'swing');
    };

    self.initiateDatePicker = function () {
        if (navigator.userAgent.match(/(iPad|iPhone)/)) {
            $startDate.attr('type', 'datetime-local');
            $endDate.attr('type', 'datetime-local');
        } else {
            $startDate.mobiscroll().date({
                theme: 'android',
                display: 'modal',
                mode: 'scroller',
                dateOrder: 'dd MM yy',
                dateFormat: 'dd.mm.yy',
                maxDate: new Date(2025, 1, 1),
                minDate: new Date(2010, 1, 1)
            });
            $endDate.mobiscroll().date({
                theme: 'android',
                display: 'modal',
                mode: 'scroller',
                dateOrder: 'dd MM yy',
                dateFormat: 'dd.mm.yy',
                maxDate: new Date(2025, 1, 1),
                minDate: new Date(2010, 1, 1)
            });
        }
    };

    self.drawAttendees = function (internal, external) {
        var html, displayText;
        html = "<table>";
        var requestCounter = 0;
        var secondCounter = 0;
        var request = {};
        request['$or'] = [{'objectType': "Person"}, {'objectType': "Company"}];
        for (var i = 0; i < external.length; i++) {
            request['_id'] = new ObjectId(external[i]['person_id']);
            ClientAPI.find(request, null, function (err, data) {
                if (!err && data) {
                    requestCounter++;
                    for (var i = 0; i < data.length; i++) {
                        var tempObj = {id: data[i]['_id']};
                        if (data[i].objectType == "Person") {
                            displayText = data[i].firstName + data[i].lastName;
                        } else {
                            displayText = data[i].companyName;
                        }
                        tempObj.name = displayText;
                        CalendarAttendees.externalIds.push(tempObj);
                        html += "<tr>"
                                + "<td>" + displayText + "</td>"
                                + "<td id='" + data[i]['_id'] + "' data-deleteNumber='" + (requestCounter - 1) + "' data-type='external' class='delete-attendee'><i class='md-delete md-2x'></i></td>"
                                + "</tr>";
                    }
                    if (requestCounter == external.length) {
                        html += "</table>";
                        $("#edit-calendar #external-group-users").empty().append(html);
                    }
                }
            });
        }
        for (var i = 0; i < internal.length; i++) {
            var html1 = "<table>";
            request['_id'] = new ObjectId(internal[i]['person_id']);
            ClientAPI.find(request, null, function (err, data) {
                if (!err && data) {
                    secondCounter++;
                    for (var i = 0; i < data.length; i++) {
                        var tempObj = {id: data[i]['_id']};
                        if (data[i].objectType == "Person") {
                            displayText = data[i].firstName + data[i].lastName;
                        } else {
                            displayText = data[i].companyName;
                        }
                        tempObj.name = displayText;
                        CalendarAttendees.internalIds.push(tempObj);
                        html1 += "<tr>"
                                + "<td>" + displayText + "</td>"
                                + "<td id='" + data[i]['_id'] + "' data-deleteNumber='" + (secondCounter - 1) + "' data-type='internal' class='delete-attendee'><i class='md-delete md-2x'></i></td>"
                                + "</tr>";
                    }
                    if (secondCounter == internal.length) {
                        html1 += "</table>";
                        $("#edit-calendar #internal-group-users").empty().append(html1);
                    }
                }
            });
        }
    };

    self.formatDate = function (date) {
        if (navigator.userAgent.match(/(iPad|iPhone)/)) {
            return new Date(date);
        } else {
            return new Date(parseInt(date.split(".")[2].split(" ")[0]),
                    parseInt(date.split(".")[1] - 1),
                    parseInt(date.split(".")[0]),
                    new Date().getHours(), new Date().getMinutes(), new Date().getSeconds());
        }
    };

    self.goBackEvent = function () {
        self.clearAttendees();
        goBack();
    };

    self.clearAttendees = function () {
        CalendarAttendees.externalIds = [];
        CalendarAttendees.internalIds = [];
    };

    return self;

}(EditCalendarFunctions || {}));