$(document).on('pageinit', '#edit-domain', function () {
    EditDomainFunctions.bindElementControls();
});

$(document).on('pageshow', '#edit-domain', function () {
    EditDomainFunctions.init();
});

$(document).on('pagehide', '#edit-domain', function () {
    EditDomainFunctions.unbindElementControls();
    EditDomainFunctions.sharedObject = null;
});

var EditDomainFunctions = (function (EditDomainFunctions)
{
    var self = EditDomainFunctions;

    var $domain = null;
    var $username = null;
    var $password = null;
    var $server = null;
    var domainObj = null;

    self.init = function () {

        $domain = $('#domain');
        $username = $('#loginUserName');
        $password = $('#loginPassword');
        $server = $('#serverAddress');

        if (pageObject) {
            $("#delete-link").show();
            var domainId = pageObject.domainId;
            DomainController.findByColumn(domainId, 'id', function (data) {
                domainObj = data[0];
                self.drawEditDomain(domainObj);
            });
            pageObject = null;
        }
        else {
            $("#serverAddress").val("https://hive.beezify.io");
            $("#delete-link").hide();
        }
    };

    self.drawEditDomain = function (domainObject) {
        if (domainObject) {
            $domain.val(domainObject.domain);
            $username.val(domainObject.username);
            $password.val(domainObject.password);
            $server.val(domainObject.server);
        }
    };

    self.unbindElementControls = function () {
        $(document).off('click', '#edit-domain #back-link');
        $(document).off('click', '#edit-domain #save-link');
        $(document).off('click', '#edit-domain #delete-link');
        $(document).off('click', '#edit-domain-more-link');
        $('#edit-domain .popup-menu-backdrop').off('click');
        $(document).off('click', '#edit-domain .additional-options');
        domainObj = null;
    };

    self.bindElementControls = function () {
        $(document).on('click', '#edit-domain #back-link', self.goBack);
        $(document).on('click', '#edit-domain #save-link', self.saveEvent);
        $(document).on('click', '#edit-domain #delete-link', self.remove);
        $(document).on('click', '#edit-domain-more-link', self.editDomainPopup);
        $('#edit-domain .popup-menu-backdrop').on('click', function () {
            $('#edit-domain .advanced-menu').hide();
            $(this).hide();
        });
        $(document).on('click', '#edit-domain .additional-options', self.saveEvent);
    };

    self.editDomainPopup = function () {
        $('#edit-domain .popup-menu-backdrop').toggle();
        $('#edit-domain .advanced-menu').css('z-index', '20000');
        $('#edit-domain .advanced-menu').slideToggle('4000', 'swing');
    };

    self.goBack = function () {
        goBack();
    };

    self.saveEvent = function () {
        $("#edit-domain .advanced-menu").hide();
        $("#edit-domain .popup-menu-backdrop").hide();
        if (window.localStorage.getItem("enableSave") == "true") {
            showActionDialog(Messages.Domain.saveDomain, "Yes", function () {
                self.save();
            }, "No", function () {
            });
        } else {
            self.save();
        }
    };


    self.save = function () {
        playSound("www/libs/blop.mp3");
        var newDomain = {
            domain: $domain.val(),
            username: $username.val(),
            password: $password.val(),
            server: $server.val()
        };

        if (self.validation(newDomain)) {
            if (domainObj != null && domainObj.id != null) {
                //update
                newDomain['id'] = domainObj.id;
                DomainController.remove(newDomain['id'], function () {
                    DomainController.save(newDomain, function () {
                        alert(Messages.Domain.saved);
                        goBack();
                    });
                });
            } else {
                newDomain['id'] = UtilController.randomNumber();
                DomainController.save(newDomain, function () {
                    alert(Messages.Domain.saved);
                    goBack();
                });
            }
        }
    };

    self.remove = function () {
        $("#edit-domain .popup-menu-backdrop").hide();
        $("#edit-domain-popup").hide();
        setTimeout(function () {
            showActionDialog(Messages.Domain.removeDomain, "Yes", function () {
                if (domainObj != null && domainObj.id != null) {
                    DomainController.remove(domainObj.id, function () {
                        goBack();
                    });
                }
            }, "No", function () {
            });
        }, 200);
    };

    self.validation = function () {
        var newDomain = {};
        if ($.trim($domain.val()) != "") {
            newDomain['domain'] = $domain.val();
        } else {
            alert(Messages.Domain.errorDomain);
            return;
        }

        if ($.trim($username.val()) != "") {
            newDomain['username'] = $username.val();
        } else {
            alert(Messages.Domain.errorUsername);
            return;
        }

        if ($.trim($password.val()) != "") {
            newDomain['password'] = $password.val();
        } else {
            alert(Messages.Domain.errorPassword);
            return;
        }

        if ($.trim($server.val()) != "") {
            newDomain['server'] = $server.val();
        } else {
            alert(Messages.Domain.errorServer);
            return;
        }
        return true;
    };

    return self;
}(EditDomainFunctions || {}));