$(document).on('pageinit', '#edit-group', function () {
    EditGroupFunctions.init();
});

$(document).on('pageshow', '#edit-group', function () {
    EditGroupFunctions.bindElementControls();
});

$(document).on('pagehide', '#edit-group', function () {
    EditGroupFunctions.unbindElementControls();
});

var EditGroupFunctions = (function (EditGroupFunctions) {
    var self = EditGroupFunctions;
    var groupObject = null;

    self.init = function () {
        PermissionController.init("#edit-group");

        if (pageObject) {
            groupObject = {
                name: pageObject.groupName,
                description: pageObject.groupDescription
            };
            self.loadGroup(groupObject);
        } else {
            $("#edit-group .title-h1").text("Add group");
        }
    };

    self.bindElementControls = function () {
        $(document).on('click', '#edit-group #group-edit-back-button', goBack);
        $(document).on('click', '#edit-group #edit-group-more-link', self.openEditGroupPopup);
        $(document).on('click', '#edit-group #edit-group-save-button', self.saveGroup);
        $('#edit-group .popup-menu-backdrop').on('click', function () {
            $('#edit-group .advanced-menu').hide();
            $(this).hide();
        });
        $(document).on('click', '#edit-group .additional-options', self.saveGroup);
    };

    self.unbindElementControls = function () {
        $(document).off('click', '#edit-group #group-edit-back-button');
        $(document).off('click', '#edit-group #edit-group-more-link');
        $(document).off('click', '#edit-group #edit-group-save-button');
        $('#edit-group .popup-menu-backdrop').off('click');
        $(document).off('click', '#edit-group .additional-options');
        groupObject = null;
    };

    self.loadGroup = function (group) {
        if (connection) {
            ClientAPI.getGroupMembers(group.name, 1, 100, function (err, data) {
                if (!err) {
                    if (data && data.length > 0) {
                        PermissionController.setGroupUsersData(data);
                    }
                }
            });
        }

        $("#edit-group #group-name").prop("readonly", true);
        if (group.name) {
            $("#edit-group #group-name").val(group.name);
        }
        if (group.description) {
            $("#edit-group #group-description").val(group.description);
        }
    };

    self.saveGroup = function () {
        playSound("www/libs/blop.mp3");
        $(this).addClass('not-clickable');
        var groupName = $("#edit-group #group-name").val();
        var groupDescription = $("#edit-group #group-description").val();
        var groupUsers = PermissionController.permission;
        var userIds = [];
        for (var i = 0; i < groupUsers.length; i++) {
            userIds.push(groupUsers[i].identity);
        }
        if (groupName && groupDescription) {
            if (groupObject) {
                if (connection) {
                    ClientAPI.updateGroup(groupName, groupDescription, function (err, data) {
                        if (!err) {
                            ClientAPI.getGroupMembers(groupName, 1, 100, function (err, data) {
                                if (!err) {
                                    if (userIds.length > data.length) {
                                        userIds.splice(0, data.length);
                                        ClientAPI.addUsersToGroup(userIds, groupName, function (err, data) {
                                            if (!err) {
                                                goBack();
                                            }
                                        });
                                    } else if (userIds.length < data.length) {
                                        var removedUserIds = PermissionController.removedUsersFromGroup;
                                        for (var i = 0; i < removedUserIds.length; i++) {
                                            ClientAPI.removeUserFromGroup(removedUserIds[i], groupName, function (err, data) {
                                            });
                                        }
                                        goBack();

                                    } else {
                                        goBack();
                                    }
                                }
                            });
                        }
                    });
                }
            } else {
                if (connection) {
                    ClientAPI.addGroup(groupName, groupDescription, function (err, data) {
                        if (!err) {
                            if (userIds.length > 0) {
                                ClientAPI.addUsersToGroup(userIds, data.object.identityName, function (err, data) {
                                    if (!err) {
                                        goBack();
                                    }
                                });
                            } else {
                                goBack();
                            }

                        }
                    });
                } else {
                    console.log("Offline Group Saving");
                }
            }
        }
    };

    self.openEditGroupPopup = function () {
        $('#edit-group .popup-menu-backdrop').toggle();
        $('#edit-group .advanced-menu').css('z-index', '20000');
        $('#edit-group .advanced-menu').slideToggle('4000', 'swing');
    };

    self.encodeItem = function(item){
        var group = {};
        group.id = item.id;
        group.name = item.identityName;
        group.description = item.identityDescription;
        return group;
    };

    return self;

}(EditGroupFunctions || {}));
