$(document).on('pageinit', '#edit-template', function () {
    //initiate richtext editor
    if (tinymce.editors.length > 0) {
        tinymce.editors = [];
    }
    tinymce.init({
        mode: "exact",
        selector: "#rich-editor",
        autoresize_min_height: 250,
        menubar: false,
        content_css: "styles/tinymce-custom.css",
        statusbar: false,
        skin: 'light',
        plugins: "autoresize",
        toolbar: [
            "bold italic underline bullist numlist outdent indent"
        ]
    });

    EditTemplate.bindElementControls();
});

$(document).on('pageshow', '#edit-template', function () {
    EditTemplate.init();
});

$(document).on('pagehide', '#edit-template', function () {
    EditTemplate.unbindElementControls();
});


var EditTemplate = (function (EditTemplate) {

    var self = EditTemplate;
    var $templateTitle = null;
    var template = null;

    self.init = function () {
        $templateTitle = $("#edit-template #template-name");
        if (pageObject) {
            template = JSON.parse(pageObject.template);
            if (template) {
                $("#edit-template .edit-template-title").show();
                self.loadTemplate();
            }
            pageObject = null;
        } else {
            $("#edit-template .add-template-title").show();
        }
    };

    self.bindElementControls = function () {
        $(document).on('click', '#edit-template #back-link', goBack);
        $(document).on('click', '#edit-template #edit-profil-more-link', self.showEditTemplatePopup);
        $('#edit-template .popup-menu-backdrop').on('click', function () {
            $('#edit-template .advanced-menu').hide();
            $(this).hide();
        });
        $(document).on('click', '#edit-template .additional-options', self.save);
        $(document).on('click', '#edit-template #save-link', self.save);
    };

    self.unbindElementControls = function () {
        $(document).off('click', '#edit-template #back-link');
        $(document).off('click', '#edit-template #edit-profil-more-link');
        $('#edit-template .popup-menu-backdrop').off('click');
        $(document).off('click', '#edit-template .additional-options');
        $(document).off('click', '#edit-template #save-link');
        template = null;
    };

    self.showEditTemplatePopup = function () {
        $('#edit-template .popup-menu-backdrop').toggle();
        $('#edit-template .advanced-menu').css('z-index', '20000');
        $('#edit-template .advanced-menu').slideToggle('4000', 'swing');
    };

    self.loadTemplate = function () {
        $templateTitle.val(template.templateName);
        setTimeout(function () {
            tinymce.activeEditor.setContent(template.template);
        }, 300);

    };

    self.save = function () {
        $(this).addClass('not-clickable');
        playSound("www/libs/blop.mp3");
        var templateTitle = $templateTitle.val();
        var templateContent = $.trim(tinymce.activeEditor.getContent());
        if (connection) {
            if (template) {
                template.templateName = templateTitle;
                template.template = templateContent;
                ClientAPI.updateTemplate(template, function (err, data) {
                    if (!err && data) {
                        goBack();
                    }
                });
            } else {
                ClientAPI.addTemplate(templateTitle, templateContent, "Template", function (err, data) {
                    if (!err && data) {
                        goBack();
                    }
                });
            }
        }
    };

    return self;

}(EditTemplate || {}));