$(document).on('pageinit', '#app-settings', function () {
    AppSettingsFunctions.init();
});

$(document).on('pageshow', '#app-settings', function () {
    AppSettingsFunctions.bindElementControls();
});

$(document).on('pagehide', '#app-settings', function () {
    AppSettingsFunctions.unbindElementControls();
});

var AppSettingsFunctions = (function (AppSettingsFunctions) {
    var self = AppSettingsFunctions;

    self.init = function () {
        //translate dropdown lists
        $("#save-list").i18n();
        $("#notification-list").i18n();
        $("#quality-list").i18n();

        self.checkSavePopupStatus();
        self.setNotificationStatus();
        self.loadPictureQuality();

        var language = window.localStorage.getItem("userLanguage");
        if (language) {
            $("#user-language").val(language);
        }

        var mode = window.localStorage.getItem("offlineOnlineMode");
        if (mode) {
            $("#online-offline-mode").val(mode);
        }

        $("#user-language").i18n();
        $("#user-language").selectmenu("refresh");

        $("#online-offline-mode").i18n();
        $("#online-offline-mode").selectmenu("refresh");

        if (localStorage.getItem('feedOffset')) {
          try {
            var numberItems = parseInt(localStorage.getItem('feedOffset'));
            $("#app-settings #items-to-show").val(numberItems);
          } catch(e) {}
        }
    };

    self.bindElementControls = function () {
        $(document).on('click', '#save-link', self.save);
        $(document).on('click', '#back-link', goBack);
        $(document).on('click', '#app-settings-more-link', self.openAppSettingsPopup);
        $(document).on('change', '#user-language', self.changeLanguage);
        $(document).on('change', '#online-offline-mode', self.changeOfflineOnlineMode);
        $('#app-settings .popup-menu-backdrop').on('click', function () {
            $('#app-settings .advanced-menu').hide();
            $(this).hide();
        });
        $(document).on('click', '#app-settings .additional-options', self.save);
    };

    self.unbindElementControls = function () {
        $(document).off('click', '#save-link');
        $(document).off('click', '#back-link');
        $(document).off('click', '#app-settings-more-link');
        $(document).off('change', '#user-language');
        $(document).off('change', '#online-offline-mode');
        $('#app-settings .popup-menu-backdrop').off('click');
        $(document).off('click', '#app-settings .additional-options');
    };

    self.openAppSettingsPopup = function () {
        $('#app-settings .popup-menu-backdrop').toggle();
        $('#app-settings .advanced-menu').css('z-index', '20000');
        $('#app-settings .advanced-menu').slideToggle('4000', 'swing');
    };

    self.changeLanguage = function () {
        var language = $("#user-language").val();
        i18n.setLng(language);
        window.localStorage.setItem("userLanguage", language);
    };

    self.changeOfflineOnlineMode = function() {
      var mode = $("#online-offline-mode").val();
      window.localStorage.setItem("offlineOnlineMode", mode);
      if (mode == 'online') {
        if (isConnected()) {
            onDeviceOnline();
        } else {
            onDeviceOffline();
        }
      } else {
        onDeviceOffline();
      }
      setHeaderOfflineOrOnline();
    };

    self.save = function () {
        $("#app-settings .advanced-menu").hide();
        $("#app-settings .popup-menu-backdrop").hide();
        if (window.localStorage.getItem("enableSave") == "true") {
            showActionDialog('Save this settings?', 'Yes', function () {
                self.setSavePopupStatus();
                self.setNotificationStatus();
                self.setPictureQuality();
                self.setNumberOfFeedListItems();
                goBack();
            }, 'No', function () {});
        } else {
            self.setSavePopupStatus();
            self.setNotificationStatus();
            self.setPictureQuality();
            self.setNumberOfFeedListItems();
            goBack();
        }
    };

    self.setNumberOfFeedListItems = function () {
        var numOfItems = $("#app-settings #items-to-show").val();
        numOfItems = parseInt(numOfItems);
        if (numOfItems > 0 && numOfItems < 100) {
            feedOffset = numOfItems;
        } else {
            feedOffset = 30;
        }
        localStorage.setItem('feedOffset', feedOffset);
    };

    self.setPictureQuality = function () {
        var qualityValue = $("#quality-list").val();
        var pictureQuality;
        if (qualityValue) {
            pictureQuality = qualityValue;
            window.localStorage.setItem("pictureQuality", pictureQuality);
        } else {
            if (window.localStorage.getItem("pictureQuality")) {
                pictureQuality = window.localStorage.getItem("pictureQuality");
            } else {
                window.localStorage.setItem("pictureQuality", "75");
                pictureQuality = "75";
            }
        }

        if(pictureQuality == "75"){
            window.localStorage.setItem("pictureQuality", "75");
            $("#quality-list").val("75");
            $("#quality-list").selectmenu("refresh");
        } else if(pictureQuality == "100"){
            window.localStorage.setItem("pictureQuality", "100");
            $("#quality-list").val("100");
            $("#quality-list").selectmenu("refresh");
        } else if (pictureQuality == "50"){
            window.localStorage.setItem("pictureQuality", "50");
            $("#quality-list").val("50");
            $("#quality-list").selectmenu("refresh");
        }

    };

    self.loadPictureQuality = function(){
        if(window.localStorage.getItem("pictureQuality")){
            $("#quality-list").val(window.localStorage.getItem("pictureQuality"));
            $("#quality-list").selectmenu('refresh');
        } else {
            window.localStorage.setItem("pictureQuality", "75");
            $("#quality-list").selectmenu('refresh');
        }
    };

    self.checkSavePopupStatus = function () {
        var enableSaveStatus = window.localStorage.getItem("enableSave");
        if (enableSaveStatus == null || enableSaveStatus == "false") {
            window.localStorage.setItem("enableSave", false);
            $("#save-list").val("no");
            $("#save-list").selectmenu('refresh');
        } else {
            window.localStorage.setItem("enableSave", true);
            $("#save-list").val("yes");
            $("#save-list").selectmenu("refresh");
        }

        var notificationStatus = window.localStorage.getItem("notificationStatus");
        if (notificationStatus == null || notificationStatus == "false") {
            window.localStorage.setItem("notificationStatus", false);
                $("#notification-list").val("no");
                $("#notification-list").selectmenu("refresh");
        } else {
            window.localStorage.setItem("notificationStatus", true);
            $("#notification-list").val("yes");
            $("#notification-list").selectmenu("refresh");

        }
    };

    self.setSavePopupStatus = function () {
        var enablePopupSave = $("#save-list").val();
        if(enablePopupSave == "yes"){
            window.localStorage.setItem("enableSave", true);
        } else {
            window.localStorage.setItem("enableSave", false);
        }
    };

    self.setNotificationStatus = function () {
        var enableCalendarNotification = $("#notification-list").val();
        if(enableCalendarNotification == "yes"){
            window.localStorage.setItem("notificationStatus", true);
        } else {
            window.localStorage.setItem("notificationStatus", false);
        }
    };

    return self;
}(AppSettingsFunctions || {}));
