$(document).on('pageinit', '#notification-settings', function () {
    SettingNotificationFunctions.init();
});

$(document).on('pageshow', '#notification-settings', function () {
    SettingNotificationFunctions.bindElementControls();   
});

$(document).on('pagehide', '#notification-settings', function () {
    SettingNotificationFunctions.unbindElementControls();
});

var SettingNotificationFunctions = (function (SettingNotificationFunctions)
{
    var self = SettingNotificationFunctions;
    var enableNotification = null;
    var smsNotification = null;
    var emailNotification = null;
    var pushNotification = null;
    var from  =null;
    var to = null;
    var personId = null;

    self.init = function () {
        self.selectElement();
        self.initiateSwitchCheckbox();
        self.loadNotificationSettings();
    };
    
    self.selectElement = function(){
        enableNotification = $("#notification-settings #enableNotification");
        smsNotification = $("#notification-settings #smsNotification");
        emailNotification = $("#notification-settings #emailNotification");
        pushNotification = $("#notification-settings #pushNotification");
        from = $("#notification-settings #timeRangeFrom");
        to = $("#notification-settings #timeRangeTo");
    };

    self.bindElementControls = function () {
        $(document).on('click', '#back-link', goBack);
        $(document).on('click', '#notification-settings #more-options-link', self.openPopupMenu);
        $(document).on('click', '#notification-settings #save-link', self.save);
        $(document).on('click', '#notification-settings .additional-options', self.save);
        $('#enableNotification').on('switchChange.bootstrapSwitch', function(event, data){
            if(data == false){
                $("#advanced-settings").removeClass('seen').addClass('unseen');
            } else {
                $("#advanced-settings").removeClass('unseen').addClass('seen');
            }
        }); 
        $(document).on('click', '#notification-settings .popup-menu-backdrop', function () {
            $('#notification-settings .advanced-menu').hide();
            $(this).hide();
        });
    };

    self.unbindElementControls = function () {
        $(document).off('click', '#back-link');
        $(document).off('click', '#notification-settings #more-options-link');
        $(document).off('click', '#notification-settings #save-link');
        $(document).off('click', '#notification-settings .additional-options');
        $('#enableNotification').off('switchChange.bootstrapSwitch'); 
        $(document).off('click', '#notification-settings .popup-menu-backdrop');
    };
    
    self.openPopupMenu = function () {
        $('#notification-settings .popup-menu-backdrop').toggle();
        $('#notification-settings .advanced-menu').css('z-index', '20000');
        $('#notification-settings .advanced-menu').slideToggle('4000', 'swing');
    };    
    
    self.initiateSwitchCheckbox = function () {
        enableNotification.bootstrapSwitch({size: 'small'});
        smsNotification.bootstrapSwitch({size: 'small'});
        emailNotification.bootstrapSwitch({size: 'small'});
        pushNotification.bootstrapSwitch({size: 'small'});
        from.clockpicker({
            placement: 'top',
            align: 'left',
            autoclose: true,
            'default': '00:00'
        });

        to.clockpicker({
            placement: 'top',
            align: 'left',
            autoclose: true,
            'default': '23:59'
        });
    };
    
    self.loadNotificationSettings = function () {
        var personInfo = JSON.parse(localStorage.getItem('userProfile'));
        personId = personInfo.domainMembership.person;
        if (personId) {
            var notifSettings = {};
            ClientAPI.findSettings({
                query: {
                    settingType: 'Notifications',
                    refObject: new ObjectId(personId)
                },
                objectType: 'Person'
            }, function (err, setting) {
                if (!err) {
                    if (setting && setting.length > 0 && setting[0].settingData) {
                        if (!setting[0].settingData.notification) {
                            enableNotification.bootstrapSwitch('state', false);
                            $("#advanced-settings").removeClass('seen').addClass('unseen');
                        } else {
                            enableNotification.bootstrapSwitch('state', true);
                            $("#advanced-settings").removeClass('unseen').addClass('seen');
                        }
                        notifSettings = {
                            pushNotification: setting[0].settingData.pushNotification,
                            emailNotification: setting[0].settingData.emailNotification,
                            smsNotification: setting[0].settingData.smsNotification
                        };
                        var fromTime = setting[0].settingData.timeRange.from;
                        var toTime = setting[0].settingData.timeRange.to;
                        fromTime = fromTime.split(":");
                        toTime = toTime.split(":");
                        
                        self.loadTimeRange({
                            from: fromTime[0] + ":" + fromTime[1],
                            to: toTime[0] + ":" + toTime[1]
                        });
                    } else {
                        notifSettings = {
                            pushNotification: true,
                            emailNotification: true,
                            smsNotification: true
                        };
                        enableNotification.bootstrapSwitch('state', true);
                        self.loadTimeRange();
                    }
                    self.loadNotificationOptions(notifSettings);
                } else {
                    alert("Error loading notification settings!");
                }
            });
        } else {
            alert("Error: User not found!");
        }
        self.loadNotificationOptions({
            smsNotification: true,
            emailNotification: false,
            pushNotification: false
        });
    };
    
    self.save = function () {
        var notificationSetting = {};
        var fromTime = from.val() + ":00";
        var toTime = to.val() + ":00";
        // notification settings configuration
        notificationSetting = {
            notification: enableNotification.bootstrapSwitch('state'),
            smsNotification: smsNotification.bootstrapSwitch('state'),
            emailNotification: emailNotification.bootstrapSwitch('state'),
            pushNotification: pushNotification.bootstrapSwitch('state'),
            timeRange: {
                from: fromTime,
                to: toTime
            }
        };
        
        console.log("Notificatio Setting Object");
        console.log(JSON.stringify(notificationSetting));
        
        ClientAPI.findSettings({
            query: {
                settingType: 'Notifications',
                refObject: new ObjectId(personId)
            },
            objectType: 'Person'
        }, function (err, setting) {
            if (!err) {
                if (setting && setting.length > 0 && setting[0].settingData) {
                    ClientAPI.updateSettings({
                        _id: new ObjectId(setting[0]._id),
                        settingType: 'Notifications',
                        settingData: notificationSetting

                    }, function (data) {
                        goBack();
                    });
                } else {
                    ClientAPI.addSettings({
                        settingType: 'Notifications',
                        settingData: notificationSetting,
                        refObjectType: 'Person',
                        refObject: personId
                    }, function () {
                        goBack();
                    });
                }
            } else {
                alert("Error loading notification settings!");
            }
        });
    };
    
    self.loadNotificationOptions = function (notificationOptions) {
        smsNotification.bootstrapSwitch('state', notificationOptions.smsNotification);
        emailNotification.bootstrapSwitch('state', notificationOptions.emailNotification);
        pushNotification.bootstrapSwitch('state', notificationOptions.pushNotification);
    };

    self.loadTimeRange = function (period) {
        if (period) {
            from.val(period.from);
            to.val(period.to);
        } else {
            from.val("00:00");
            to.val("23:59");
        }
    };
    
    return self;
}(SettingNotificationFunctions || {}));

