$(document).on('pageinit', '#sip', function () {
    SipFunctions.bindElementControls();
});

$(document).on('pageshow', '#sip', function () {
    SipFunctions.init();
});

$(document).on('pagehide', '#sip', function () {
    SipFunctions.unbindElementControls();
});

var SipFunctions = (function (SipFunctions)
{
    var self = SipFunctions;
    var sipObjects = [];
    var currentSipId = null;

    self.init = function () {
        if (connection) {
            ClientAPI.getUserSettings(function (error, data) {
                sipObjects = data.userSettings.sipProfiles;
                self.drawSipList("#sip-list", data);
            });
        }
    };
    self.bindElementControls = function () {
        $(document).on('click', '#sip #back-link', self.goBack);
        $(document).on('click', '#sip .sip-item', self.gotoEditSip);
        $(document).on('click', '#sip .unactive-sip-domain', self.openActivationPopup);
        $(document).on('click', '#sip #cancel-sip-domain', self.closeCommentPopup);
        $(document).on('click', '#sip #activate-sip-domain', self.activateSipDomain);
        $(document).on('click', '#sip #new-sip-link', self.addNewSipDomain);
    };

    self.unbindElementControls = function () {
        $(document).off('click', '#sip #back-link');
        $(document).off('click', '#sip .sip-item');
        $(document).off('click', '#sip .unactivate-sip-domain');
        $(document).off('click', '#sip #cancel-sip-domain');
        $(document).off('click', '#sip #activate-sip-domain');
        $(document).off('click', '#sip #new-sip-link');
    };

    self.goBack = function () {
        goBack();
    };

    self.gotoEditSip = function () {
        var sipId = $(this).attr("sipId");
        gotoPage({url: "sipSettings", param: {sipId: sipId}}, {url: "sip", param: null});
    };
    
    self.addNewSipDomain = function(){
      gotoPage({url: "sipSettings", param: null}, {url: "sip", param: null});  
    };

    self.closeCommentPopup = function () {
        $('#sip #sip-popup').popup("close");
        setTimeout(function () {
            $("#sip-pass").val("");
            $('input[sipId="' + currentSipId + '"]').prop("checked", false);
            $("input.active-sip-domain").prop("checked", true);
        }, 300);
    };

    self.drawSipList = function (viewId, items) {
        var $list = $(viewId);
        var html = "";
        if (items.userSettings && items.userSettings.sipProfiles && items.userSettings.sipProfiles.length > 0) {
            var radioButtonClass = "unactive-sip-domain";
            var elements = items.userSettings.sipProfiles;
            for (var elem in elements) {
                radioButtonClass = "unactive-sip-domain";
                if (elements[elem].sipDomain == window.localStorage.getItem("sipDomain")) {
                    radioButtonClass = "active-sip-domain";
                }
                html += '<li style="position: relative">';
                html += '<a href="#" class="sip-item" style="padding:10px;" sipId="' + elements[elem].sipDomain + '">';
                html += '<div class="title">' + elements[elem].sipDomain + '</div>';
                if (elements[elem].sipUsername) {
                    html += '<div class="subtitle">' + elements[elem].sipUsername + '</div>';
                }
                html += '<div>' + elements[elem].authUsername + '</div>';
                html += '</a>';
                html += '<input type="radio" name="sip-option" id="' + elements[elem].sipDomain + '" class="' + radioButtonClass + '" sipId="' + elements[elem].sipDomain + '" style="  position: absolute; top: 30px; right: 0px; width:25px; height: 25px"/>';
                html += '</li>';
            }
        } else {
            html = emptyListHtml;

        }
        $list.empty().append(html);
        $("#sip .active-sip-domain").prop("checked", true);
    };

    self.openActivationPopup = function () {
        currentSipId = $(this).attr("id");
        $("#sip-popup").popup("open");
    };

    self.activateSipDomain = function () {
        var sipPassword = $("#sip-pass").val();
        if (sipPassword) {
            for (var elem in sipObjects) {
                if (currentSipId == sipObjects[elem].sipDomain) {
                    // setting sip elements
                    window.localStorage.setItem("sipUsername", sipObjects[elem].sipUsername);
                    window.localStorage.setItem("sipPassword", sipPassword);
                    window.localStorage.setItem("port", sipObjects[elem].port);
                    window.localStorage.setItem("proxy", sipObjects[elem].proxy);
                    window.localStorage.setItem("authUsername", sipObjects[elem].authUsername);
                    window.localStorage.setItem("sipDomain", sipObjects[elem].sipDomain);
                    SipSettingsFunctions.sipLogin(sipObjects[elem].sipUsername, sipPassword, function (data) {
                        $("#sip-popup").popup("close");
                        setTimeout(function () {
                            alert(data);
                        }, 300);
                    }, function (data) {
                        alert(data);
                    });
                }
            }
        } else {
            $("#sip-popup").popup("close");
            setTimeout(function () {
                alert("Enter password!");
            }, 300);
        }

    };
    
    self.checkSipDomainStatus = function(){
        
    };

    return self;
}(SipFunctions || {}));