$(document).on('pageinit', '#sip-messages', function () {
    SipMessagesFunctions.init();
});

$(document).on('pagebeforeshow', '#sip-messages', function () {
    SipMessagesFunctions.bindElementControls();
    SipMessagesFunctions.showMessageNumber();
    SipMessagesFunctions.filterMessages();
});

$(document).on('pagehide', '#sip-messages', function () {
    SipMessagesFunctions.unbindElementControls();
});

var SipMessagesFunctions = (function (SipMessagesFunctions) {
    var self = SipMessagesFunctions;

    var Settings = {
        'CURRENT_POSTIION': 'recived-message',
        'RECIVED_MESSAGE': 'recived-message',
        'SENT_MESSAGE': 'sent-message',
        'OPEN_MESSAGE': '1',
        'CLOSE_MESSAGE': '0'
    };

    var $sipMessagesList = null;
    var $recivedMessage = null;
    var $sentMessage = null;
    var $emptyListMessage = null;

    self.init = function () {
        $sipMessagesList = $('#sip-messages-list');
        $recivedMessage = $('#recived-message');
        $sentMessage = $('#sent-message');
        $emptyListMessage = $('#sip-empty-list-message');
    };

    self.showMessageNumber = function () {
        db.selectByQuery('SELECT count(*) as count FROM infoMessage WHERE toUser =? ', [window.localStorage.getItem("userId")], 'infoMessage', function (data) {
            $("#recived-message").html(data[0].count);
        });
        db.selectByQuery('SELECT count(*) as count FROM infoMessage WHERE toUser <>? ', [window.localStorage.getItem("userId")], 'infoMessage', function (data) {
            $("#sent-message").html(data[0].count);
        });
    };

    self.bindElementControls = function () {
        $(document).on('click', '#sip-messages #recived-message', self.filterMessages);
        $(document).on('click', '#sip-messages #sent-message', self.filterMessages);
        $(document).on('click', '#sip-messages #sip-messages-list .message', self.setMessageStatus);
        $(document).on('click', '#sip-messages #refresh-link-message', self.filterMessages);
        $(document).on('click', '#sip-messages #back-link', goBack);
        $(document).on('input', "#sip-messages input:jqmData(type='search')", self.searchInputEvent);
        $(document).on('click', '#sip-messages .ui-input-clear', self.searchInputEvent);
        $('body').on('submit', function (event) {
            event.preventDefault(); // cancel default form submit
            event.stopPropagation();
            return false;
        });
    };

    self.unbindElementControls = function () {
        $(document).off('click', '#sip-messages #recived-message');
        $(document).off('click', '#sip-messages #sent-message');
        $(document).off('click', '#sip-messages #sip-messages-list .message');
        $(document).off('click', '#sip-messages #refresh-link-message');
        $(document).off('click', '#sip-messages #back-link');
        $(document).off('input', "#sip-messages input:jqmData(type='search')");
        $(document).off('click', '#sip-messages .ui-input-clear');
    };

    self.filterMessages = function () {
        var messageType = '';
        if ($(this).attr("id") == null) {
            messageType = Settings['CURRENT_POSTIION'];
            self.getMessages(messageType);
        } else {
            if ($(this).attr("id") == 'refresh-link-message') {
                SipMessageSettingFunctions.getDataFromServer(function () {
                    messageType = Settings['CURRENT_POSTIION'];
                    self.getMessages(messageType);
                });
            } else {
                messageType = $(this).attr("id");
                self.getMessages(messageType);
            }
        }
    };

    self.getMessages = function (messageType) {
        var myId = window.localStorage.getItem("userId");
        if (messageType == Settings['RECIVED_MESSAGE']) {
            Settings['CURRENT_POSTIION'] = Settings['RECIVED_MESSAGE'];
            $recivedMessage.addClass("recived-message-icon-current").removeClass("recived-message-icon");
            $sentMessage.addClass("sent-message-icon").removeClass("sent-message-icon-current");

            db.selectByQuery('SELECT * FROM infoMessage WHERE toUser =? ORDER BY status ASC, updated DESC', [myId], 'infoMessage', function (data) {
                console.log("Length: " + data.length);
                $("#recived-message").html(data.length);
                self.drawMessages(data);
            });
        } else {
            Settings['CURRENT_POSTIION'] = Settings['SENT_MESSAGE'];
            $recivedMessage.addClass("recived-message-icon").removeClass("recived-message-icon-current");
            $sentMessage.addClass("sent-message-icon-current").removeClass("sent-message-icon");
            db.selectByQuery('SELECT * FROM infoMessage WHERE toUser <>? ORDER BY status ASC, updated DESC', [myId], 'infoMessage', function (data) {
                console.log("Length: " + data.length);
                $("#sent-message").html(data.length);
                self.drawMessages(data);
            });
        }
    };

    self.drawMessages = function (data) {
        $sipMessagesList.empty();
        var messageClass = "";
        var html = "";
        var myId = window.localStorage.getItem("userId");
        var requestMessageStatusChange = [];

        if (data.length >= 1) {
            $(data).each(function (index, value) {
                if (parseInt(value.status) <= 1) {
                    if (value.status == Settings['CLOSE_MESSAGE'] && myId == value.toUser) {
                        requestMessageStatusChange.push({'messageid': value._id});
                        SipMessageSettingFunctions.setMessageStatus(requestMessageStatusChange);
                    }

                    if (value.status == 0) {
                        messageClass = "message-close-icon";
                    } else {
                        messageClass = "message-open-icon";
                    }

                    var temp = JSON.stringify(new Date(value.updated)).split('T');
                    var date = temp[0].substring(1, temp[0].length);
                    var time = temp[1].substring(0, temp[1].length - 9);
                    html += '<li>'
                            + '<a href="#" toUser="' + value.toUser + '" id="' + value._id + '" class="message" taskid="' + value.body.taskid + '" status="' + value.status + '">'
                            + '<h1>Message: ' + value.body.body + '</h1>'
                            + '<p class="ui-li-desc">From: ' + ALL_USERS[value.fromUser].name + '</p>'
                            + '<p class="ui-li-desc">To: ' + ALL_USERS[value.toUser].name + '</p>'
                            + '<p>Date: ' + date + '</p>'
                            + '<p>Time: ' + time + '</p>'

                            + '</a>'
                            + '<a href="" toUser="' + value.toUser + '" id="' + value._id + '" taskId="' + value.body.taskid + '" class="message ' + messageClass + '" status="' + value.status + '"'
                            + 'data-iconpos="notext" data-icon="false">'
                            + '</a></li>';
                }
            });
            $sipMessagesList.append(html);
            $sipMessagesList.listview('refresh');
            $emptyListMessage.hide();
        } else {
            $emptyListMessage.show();
        }
    };

    self.searchInputEvent = function () {
        if ($('#sip-messages-list li:visible').length == 0) {
            $emptyListMessage.show();
        } else {
            $emptyListMessage.hide();
        }
    };

    self.setMessageStatus = function () {
        var id = null;
        var ids = [];

        id = $(this).attr('taskId');
        gotoPage({url: "viewTask", param: {id: id, ids: ids}}, {url: "sipMessages", param: null});
    };
    return self;
}(SipMessagesFunctions || {}));
;