$(document).on('pageinit', '#sip-settings', function () {
    SipSettingsFunctions.init();
});

$(document).on('pageshow', '#sip-settings', function () {
    SipSettingsFunctions.bindElementControls();
});

$(document).on('pagehide', '#sip-settings', function () {
    SipSettingsFunctions.unbindElementControls();
});

var SipSettingsFunctions = (function (SipSettingsFunctions) {
    var self = SipSettingsFunctions;
    var sipId = null;

    self.bindElementControls = function () {
        $(document).on('click', '#save-link', self.save);
        $(document).on('click', '#back-link', goBack);
        $(document).on('click', '#sip-settings-more-link', self.openSipSettingsPopup);
        $('#sip-settings .popup-menu-backdrop').on('click', function () {
            $('#sip-settings .advanced-menu').hide();
            $(this).hide();
        });
        $(document).on('click', '#sip-settings .additional-options', self.save);
    };

    self.unbindElementControls = function () {
        $(document).off('click', '#save-link');
        $(document).off('click', '#back-link');
        $(document).off('click', '#sip-settings-more-link');
        $('#sip-settings .popup-menu-backdrop').off('click');
        $(document).off('click', '#sip-settings .additional-options');
        sipId = null;
    };

    self.openSipSettingsPopup = function () {
        $('#sip-settings .popup-menu-backdrop').toggle();
        $('#sip-settings .advanced-menu').css('z-index', '20000');
        $('#sip-settings-popup').slideToggle('4000', 'swing');
    };

    self.init = function () {
        if (pageObject) {
            sipId = pageObject.sipId;
            $("#sip-settings #sipDomain").prop("disabled", true);
            // get user sip settings
            if (connection) {
                ClientAPI.getUserSettings(function (error, data) {
                    if (data.userSettings && data.userSettings.sipProfiles && data.userSettings.sipProfiles.length > 0) {
                        var sipProfile;
                        for (var i = 0; i < data.userSettings.sipProfiles.length; i++) {
                            if (sipId == data.userSettings.sipProfiles[i].sipDomain) {
                                sipProfile = data.userSettings.sipProfiles[i];
                            }
                        }
                        // ucitavanje podataka
                        $('#sipUserName').val(sipProfile.sipUsername);
                        $('#port').val(sipProfile.port);
                        $('#proxy-text').val(sipProfile.proxy);
                        $('#auth-username-text').val(sipProfile.authUsername);
                        $('#sipDomain').val(sipProfile.sipDomain);

                        var sipPassword = window.localStorage.getItem("sipPassword");
                        if (sipPassword) {
                            $('#sipPassword').val(sipPassword);
                        }
                    }
                });
            }

        } else {
            $("#sip-settings #sipDomain").prop("disabled", false);
        }
        pageObject = null;
    };

    self.save = function () {
        var sipUsername = $.trim($('#sipUserName').val()).replace(/ /g, '');
        var sipPassword = $.trim($('#sipPassword').val());
        var port = $.trim($('#port').val());
        var proxy = $.trim($('#proxy-text').val());
        var authUsername = $.trim($('#auth-username-text').val());
        var sipDomain = $.trim($('#sipDomain').val());

        if (sipUsername == null || sipUsername == "") {
            alert("Enter username!");
            self.removeSaveOption();
            return;
        }

        if (sipPassword == null || sipPassword == "") {
            alert("Enter password!");
            self.removeSaveOption();
            return;
        }

        if (sipDomain == null || sipDomain == "") {
            alert("Enter domain!");
            self.removeSaveOption();
            return;
        }

        if (window.localStorage.getItem("enableSave") == "true") {
            showActionDialog('Save this settings?', 'Yes', function () {
                window.localStorage.setItem("sipUsername", sipUsername);
                window.localStorage.setItem("sipPassword", sipPassword);
                window.localStorage.setItem("port", port);
                window.localStorage.setItem("proxy", proxy);
                window.localStorage.setItem("authUsername", authUsername);
                window.localStorage.setItem("sipDomain", sipDomain);
                $('#sipUserName').val(sipUsername);

                var sipProfile = {
                    sipUsername: sipUsername,
                    port: port,
                    proxy: proxy,
                    authUsername: authUsername,
                    sipDomain: sipDomain
                };

                self.sipLogin(sipUsername, sipPassword, function (data) {
                    console.log(JSON.stringify(data));
                    if (data != null) {
                        if (connection) {
                            ClientAPI.updateUserSettings({sipProfile: sipProfile}, function (err, data) {
                                if (!err) {
                                    alert("Sip is successfully saved!");
                                    goBack();
                                } else {
                                    alert("Update user settings failed!");
                                    goBack();
                                }
                            });
                        }
                    } else {
                        alert("Sip registration failed!");
                    }
                }, function (data) {
                    alert(data);
                    self.removeSaveOption();
                });
            }, 'No', function () {
            });
        } else {
            window.localStorage.setItem("sipUsername", sipUsername);
            window.localStorage.setItem("sipPassword", sipPassword);
            window.localStorage.setItem("port", port);
            window.localStorage.setItem("proxy", proxy);
            window.localStorage.setItem("authUsername", authUsername);
            window.localStorage.setItem("sipDomain", sipDomain);
            $('#sipUserName').val(sipUsername);

            var sipProfile = {
                sipUsername: sipUsername,
                port: port,
                proxy: proxy,
                authUsername: authUsername,
                sipDomain: sipDomain
            };

            // sip login
            self.sipLogin(sipUsername, sipPassword, function (data) {
                console.log(JSON.stringify(data));
                if (data != null) {
                    if (connection) {
                        ClientAPI.updateUserSettings({sipProfile: sipProfile}, function (err, data) {
                            if (!err) {
                                alert("Sip is successfully saved!");
                                goBack();
                            } else {
                                alert("Update user settings failed!");
                                goBack();
                            }
                        });
                    }
                } else {
                    alert("Sip registration failed!");
                }
            }, function (data) {
                alert(data);
                self.removeSaveOption();
            });
        }
    };

    self.removeSaveOption = function () {
        $("#sip-settings .popup-menu-backdrop").hide();
        $("#sip-settings #sip-settings-popup").hide();
    };

    self.sipLogin = function (sipUsername, sipPassword, onSuccess, onError) {
        var userDataName = window.localStorage.getItem("userDataName");
        var sipDomain = window.localStorage.getItem("sipDomain");
        var authUsername = window.localStorage.getItem("authUsername");
        var proxy = window.localStorage.getItem("proxy");
        var port = window.localStorage.getItem("port");

        if (isNaN(sipUsername) || sipDomain == null || sipDomain == "") {
            onError("Data is not valid!");
        } else {
            window.localStorage.setItem("sipUsername", sipUsername);
            window.plugins.SipPlugin.closeConnection(function () {
                var arg = [sipUsername, sipDomain, sipPassword, userDataName, authUsername, proxy, port];
                window.plugins.SipPlugin.initialize(arg, function () {
                    var objectSip = {
                        user: sipUsername,
                        pass: sipPassword,
                        domain: sipDomain,
                        expires: SIP_EXPIRE_TIME
                    };

                    self.sipRegistration(objectSip, onSuccess);
                }, function () {
                    console.log('SipPlugin Failure sip login');
                    onError("Login failed!");
                });
            }, function () {
                console.log("SIP CLOSE CONNECTION ERROR");
                onError("Close connection failed!");
            });
        }
    };

    self.sipRegistration = function (objectSip, callback) {
        if (connection) {
            ClientAPI.registerSip(objectSip, function () {
                var objUpdateSip = {
                    domain: window.localStorage.getItem('sipDomain'),
                    status: 1
                };
                ClientAPI.updateSipStatus(objUpdateSip, function (error, data) {
                    console.log("== updateSipStatus vraceni podatak ==");
                    console.log(JSON.stringify(data));
                    if (data) {
                        window.localStorage.setItem("sip_status", "on");
                        if (callback != null) {
                            callback(Messages.Sip.saveSipCompleted);
                        }
                    } else {
                        window.localStorage.setItem("sip_status", "off");
                        if (callback != null) {
                            callback();
                        }
                    }
                });
            });
        }
    };

    return self;
}(SipSettingsFunctions || {}));