$(document).on('pageinit', '#edit-task', function () {
    TaskEditFunctions.init();
});

$(document).on('pageshow', '#edit-task', function () {
    TaskEditFunctions.bindElementControls();
    TaskEditFunctions.setConnectionMode();
    //initiate richtext editor
    if (tinymce.editors.length > 0) {
        tinymce.editors = [];
    }
    tinymce.init({
        mode: "exact",
        selector: "#task-body",
        autoresize_min_height: 250,
        menubar: false,
        content_css: "styles/tinymce-custom.css",
        statusbar: false,
        skin: 'light',
        plugins: "autoresize",
        toolbar: [
            "bold italic underline bullist numlist outdent indent"
        ]
    });
});

$(document).on('pagehide', '#edit-task', function () {
    TaskEditFunctions.unbindElementControls();
});

var TaskEditFunctions = (function (TaskEditFunctions)
{
    var self = TaskEditFunctions;
    var taskObject = null;
    var taskId = null;
    var parentId = null;
    var parentType = null;

    var $appendDealTask = null,
            $taskCategoryId = null,
            $taskFrame = null,
            $dateTimePicker = null,
            $editTaskPublic = null,
            $taskBody = null,
            $btnSave = null,
            $btnCompleted = null,
            $btnNotification = null,
            $viewTaskTasksLink = null,
            $dealContactDataVisibleGroup = null,
            $viewTaskNotesAddLink = null,
            $taskNoteRefreshLink = null,
            $viewTaskNotesLink = null,
            $taskPriority = null,
            $taskContactIdGroup = null,
            $taskContactId = null,
            $autoCompleteResponsible = null,
            $autoCompleteOwner = null,
            $autoCompleteAppend = null,
            $autoCompleteAppendList = null,
            $dateTimePickerContainer = null,
            $taskDueStatus = null,
            $percentCompletedContainer = null;
    var visibilityObjectId = null;
    var visibilityObjectType = null;
    var $taskPercentage = null;
    var $taskPercentSlider = null;
    var taskEditing = false;
    var userOwnerId = null;

    self.init = function () {
        self.selectElement();
        self.initDateTimePicker();
        self.initTaskCategory();
        self.initAppendTo();

        PermissionController.init('#edit-task');
        AutoCompleteController.init('#edit-task');
        AutoCompleteController.addViewId('autocomplete-responsible', 'autocomplete-responsible-list', 'autocomplete-responsible');
        AutoCompleteController.addViewId('autocomplete-owner', 'autocomplete-owner-list', 'autocomplete-owner');
        AutoCompleteController.addViewId('autocomplete-append', 'autocomplete-append-list', 'autocomplete-append', function (data) {
            setTimeout(function () {
                var caseObject;
                visibilityObjectId = data.id;
                visibilityObjectType = data.objectType;
                if (visibilityObjectType == "Project") {
                    CaseController.findById(visibilityObjectId, function (data) {
                        if (data[0] == null) {
                            data = CaseController.encodeItem(data);
                            caseObject = data;
                        } else {
                            data = data[0].kase;
                            caseObject = data;
                        }
                        if (caseObject.status == "Closed") {
                            showActionDialog("Case is closed! Do you wish to proceed ?", "Yes", self.setACL, "No", function () {
                            });
                        } else {
                            self.setACL();
                        }
                    });
                } else {
                    showActionDialog("Take visibility of parent object?", "Yes", self.updateACL, "No", function () {
                    });
                }
            }, 2000);
        });
        $autoCompleteAppend.parent().parent().hide();

        //translate dropdown menus
        $taskPriority.i18n();
        $taskPriority.selectmenu('refresh');
        $appendDealTask.i18n();
        $appendDealTask.selectmenu('refresh');
        $taskFrame.i18n();
        $taskFrame.selectmenu('refresh');

        if (pageObject) {
            taskId = pageObject.id;
            if (taskId) {
                $("#task-current-status").show();
                $('#task-edit-title').show();
                taskEditing = true;
                $percentCompletedContainer.show();
                TaskController.findById(taskId, function(data){
                   if(connection){
                       taskObject = TaskController.encodeItem(data);
                   } else {
                       taskObject = data[0].task;
                   }
                    if (taskObject["recording-id"] && taskObject["recording-type"]) {
                        self.selectObjectFromDatabase(taskObject["recording-id"]["text"], taskObject["recording-type"], taskObject["recording-id"]["text"], taskObject["recording-type"]);
                    }
                    self.loadTask();
                    EffortService.loadEffort(taskObject.id.text, "Task", "edit-task", "edit-effort-link-table", true);
                    self.taskFrameEventChange();
                });
            } else {
                taskEditing = false;
                $("#task-current-status").hide();
                $('#task-add-title').show();
                $taskPercentage.text("0 %");
                $percentCompletedContainer.hide();
                self.setDefaultTime();
                parentId = pageObject.parentId;
                parentType = pageObject.parentType;
                var accessList = pageObject.accessList;
                if (accessList) {
                    taskObject = {"visible-to": accessList};
                    PermissionController.setData(taskObject['visible-to']);
                }
                if (parentId && parentType) {
                    $dealContactDataVisibleGroup.hide();
                    self.selectObjectFromDatabase(parentId, parentType, parentId, parentType);
                } else if (pageObject && pageObject.ownerId) {
                    userOwnerId = pageObject.ownerId;
                } else {
                    $dealContactDataVisibleGroup.show();
                }
                $btnCompleted.hide();
            }
        } else {
            taskEditing = false;
            $("#task-current-status").hide();
            $('#task-add-title').show();
            $percentCompletedContainer.hide();
            $taskPercentage.text("0 %");
            self.setDefaultTime();
        }
        pageObject = null;

        try {
            if (userOwnerId) {
                AutoCompleteController.setObject('responsible', userOwnerId);
                $autoCompleteResponsible.val(ALL_USERS[userOwnerId].name);
            } else {
                AutoCompleteController.setObject('responsible', window.localStorage.getItem('userId'));
                $autoCompleteResponsible.val(ALL_USERS[window.localStorage.getItem('userId')].name);
            }

        } catch (e) {
            console.log('Responsible:' + JSON.stringify(e));
        }

        try {
            AutoCompleteController.setObject('owner', window.localStorage.getItem('userId'));
            $autoCompleteOwner.val(ALL_USERS[window.localStorage.getItem('userId')].name);
        } catch (e) {
            console.log('Responsible:' + JSON.stringify(e));
        }

    };

    self.setACL = function () {
        setTimeout(function () {
            showActionDialog("Take visibility of parent object?", "Yes", self.updateACL, "No", function () {
            });
        }, 1000);
    };

    self.updateACL = function () {
        var aclObjectVisibility = [];
        var request = "";
        switch (visibilityObjectType) {
            case "Company":
                request = {'objectType': "Company", _id: new ObjectId(visibilityObjectId)};
                break;
            case "Person":
                request = {'objectType': "Person", _id: new ObjectId(visibilityObjectId)};
                break;
            case "Deal":
                request = {'objectType': "Deal", _id: new ObjectId(visibilityObjectId)};
                break;
            case "Project":
                request = {'objectType': "Project", _id: new ObjectId(visibilityObjectId)};
                break;
            case "Note":
                request = {'objectType': "Note", _id: new ObjectId(visibilityObjectId)};
                break;
            case "Task":
                request = {'objectType': "Task", _id: new ObjectId(visibilityObjectId)};
                break;
        }
        if (connection) {
            ClientAPI.find(request, null, function (err, data) {
                if (data && data.length > 0) {
                    aclObjectVisibility = data[0]['accessList'];
                    if (aclObjectVisibility.length > 0) {
                        if (!PermissionController.changeACL) {
                            PermissionController.changeACL = true;
                        }
                        PermissionController.setData(aclObjectVisibility);
                    }
                }
            });
        }
    };

    self.setDefaultTime = function () {
        $dateTimePickerContainer.hide();
        $taskFrame.val("Today").selectmenu("refresh");
        var now = new Date();
        var today = new Date(now.getUTCFullYear(), now.getUTCMonth(), now.getDate(), 23, 59, 0, 0);
        $dateTimePicker.val(UtilController.dateToStringMobiScroll(today));
    };

    self.setSpecificTime = function () {
        $taskFrame.val("Specific").selectmenu("refresh");
        $dateTimePickerContainer.show();
    };



    self.unbindElementControls = function () {
        taskObject = null;
        $(document).off('click', '#edit-task #back-link');
        $(document).off('click', '#edit-task #save-link');
        $(document).off('click', '#edit-task .task-status-uncompleted');
        $(document).off('change', '#edit-task #task-frame');
        $(document).off('change', '#edit-task #append-data-deal-task');
        $(document).off('click', '#edit-task #edit-task-more-link');
        $('#edit-task .popup-menu-backdrop').off('click');
        $("#edit-task #task-percent").off('change');
        $(document).off('click', '#edit-task .additional-options');
        $(document).off('click', '#edit-task #add-effort-link');
        $(document).off('click', '#edit-task .remove-effort-link');
        taskObject = null;
        parentId = null;
        parentType = null;
        userOwnerId = null;
    };

    self.bindElementControls = function () {
        $(document).on('click', '#edit-task #back-link', self.backLinkEvent);
        $(document).on('click', '#edit-task #save-link', self.saveLinkEvent);
        $(document).on('click', '#edit-task .task-status-uncompleted', self.setStatusForTask);
        $(document).on('change', '#edit-task #task-frame', self.taskFrameEventChange);
        $(document).on('change', '#edit-task #append-data-deal-task', self.taskAppendToDealTaskEventChange);
        $(document).on('click', '#edit-task #edit-task-more-link', self.openEditTaskPopup);
        $('#edit-task .popup-menu-backdrop').on('click', function () {
            $('#edit-task .advanced-menu').hide();
            $(this).hide();
        });
        $("#edit-task #task-percent").on('change', function () {
            var percent = $(this).val();
            $("#edit-task #task-percent-value").text(percent + " %");
        });

        $(document).on('click', '#edit-task .additional-options', self.saveLinkEvent);
        $(document).on('click', '#edit-task #add-effort-link', self.addTaskEffort);
        $(document).on('click', '#edit-task .remove-effort-link', self.removeEffortLink);
    };

    self.openEditTaskPopup = function () {
        $('#edit-task .popup-menu-backdrop').toggle();
        $('#edit-task .advanced-menu').css('z-index', '20000');
        $('#edit-task .advanced-menu').slideToggle('4000', 'swing');
    };

    self.selectElement = function () {
        $appendDealTask = $("#append-data-deal-task", '#edit-task');
        $taskCategoryId = $('#task-category-id', '#edit-task');
        $taskFrame = $('#task-frame', '#edit-task');
        $taskContactIdGroup = $('#task-contact-id-group', '#edit-task');
        $dateTimePicker = $('#date-time-picker', '#edit-task');
        $editTaskPublic = $("#edit-task-public", '#edit-task');
        $taskBody = $("#task-body", '#edit-task');
        $btnSave = $('#save-link', '#edit-task');
        $btnCompleted = $('#task-current-status', '#edit-task');
        $btnNotification = $('#task-notification-link', '#edit-task');
        $dealContactDataVisibleGroup = $('#deal-contact-data-visible-group', '#edit-task');
        $taskContactId = $('#task-contact-id', '#edit-task');
        $viewTaskTasksLink = $('#view-task-tasks-link', '#edit-task');
        $viewTaskNotesAddLink = $('#view-task-notes-add-link', '#edit-task');
        $taskNoteRefreshLink = $('#task-note-refresh-link', '#edit-task');
        $viewTaskNotesLink = $('#view-task-notes-link', '#edit-task');
        $taskPriority = $('#task-priority', '#edit-task');
        $autoCompleteResponsible = $('#autocomplete-responsible', '#edit-task');
        $autoCompleteOwner = $('#autocomplete-owner', '#edit-task');
        $autoCompleteAppend = $('#autocomplete-append', '#edit-task');
        $autoCompleteAppendList = $('#autocomplete-append-list', '#edit-task');
        $dateTimePickerContainer = $('#date-time-picker-container', '#edit-task');
        $taskDueStatus = $("#task-due-status", '#edit-task');
        $taskPercentage = $("#edit-task #task-percent-value");
        $taskPercentSlider = $("#edit-task #task-percent");
        $percentCompletedContainer = $("#edit-task .completion-percent-container");
    };

    self.initDateTimePicker = function () {
        if (navigator.userAgent.match(/(iPad|iPhone)/)) {
            $dateTimePicker.attr('type', 'datetime-local');
        } else {
            $('#disable-date-time').removeClass("disable-date-time-class");
            $dateTimePicker.mobiscroll().date({
                theme: 'android',
                display: 'modal',
                mode: 'scroller',
                dateOrder: 'dd MM yy',
                dateFormat: 'dd.mm.yy',
                timeFormat: 'HH:ii',
                timeWheels: 'HH:ii',
                maxDate: new Date(2025, 1, 1),
                minDate: new Date(2010, 1, 1)
            });
        }
    };

    self.initTaskCategory = function () {
        var html = "";
        var key;
        $(taskCategories).each(function (index, value) {
            switch (value.name) {
                case "Call":
                    key = "task.call";
                    break;
                case "Document":
                    key = "task.document";
                    break;
                case "Demo":
                    key = "task.demo";
                    break;
                case "Outbound call":
                    key = "task.outboundCall";
                    break;
                case "Meeting":
                    key = "task.meeting";
                    break;
                case "Analysis":
                    key = "task.analysis";
                    break;
                case "Review":
                    key = "task.review";
                    break;
                case "Approval":
                    key = "task.approval";
                    break;
                default:
                key = null;
                break;
            }
            if (key) {
              html += '<option value="' + value.id.text + '" data-i18n="' + key + '">' + value.name + '</option>';
            } else {
              html += '<option value="' + value.id.text + '">' + value.name + '</option>';
            }

        });
        $taskCategoryId.empty().append(html);
        $taskCategoryId.i18n();
        $taskCategoryId.selectmenu('refresh');
    };

    self.initAppendTo = function () {
        if (!connection) {
            $("option[value$='Note']", '#append-data-deal-task').remove();
        }
    };

    self.backLinkEvent = function () {
        if (tinymce.activeEditor.getContent()) {
            if (taskObject == null) {
                showActionDialog("There is unsaved data left, would you like to save it?", "Yes", function () {
                    self.saveTask();
                }, "No", function () {
                    goBack();
                });
            } else {
                if (htmlStrip(taskObject.body) != htmlStrip(tinymce.activeEditor.getContent()) || taskObject.name != $("#edit-task #task-title").val()) {
                    showActionDialog("There is unsaved data left, would you like to save it?", "Yes", function () {
                        self.saveTask();
                    }, "No", function () {
                        goBack();
                    });
                } else {
                    goBack();
                }
            }
        } else {
            goBack();
        }
    };

    self.loadTask = function () {
        try {
            if (taskObject) {
                setTimeout(function () {
                    tinymce.activeEditor.setContent(taskObject.body);
                }, 300);
                if (taskObject != null && taskObject['due-at'] != null && taskObject['due-at'].text != null) {
                    var taskDate = new Date(taskObject['due-at'].text);
                    taskDate.setHours(0, 0, 0, 0);
                    var now = new Date();
                    now.setHours(0, 0, 0, 0);
                    var tomorrow = new Date(now.getTime() + (24 * 60 * 60 * 1000));
                    tomorrow.setHours(0, 0, 0, 0);
                    var thisWeek = new Date(now.getTime() + (6 - now.getDay()) * 24 * 60 * 60 * 1000);
                    thisWeek.setHours(0, 0, 0, 0);
                    var nextWeek = new Date(now.getTime() + (24 * 60 * 60 * 1000 * (13 - now.getDay())));
                    nextWeek.setHours(0, 0, 0, 0);
                    if (taskDate.valueOf() == now.valueOf()) {
                        $taskFrame.val("Today").selectmenu("refresh");
                    } else if (taskDate.valueOf() == tomorrow.valueOf()) {
                        $taskFrame.val("Tomorrow").selectmenu("refresh");
                    } else if (taskDate.valueOf() == thisWeek.valueOf()) {
                        $taskFrame.val("ThisWeek").selectmenu("refresh");
                    } else if (taskDate.valueOf() == nextWeek.valueOf()) {
                        $taskFrame.val("NextWeek").selectmenu("refresh");
                    } else {
                        $taskFrame.val("Specific").selectmenu("refresh");
                    }
                    var selectDate = "";
                    if (navigator.userAgent.match(/(iPad|iPhone)/)) {
                        selectDate = taskObject['due-at'].text.replace('Z', '');
                    } else {
                        var date = taskObject['due-at'].text.split('T')[0];
                        selectDate = date.split("-")[2] + '.' + date.split("-")[1] + '.' + date.split("-")[0];
                    }

                    $dateTimePicker.val(selectDate);
                } else {
                    $taskFrame.val("Later").selectmenu("refresh");
                }

                try {
                    AutoCompleteController.setObject('responsible', taskObject['responsible-id'].text);
                    $autoCompleteResponsible.val(ALL_USERS[taskObject['responsible-id'].text].name);
                } catch (e) {
                    console.log('Responsible:' + JSON.stringify(e));
                }

                try {
                    AutoCompleteController.setObject('owner', taskObject['owner-id'].text);
                    $autoCompleteOwner.val(ALL_USERS[taskObject['owner-id'].text].name);
                } catch (e) {
                    console.log('Responsible:' + JSON.stringify(e));
                }

                var taskCategoryIdValue = taskObject['category-id'] != null && taskObject['category-id'].text != null ? taskObject['category-id'].text : "None";
                $taskCategoryId.val(taskCategoryIdValue);
                $taskCategoryId.i18n();
                $taskCategoryId.selectmenu('refresh');
                $taskPriority.val(taskObject['priority']);
                $taskPriority.i18n();
                $taskPriority.selectmenu('refresh');
                PermissionController.setData(taskObject['visible-to']);

                if (typeof taskObject['subject-type'] != "undefined" && taskObject['subject-id'] != null && taskObject['subject-id'].text != null) {
                    self.selectObjectFromDatabase(taskObject['subject-id'].text, taskObject['subject-type'], taskObject['recording-id'], taskObject['recording-type']);
                }
                if (taskObject['percent-completed']) {
                    $taskPercentSlider.val(taskObject['percent-completed']);
                    $taskPercentage.text(taskObject['percent-completed'] + " %");
                } else {
                    $taskPercentSlider.val(0);
                    $taskPercentage.text("0 %");
                }

                if (taskObject.name) {
                    $("#edit-task #task-title").val(taskObject.name);
                } else {
                    $("#edit-task #task-title").val(taskObject.body);
                }

            }
        } catch (e) {
            console.log(JSON.stringify(e));
        }
    };

    self.selectObjectFromDatabase = function (idSubject, typeSubject, idRecording, typeRecording) {
        var id, type;
        if (idSubject != idRecording) {
            id = idRecording;
            type = typeRecording;
        } else {
            id = idSubject;
            type = typeSubject;
        }
        var tableName, query, queryData = [getTextTypeObject(id, 'integer')];
        if (type.toLowerCase() == "Note".toLowerCase()) {
            $appendDealTask.val('Note').selectmenu('refresh');
            $autoCompleteAppend.attr('data-objecttype', $appendDealTask.val());
            $autoCompleteAppend.parent().parent().show();
            if (connection) {
                NoteController.findById(id, function (data) {
                    if (data && data.length > 0) {
                        AutoCompleteController.setObject("Note", data[0]._id);
                        $autoCompleteAppend.val(data[0].note);
                    }
                });
            }
        } else {
            if ("deal" == type.toLowerCase()) {
                $appendDealTask.val('Deal').selectmenu('refresh');
                $autoCompleteAppend.parent().parent().show();
                query = 'SELECT * FROM deal where id =?';
                tableName = "deal";
            } else if ("kase" == type.toLowerCase() || "project" == type.toLowerCase()) {
                $appendDealTask.val('Project').selectmenu('refresh');
                $autoCompleteAppend.parent().parent().show();
                query = 'SELECT * FROM kase where id =?';
                tableName = "kase";
            } else if ("company" == type.toLowerCase()) {
                $appendDealTask.val('Company').selectmenu('refresh');
                $autoCompleteAppend.parent().parent().show();
                query = 'SELECT * FROM company where id =?';
                tableName = "company";
            } else if ("people" == type.toLowerCase() || "person" == type.toLowerCase()) {
                $appendDealTask.val('Person').selectmenu('refresh');
                $autoCompleteAppend.parent().parent().show();
                query = 'SELECT * FROM people where id =?';
                tableName = "people";
            } else if ("task" == type.toLowerCase()) {
                $appendDealTask.val('Task').selectmenu('refresh');
                $autoCompleteAppend.parent().parent().show();
                query = 'SELECT * FROM task where id =?';
                tableName = "task";
            }

            self.taskAppendToDealTaskEventChange();
            db.selectByQuery(query, queryData, tableName, function (data) {
                if (data != null && data[0] != null) {
                    AutoCompleteController.setObject($autoCompleteAppend.attr('data-objecttype'), data[0].id.text);
                    if ("people" == type.toLowerCase() || "person" == type.toLowerCase()) {
                        $autoCompleteAppend.val(data[0]['first-name'] + ' ' + data[0]['last-name']);
                    } else {
                        if (type.toLowerCase() == "task") {
                            if (data[0].name) {
                                $autoCompleteAppend.val(data[0].name);
                            } else {
                                $autoCompleteAppend.val(data[0].body);
                            }
                        } else {
                            $autoCompleteAppend.val(data[0].name);
                        }
                    }
                }
            });
        }
    };

    self.saveLinkEvent = function () {
        $("#edit-task .advanced-menu").hide();
        $("#edit-task .popup-menu-backdrop").hide();
        $(this).addClass('not-clickable');
        if (window.localStorage.getItem("enableSave") == "true") {
            showActionDialog(Messages.Task.saveTaskMessage, "Yes", self.saveTask, "No", function () {
            });
        } else {
            self.saveTask();
        }
    };

    self.saveTask = function () {
        playSound("www/libs/blop.mp3");
        var body = $.trim(tinymce.activeEditor.getContent());
        var frame = $taskFrame.val();
        var responseId = AutoCompleteController.getObject('responsible');
        var ownerId = AutoCompleteController.getObject('owner');
        var priority = $taskPriority.val();
        var categoryId = $taskCategoryId.val();
        var subjectId = null;
        var dueAt = null;
        var status = null;
        var visibleTo = PermissionController.permission;
        var taskPercent = $taskPercentSlider.val();
        taskPercent = parseInt(taskPercent);
        var taskTitle = $("#edit-task #task-title").val();

        //validate if task title is entered
        if (taskTitle == "undefined" || taskTitle == "") {
            alert("Add task title!");
            //if save button is disabled enable it again
            if ($('#edit-task #save-link').hasClass('not-clickable')) {
                $('#edit-task #save-link').removeClass('not-clickable');
            }
            if ($('#edit-task .additional-options').hasClass('not-clickable')) {
                $('#edit-task .additional-options').removeClass('not-clickable');
            }
            return;
        }

        if (parentId && parentType) {
            subjectId = parentId;
        } else {
            subjectId = AutoCompleteController.getObject($appendDealTask.val());
        }

        if (ownerId == window.localStorage.getItem("userId")) {
            status = "Created";
        } else {
            status = "Assigned";
        }

        if ($taskFrame.val() != "Later") {
            if (navigator.userAgent.match(/(iPad|iPhone)/)) {
                dueAt = new Date($dateTimePicker.val());
            } else {
                dueAt = new Date(parseInt($dateTimePicker.val().split(".")[2].split(" ")[0]),
                        parseInt($dateTimePicker.val().split(".")[1] - 1),
                        parseInt($dateTimePicker.val().split(".")[0]), 23, 59, 0);
            }
        }

        if (taskObject && taskObject.id) {
            if (!PermissionController.changeACL) {
                visibleTo = null;
            }

            TaskController.update(taskObject.id.text, body, categoryId, dueAt, ownerId, responseId, priority, frame, status, subjectId, visibleTo, taskTitle,
            taskPercent, function (data) {
                if (navigationParams.length >= 2) {
                    navigationParams[navigationParams.length - 2].object = data[0];
                }
                if (taskObject && window.localStorage.getItem("notificationStatus") == "true") {
                    var oldTaskTitle = taskObject.body;
                    var oldStartDate = new Date(taskObject['due-at'].text);
                    oldStartDate.setHours(8, 0, 0, 0);
                    var oldEndDate = new Date(taskObject['due-at'].text);
                    oldEndDate.setHours(8, 15, 0, 0);
                    updateCalendarEvent(oldTaskTitle, oldStartDate, oldEndDate, data[0]);
                }
                // save effort for related object
                EffortService.createOrUpdateEffort(data[0].id.text, "Task", data[0]['created-at'].text, "edit-effort-link-table", function(err, data){
                    goBack();
                });
            });
        } else {
            TaskController.save(body, categoryId, dueAt, ownerId, responseId, priority, frame, status, subjectId, visibleTo,
            taskTitle, taskPercent, function (data) {
                if (ownerId != null) {
                    if (ownerId != window.localStorage.getItem('userId')) {
                        SipMessageSettingFunctions.sendInfoMessage(ownerId, {
                            taskid: data.id.text,
                            body: data.body
                        }, function () {
                        });
                    }
                    //insert task in calendar;
                    if (window.localStorage.getItem("notificationStatus") == "true") {
                        createCalendarEvent(data);
                    }

                    //save effort for related object
                    EffortService.createOrUpdateEffort(data.id.text, "Task", data['created-at'].text, "edit-effort-link-table", function(err, data){
                        goBack();
                    });
                }
            });
        }
    };

    self.taskFrameEventChange = function () {
        var now = new Date();
        var today = new Date(now.getUTCFullYear(), now.getUTCMonth(), now.getDate(), 23, 59, 0, 0);
        if ($taskFrame.val() == 'Specific') {
            $dateTimePickerContainer.show();
            $dateTimePicker.show();
        } else {
            $dateTimePickerContainer.hide();
            // $dateTimePicker.hide();
            if ($taskFrame.val() == "Today") {
                $dateTimePicker.val(UtilController.dateToStringMobiScroll(today));
            } else if ($taskFrame.val() == "Tomorrow") {
                var tomorrow = new Date(now.getTime() + (24 * 60 * 60 * 1000));
                tomorrow = new Date(tomorrow.getUTCFullYear(), tomorrow.getUTCMonth(), tomorrow.getDate(), 23, 59, 0, 0);
                $dateTimePicker.val(UtilController.dateToStringMobiScroll(tomorrow));
            } else if ($taskFrame.val() == "ThisWeek") {
                var thisWeek = new Date(now.getTime() + (24 * 60 * 60 * 1000 * (6 - now.getDay())));
                thisWeek = new Date(thisWeek.getUTCFullYear(), thisWeek.getUTCMonth(), thisWeek.getDate(), 23, 59, 0, 0);
                $dateTimePicker.val(UtilController.dateToStringMobiScroll(thisWeek));
            } else if ($taskFrame.val() == "NextWeek") {
                var nextWeek = new Date(now.getTime() + (24 * 60 * 60 * 1000 * (13 - now.getDay())));
                nextWeek = new Date(nextWeek.getUTCFullYear(), nextWeek.getUTCMonth(), nextWeek.getDate(), 23, 59, 0, 0);
                $dateTimePicker.val(UtilController.dateToStringMobiScroll(nextWeek));
            } else if ($taskFrame.val() == "Later") {
                $dateTimePicker.val('');
            }
        }
    };

    self.taskAppendToDealTaskEventChange = function () {
        $autoCompleteAppend.val('');
        $autoCompleteAppendList.empty();
        AutoCompleteController.setObject($autoCompleteAppend.attr('data-objecttype'), null);
        if ($appendDealTask.val() != "none") {
            AutoCompleteController.setObject('Company', null);
            AutoCompleteController.setObject('Person', null);
            AutoCompleteController.setObject('Deal', null);
            AutoCompleteController.setObject('Project', null);
            AutoCompleteController.setObject('Note', null);
            AutoCompleteController.setObject('Task', null);
            $autoCompleteAppend.attr('data-objecttype', $appendDealTask.val());
            $autoCompleteAppend.parent().parent().show();
        } else {
            $autoCompleteAppend.parent().parent().hide();
        }
    };

    self.setStatusForTask = function () {
        $("#edit-task-popup").hide();
        $("#edit-task .popup-menu-backdrop").hide();
        gotoPage({url: "changeStatus", param: {parentId: taskId, parentType: "Task"}}, {url: "editTask", param: {id: taskId}});
    };

    self.setConnectionMode = function () {
        if (!connection) {
            $('#' + $.mobile.activePage.attr('id') + ' #task-current-status').hide();
        }
    };

    self.addTaskEffort = function () {
        EffortService.drawEffortTable('edit-task', 'edit-effort-link-table');
        initDateTimePicker('#edit-task .date-time-picker');

    };

    self.removeEffortLink = function (evt) {
        $(evt.currentTarget).parent().parent().remove();
    };

    return self;
}(TaskEditFunctions || {}));
